<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: NewsController.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Default
 * @copyright  Copyright (c) 2012 VEC
 */

class NewsController extends Zend_Controller_Action
{
    
    protected $_recordsPerPage = 10;
    
    public function indexAction ()
    {
        //Input
        $request = $this->getRequest();
        $startIndex = intval($request->getUserParam('startIndex'));

        if($startIndex < 0) {
            throw new Giscuit_Exception('Invalid startIndex value (' . $startIndex . ')');    
        }
        
        $newsTable = new Giscuit_Db_Table_News();

        $rows = $newsTable->getItems(array(
            'withDesc' => true , 
            'withTitle' => true , 
            'whereActive' => 't' , 
            'limit' => $this->_recordsPerPage , 
            'limitStart' => $startIndex
        ));

        $this->view->rows = $rows;
        $this->view->startIndex = $startIndex;
        $this->view->recordsPerPage = $this->_recordsPerPage;
        $this->view->totalRecords = $newsTable->count(array(
            'onlyActive' => true
        ));
    }

    public function detailedAction ()
    {
        $config = Zend_Registry::get('config');
        
        //Input
        $request = $this->getRequest();
        $id = intval($request->getUserParam('id'));
        
        $newsTable = new Giscuit_Db_Table_News();
        
        //Get records
        $rows = $newsTable->getItems(array(
            'withText' => true , 
            'withDesc' => true , 
            'withTags' => true , 
            'withTitle' => true , 
            'whereId' => $id , 
            'whereActive' => 't'
        ));
        
        if(! count($rows)) {
            throw new Giscuit_Exception('Invalid article id (' . $id . ')');
        }
        $row = $rows[0];
                
        //Increment views
        $newsTable->incrementViews($row);
        
        //Comments
        $commTable = new Giscuit_Db_Table_Comm();
        
        $comments = $commTable->getItems($id);
        
        $this->view->comments = $comments;
        $this->view->commTotalRecords = $commTable->count(array(
            'part' => $row->id
        ));
        
        $this->view->commForm = $this->getCommentForm($id);
        
        $this->view->row = $row;
    }
    
    public function submitcommentAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $commTable = new Giscuit_Db_Table_Comm();
        
        $newsId = intval($request->getParam('newsId'));
        $form = $this->getCommentForm($newsId);
        $form->populate($request->getParams());

        //Invalid input data
        $comment = substr($form->getValue('text'), 0, 500);
        if ($comment != false) {
            $commTable->addItem($newsId, $form);
            
            $result['newsId'] = $newsId;
        } else {
            
        }
        
        $this->_helper->json($result);
    }
    
    public function deletecommentAction()
    {
        $result['replyCode'] = 200;

        $request = $this->getRequest();
        $commTable = new Giscuit_Db_Table_Comm();

        $id = intval($request->getParam('id'));
        $commTable->delete('id = ' . $id);
        
        $this->_helper->json($result);
    }
    
    public function getCommentForm($newsId)
    {
        $form = new Giscuit_Form_Comm();
        
        $action = $this->_helper->url->url(array(
            'language' => Zend_Registry::get('Zend_Translate')->getLocale() ,
            'module' => 'default' ,
            'controller' => 'news' ,
            'action' => 'submitcomment' ,
                'newsId' => $newsId
        ), null, true);
        $form->setAction($action);
        
        return $form;
    }

}

