<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: SettingsController.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Default
 * @copyright  Copyright (c) 2012 VEC
 */

class SettingsController extends Zend_Controller_Action
{

    public function indexAction ()
    {
        $result['replyCode'] = 200;
        
        $form = $this->getForm();
        $translate = Zend_Registry::get('Zend_Translate');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if(! $this->_request->isPost()) {
            
            //Get user data
            $users = new Giscuit_Db_Table_Users();
            $userData = $users->getItems(array(
                'whereId' => $userId
            ))->current()->toArray();
            
            //Filter
            $charToBoolHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('charToBool');
            $userData['snap'] = $charToBoolHelper->direct($userData['snap']);
            $userData['tooltip'] = $charToBoolHelper->direct($userData['tooltip']);
            $userData['giscuit_language'] = $userData['language'];
            
            //Populate
            $form->populate($userData);
            
            $this->view->form = $form;
            $this->view->email = $userData['email'];
        } else {
            //Update cache
            Zend_Registry::get('settings')->offlineTimestamp = time();
                        
            $form->populate($this->_request->getParams());
            
            if ($form->isValid($this->_request->getParams())) {
                
                $users = new Giscuit_Db_Table_Users();
                $users->editItem($userId, $form);
                
                $result['result'] = $translate->_('changesSaved');
            } else {
                $result['form'] = (string) $form;
            }
            
            $this->_helper->json($result);
        }
    }

    public function changelangAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $config = Zend_Registry::get('config');
        
        $jqm = $this->_request->getParam('jqm', false);
        $key = $this->_request->getParam('key');
        
        $languages = array_keys(Zend_Registry::get('config')->language->toArray());
        if (! in_array($key, $languages)) {
            return;
        }
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if (! empty($userId)) {
            $users = new Giscuit_Db_Table_Users();
            $users->editLanguage($userId, $key);
        }
        
        if(! $jqm) {
            $route = array_merge( 
                $config->defaultRoute->toArray(),
                array (
                    'language' => $key 
                )
            );
        } else {
            $route = array(
                'module' => 'mobile' ,
                'controller' => 'menu' ,
                'language' => $key
            );
        }
        
        $redirector = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
        $redirector->gotoUrl($this->_helper->url->url($route, null, true));
    }
    
    public function confirmemailAction ()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $success = $error = '';
        $data = array();
        
        //
        $account = $this->_request->account;
        $emailAddressValidator = new Zend_Validate_EmailAddress();
        if (! $emailAddressValidator->isValid($account)) {
            throw new Giscuit_Exception($translate->_('emailAddressInvalidFormat'));
        }
        
        //
        $secret = $this->_request->secret;
        $htmlSpecialCharsFilter = new Giscuit_Filter_HtmlSpecialChars();
        $secret = $htmlSpecialCharsFilter->filter($secret);
        
        $usersTable = new Giscuit_Db_Table_Users();
        $result = $usersTable->getItems(array(
            'whereAccount' => $account
        ));
        
        if(! count($result)) {
            $error = $translate->_('invalidAccount');
        }  else {
            $result = $result->current();
            
            if($secret != $result->email_secret) {
                $error = $translate->_('invalidSecret');
            }
                        
            if(! $error) {
                //
                if(! $result->lastlogin_date) {
                    if($config->signup->activate == 2) {
                        $data['active'] = 't'; 
                    
                        $success = $translate->_('accountWasActivated');
                    } else {
                        $success = $translate->_('emailWasConfirmed');
                    }
                }
                    
                //
                $to = $this->_request->to;
                if($to) {
                    if (! $emailAddressValidator->isValid($to)) {
                        throw new Giscuit_Exception($translate->_('emailAddressInvalidFormat'));
                    }
                    $data['email'] = $to;
                    
                    $success = $translate->_('emailChanged');
                }
                
                $data['email_active'] = 't';
                $data['email_secret'] = md5(uniqid(rand(), true));
 
                $where = $usersTable->getAdapter()->quoteInto('id = ?', $result->id);
                $usersTable->update($data, $where);

                
            }
        }
        
        $this->view->error = $error;
        $this->view->success = $success;
    }
    
    public function changeemailAction()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $result['replyCode'] = 200;
        
        $usersTable = new Giscuit_Db_Table_Users();
        
        $email = $this->_request->email;
        $emailAddressValidator = new Zend_Validate_EmailAddress();
        if (! $emailAddressValidator->isValid($email)) {
            $result['result'] = '<span class="errors">' . $translate->_('emailAddressInvalidFormat') . '</span>'; 
            
            $this->_helper->json($result);
            return;
        }
        
        //
        $secret = $usersTable->find($userId)->current()->email_secret;
                    
        //Mail
        $mailView = new Zend_View();
        $mailView->host = $_SERVER['HTTP_HOST'];
        $mailView->email = $email;
        $mailView->secret = $secret;
                
        $mailView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'default' . DIR_SEP . 
            'views' . DIR_SEP . 'scripts' . DIR_SEP . 'settings');
        
           $mailBody = $mailView->render('changeemail_mail' . GT_SCRIPT_EXT);
            
           Giscuit_Mail::batch(array(
            array(
                   'to' => $email,
                   'subject' => $translate->_('confirmEmailChangeTitle') ,
                   'text' => $mailBody
            )
        ));
        
        $result['result'] = $translate->_('emailConfirm');
        
        $this->_helper->json($result);
    }

    public function getForm ()
    {
        $form = new Giscuit_Form_Settings();
        
        $action = $this->_helper->url->url(array(
            'language' => Zend_Registry::get('Zend_Translate')->getLocale() ,
            'module' => 'default' ,
            'controller' => 'settings' ,
            'action' => 'index' 
        ), null, true);
        $form->setAction($action);
        
        return $form;
    }

}