<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id$
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Mobile
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see EditgeomController
 */
require_once GT_MODULES_PATH . DIR_SEP . 'default' . DIR_SEP . 'controllers' . DIR_SEP . 'EditgeomController.php';

class Mobile_EditgeomController extends EditgeomController
{
    public function init ()
    {
        if($this->getRequest()->getActionName() != 'bulk') {
              parent::init();
        }
    }
    
    public function getForm($class) {
        $formName = 'Giscuit_Form_Layer_' . $class;
        
        $configPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $class . '.xml';
        if (file_exists($configPath)) {
            $configContent = new Zend_Config_Xml($configPath);
            $form = new $formName($configContent);
        } else {
            $form = new $formName();
        }
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtEditattrForm'
        ));
        
        $form->applyJqmFixes(); 
        
        return $form;
    } 
    
    public function bulkAction ()
    {
        $translate = Zend_Registry::get('Zend_Translate');

        $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
        
        //Id
        $id = intval($this->_request->getParam('id'));
        
        //Data
        $data = Zend_Json::decode($this->_request->getParam('data'));
        if(empty($data)) {
            $this->_helper->json($result);
        }
        
        //Overwrite
        $overwrite = $this->_request->getParam('overwrite', '');
        $overwrite = explode(',', $overwrite);
        $overwrite = array_map('intval', $overwrite);
        
        //Skip
        $skip = $this->_request->getParam('skip', '');
        $skip = explode(',', $skip);
        $skip = array_map('intval', $skip);
        
        $errors = $recordsGidList = $changedDates = array();
        
        $result['replyCode'] = 200;
        $result['createRecords'] = array();
        $result['updateRecords'] = array();
        $result['deleteRecords'] = array();
        $result['layerId'] = $id;
        
        
        //
        $reservedFields = array(
            'hgid' ,
            'h_id' ,
            'hstatus' ,
            'changed_date' ,
            'changed_date_timestamp' ,
             'created_by' ,
            'verified_by'
        );
        
        $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
        
        $validateGeometry = new Giscuit_Validate_Geometry();
        
        $mapOptions = array(
            'orderDirection' => 'asc' , 
            'whereActive' => 't' , 
            'withLayerSettings' => true , 
            'withAcl' => true , 
        );
        
        //
        $layersTable = new Giscuit_Db_Table_Layers();
        $layersSettings = $layersTable->getItems(array_merge(array(
            'whereId' => $id
        ), $mapOptions));
        
        if(empty($layersSettings)) {
            throw new Giscuit_Db_Table_Exception('Invalid layer id');
        }
        
        //FIXME: Check if can be edited
        $layerSettings = $layersSettings[$id];
        
        //Class
        $className = 'Giscuit_Map_Layer_' . $layerSettings['class'];
        if (! @class_exists($className) 
            || ! is_subclass_of($className, 'Giscuit_Map_Layer_Vector')) {
                
            throw new Giscuit_Exception("Invalid layer class ({$className})");
        }
                
        //Layer
        $layerClassName = 'Giscuit_Map_Layer_' . $layerSettings['class'];
        $map = new Giscuit_Map_Map(GT_CONFIG_PATH . DIR_SEP . 'default.map');
        $map->setParamsFromConfig();
        $layer = new $layerClassName($map, $layerSettings);
        
        //Table
        $table = $layer->getTable();
        $tableName = $table->info('name');
        $pkeyName = $table->getPrimaryKey(array(
            'name' => true
        ));
        
        //Form
        $formName = 'Giscuit_Form_Layer_' . $layerSettings['class'];
        $configPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $layerSettings['class'] . '.xml';
        if (file_exists($configPath)) {
            $configContent = new Zend_Config_Xml($configPath);
            $form = new $formName($configContent);
        } else {
            $form = NULL;
        }
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtEditattrForm'
        ));
        
        //History Table
        if($table instanceof Giscuit_Db_Table_Layer_History) {
            $historyTableName = 'Giscuit_Db_Table_Layer_' . $layerSettings['class'];
            $historyTable = new $historyTableName(array(
                'name' => $layerSettings['table'] ,
                'withHistory' => true ,
            ));
        }
        
        //History changed dates
        if($table instanceof Giscuit_Db_Table_Layer_History) {
            foreach ($data as $params) {
                if($params['hstatus'] != 'C' && isset($params[$pkeyName])) {
                    $recordsGidList[] = intval($params[$pkeyName]);
                }
            }
            
            if(! empty($recordsGidList)) {
                $dbSelect = $historyTable->getSelect('all_table', array(
                    'whereStm' => "\"$tableName\".\"$pkeyName\" IN (" . implode(',', $recordsGidList) . ')' ,
                    'fields' => array(
                        'changed_date_timestamp' => new Zend_Db_Expr("extract(epoch from globhist.activate_date)")
                    ) ,
                    'geometryAsGeoJSON' => true
                ));
            
                $dbRecords = $historyTable->fetchAll($dbSelect)->toArray();
                
                foreach ($dbRecords as $dbKey => $dbRecord) {
                    //
                    $changedDates[$dbRecord[$pkeyName]] = intval($dbRecord['changed_date_timestamp']);
                    
                    //
                    foreach ($dbRecord as $key => $value) {
                        if(in_array($key, $reservedFields)) {
                            unset($dbRecord[$key]);
                        }
                    }
    
                    //
                    $dbRecords[$dbRecord[$pkeyName]] = $dbRecord;
                    unset($dbRecords[$dbKey]);
                }
            }
            
        }
        
        foreach ($data as $params) {
            //FIXME:
            if($params['hstatus'] != 'C' && in_array($params['__fid'], $skip)) {
                continue;
            }
            
            if($params['hstatus'] != 'C' && ! isset($params[$pkeyName])) {
                throw new Giscuit_Exception('Empty gid for a feature with status U or D');
            }
            
            $form->populate($params);
            
            $fid = $params['__fid'];
            if($params['hstatus'] != 'C') {
                $params[$pkeyName] = intval($params[$pkeyName]);
            }
            
            //Validate geometry
            if(! empty($params['the_geom'])) {
                if (! $validateGeometry->isValid($params['the_geom'])) {
                    foreach ($validateGeometry->getMessages() as $key => $value) {
                        $errors[$fid][][] = array(
                           $key,
                           'geometry'
                        );
                    }
                    $params['the_geom'] = NULL;
                }
            } else {
                $errors[$fid][][] = array(
                       $translate->_('invalidGeometry'),
                       'geometry'
                );
                $params['the_geom'] = NULL;
            }
                        
            if($table instanceof Giscuit_Db_Table_Layer_History 
                && $params['hstatus'] != 'C'
                && ! in_array($params['__fid'], $overwrite)
                && ! in_array($params['__fid'], $skip)) {
                    
                //Check for delete conflicts in features scheduled for update
                if(! isset($changedDates[$params[$pkeyName]]) && $params['hstatus'] == 'U') {
                    $errors[$fid][][] = array(
                           $translate->_('deleteConflict'),
                           'conflict'
                    );    
                }
                
                //Check for update conflicts
                if(isset($changedDates[$params[$pkeyName]]) && $changedDates[$params[$pkeyName]] > intval($params['__hdate'])) {
                    $errors[$fid][][] = array(
                           $translate->_('updateConflict'),
                           'conflict'
                    );
                }
            }
            
            //Remove file elements
            foreach ($form->getElements() as $element) {
                if($element instanceof Zend_Form_Element_File) {
                    $form->removeElement($element->getName());
                }
            }
            
            if ($form->isValid($params) && empty($errors[$fid])) {
                if($params['hstatus'] == 'C') {
                    $result['createRecords'][$fid] = $editingPlugin->addItemForEdit($table, $params['the_geom'], $form);
                } else {
                    if(! in_array($params[$pkeyName], $skip)) {
                        if($params['hstatus'] == 'U') {
                               $editingPlugin->updateItemForEdit($params[$pkeyName], $table, $params['the_geom'], $form);
                        } else if($params['hstatus'] == 'D') {
                            $editingPlugin->deleteItemForEdit($params[$pkeyName], $table, $form);
                        }
                    } else {
                        //Revert skipped to initial state
                        if(isset($dbRecords[$params[$pkeyName]])) {
                            $data = new stdClass();
                            $data->type = 'Feature';
                            $data->geometry = Zend_Json::decode($dbRecords[$params[$pkeyName]]['the_geom']);
                    
                            unset($dbRecords[$params[$pkeyName]]['the_geom']);
                            $data->properties = $dbRecords[$params[$pkeyName]];
                                    
                            $result['updateRecords'][] = $data;
                        } else {
                            $result['deleteRecords'][] = $params[$pkeyName];
                        }
                    }
                }
            } else {
                if(! isset($errors[$fid])) {
                    $errors[$fid] = array();
                }
                $errors[$fid] = array_merge($errors[$fid], $form->getMessages());
            }
        }
        
        if(count($errors)) {
            $result['errors'] = $errors;
        }
        
        $result['timestamp'] = time();
        $this->_helper->json($result);
    }
    
    public function editgeometryAction()
    {
        $result['replyCode'] = 200;
        $translate = Zend_Registry::get('Zend_Translate');
        
        if($this->_editgeom == false && $this->_editattr == false) {
            $this->triggerError("Editing is forbidden");    
        }
        
        $geometry = $this->_request->getParam('geometry');
        
        //Validate geometry
        if(! empty($geometry)) {
            $validateGeometry = new Giscuit_Validate_Geometry();
            $validateResult = $validateGeometry->isValid($geometry);
            if ($validateResult !== true) {
                foreach ($validateGeometry->getMessages() as $key => $value) {
                    $result['errors'][] = $translate->_('invalidGeometry') . ": " . $key;
                }
            }
        } else {
            $result['errors'][] = $translate->_('geometryEmpty');
        }
        
        if (! isset($result['errors'])) {
            $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
            $editingPlugin->updateItemForEdit($this->_id, $this->_table, $geometry, NULL);
        }
        
        $this->_helper->json($result);
    }
}