<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: PrivatelayersController.php 2946 2012-08-17 15:10:49Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_PrivatelayersController extends Zend_Controller_Action
{

    public function editAction ()
    {
        //Update cache
        Zend_Registry::get('settings')->offlineTimestamp = time();
        
        $this->_helper->viewRenderer->setNoRender();
        
        //Filters
        $filters = array(
            '*' => 'StringTrim'
        );
        
        //Validators
        $columnValidator = new Zend_Validate_InArray(array(
            'view' , 
            'select' , 
            'label' , 
            'edit'
        ));
        $valueValidator = new Zend_Validate_InArray(array(
            'true' , 
            'false'
        ));
        
        $validators = array(
            'column' => $columnValidator , 
            'value' => $valueValidator , 
            'id' => 'Digits'
        );
        
        //Input
        $input = new Zend_Filter_Input($filters, $validators, $this->getRequest()->getParams());
        
        if ($input->isValid()) {
            $result['replyCode'] = 200;
            
            $layerSettingsTable = new Giscuit_Db_Table_LayerSettings();
            $layerSettingsTable->editSingleItem($input->id, $input);
        
        } else {
            $result['replyCode'] = 500;
            $result['replyError'] = Zend_Debug::dump($input->getMessages(), null, true);
        }
        
        $this->_helper->json($result);
    }

    public function settingsAction ()
    {
        //Update cache
        Zend_Registry::get('settings')->offlineTimestamp = time();
        
        $config = Zend_Registry::get('config');
        $result['replyCode'] = 200;
        
        $id = intval($this->_request->getParam('id', 0));
                            
        $form = $this->getSettingsForm();
        
        if(! $this->_request->isPost()) {
            $form->populate(array(
                'id' => $id
            ));
            
            $mapOptions = array(
                'orderDirection' => 'asc' , 
                'whereActive' => 't' , 
                'withLayerSettings' => true , 
                'withFields' => true , 
                'withAcl' => true , 
                'id' => $id
            );
            $table = Giscuit_Db_Table_Layers::getLayerTableById($id, $mapOptions);
            $fields = $table->getPublicFields();
            if (count($fields) > $config->maxFieldsAmount) {
                $result['showWarning'] = true;
            } else {
                $result['showWarning'] = false;
            }
            
            $result['form'] = (string) $form;
        } else {
            $form->populate($this->_request->getParams());
            
            if ($form->isValid($this->_request->getParams())) {
                $layerSettingsTable = new Giscuit_Db_Table_LayerSettings();
                $layerSettingsTable->editItem($id, $form);
            } else {
                $result['form'] = (string) $form;
            }
        }
        
        $this->_helper->json($result);
    }

    public function getSettingsForm ()
    {
        $form = new Giscuit_Form_PrivatelayersSettings();

        $form->removeElement('view');
        $form->removeElement('select');
        $form->removeElement('label'); 
        $form->removeElement('edit');
        
        return $form;
    }
       
    public function moveAction ()
    {
        //Input
        $request = $this->getRequest();
        
        $layerSettingsTable = new Giscuit_Db_Table_LayerSettings();
        
        $first = intval($request->getParam('first'), 0);
        $second = intval($request->getParam('second'), 0);
        
        $firstRow = $layerSettingsTable->getCurrentUserRowByLayerId($first);
        $secondRow = $layerSettingsTable->getCurrentUserRowByLayerId($second);
        
        if (empty($first) || empty($second)) {
            $result['replyCode'] = 500;
            $result['replyError'] = 'Invalid input';
        } else {
            
            $layerSettingsTable->moveItem($firstRow->id, $secondRow->id);
            
            $result['replyCode'] = 200;
        }
        
        $this->_helper->json($result);
    }
    
    public function editenableAction ()
    {
        //Update cache
        Zend_Registry::get('settings')->offlineTimestamp = time();
        Zend_Registry::get('settings')->editingHistory = array();
        
        $result['replyCode'] = 200;
        
        $layersTable = new Giscuit_Db_Table_Layers();
        
        $editResult = $layersTable->getEditLayer();
                
        $result['editgeom'] = $editResult['editgeom'];
        $result['editattr'] = $editResult['editattr'];
        $result['edit_maxscale'] = NULL;
        $result['type'] = '';
        $result['history'] = false;
        $result['changesCount'] = 0;
        
        if (! empty($editResult['settings'])) {
            $result['edit_maxscale'] = $editResult['settings']['edit_maxscale'];
            
            $layerSettings = $editResult['settings'];
            
            $layerClassName = 'Giscuit_Map_Layer_' . $layerSettings['class'];
            if (class_exists($layerClassName)) {
                if (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Point')) {
                    $result['type'] = 'Point';
                } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Line')) {
                    $result['type'] = 'Line';
                } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Polygon')) {
                    $result['type'] = 'Polygon';
                }
            }
            
            //SLD
            $map = new Giscuit_Map_Map(GT_CONFIG_PATH . DIR_SEP . 'default.map');
            $map->setParamsFromConfig();
            $layer = new $layerClassName($map, $editResult['settings']);
            $result['sld'] = $layer->generateSLD('giscuit_edit');
                        
            $tableClassName = 'Giscuit_Db_Table_Layer_' . $layerSettings['class'];
            if (class_exists($tableClassName) &&
                is_subclass_of($tableClassName, 'Giscuit_Db_Table_Layer_History')) {
                $result['history'] = true; 
            }
        }
        
        $settings = Zend_Registry::get('settings');
        
        //Enable editing
        $settings->editingActive = true;
        
        //Reset settings
        $settings->historyId = NULL;
        $settings->historyDateBegin = NULL;
        $settings->historyDateEnd = NULL;
        
        //TODO: Reset querybuilder settings
        
        $this->_helper->json($result);
    }
    
    public function editdisableAction ()
    {
        //Update cache
        Zend_Registry::get('settings')->offlineTimestamp = time();
        
        $result['replyCode'] = 200;
        
        //Disable editing
        Zend_Registry::get('settings')->editingActive = false;
                
        $this->_helper->json($result);
    }
    
    public function orderAction ()
    {
        //Update cache
        Zend_Registry::get('settings')->offlineTimestamp = time();
        
        $layerSettingsTable = new Giscuit_Db_Table_LayerSettings();
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $ids = $request->getParam('ids');

        //
        $ids = explode(',', $ids);
        $ids = array_map('intval', $ids);
        
        //Update
        for($i = 0; $i < count($ids); $i++) {
            if(empty($ids[$i])) {
                continue;
            }
            $layerSettingsTable->update(array('order' => $i + 1), "layer_id = {$ids[$i]} AND user_id = {$userId}");
        }
                
        $this->_helper->json($result);
    }

}