<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: SearchController.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_SearchController extends Zend_Controller_Action
{
    public $timeout = 15000;
    
    public $limit = 100;
    
    public function indexAction ()
    {
        $db = Zend_Registry::get('db');
        
        $result['replyCode'] = 200;
        $result['timeout'] = $this->timeout;
        $result['limit'] = $this->limit;
        
        $form = $this->getForm();
        
        if(! $this->_request->isPost()) {
            $form->populate($this->_request->getParams());
            $result['form'] = (string) $form;
                
        } else {
            $form->populate($this->_request->getParams());
            
            if ($form->isValid($this->_request->getParams())) {
                
                $field = $form->getValue('field');
                $text = $form->getValue('text');
                $where = $form->getValue('where');
                
                if ($where != 'everywhere') {
                    $config = Zend_Registry::get('config');
                    
                    $where = explode(',', $where);
                    $where = array_map('floatval', $where);
                    
                    if (count($where) != 4) {
                        throw new Giscuit_Db_Table_Exception('Bounding box does not have 4 coordinates');
                    }
                    
                    $bbox = "setSRID('BOX3D({$where[0]} {$where[1]}, {$where[2]} {$where[3]})'::box3d, {$config->map->srid})";
                } else {
                    $bbox = NULL;
                }
                
                //Table
                $tableOptions = $form->getMapOptions();
                if(! empty(Zend_Registry::get('settings')->historyDateEnd)) {
                    $tableOptions['withHistory'] = true;
                }
                $table = Giscuit_Db_Table_Layers::getLayerTableById($form->getValue('layer'), $tableOptions);
                
                
                
                //Form
                $settings = $table->getSettings();
                $formName = 'Giscuit_Form_Layer_' . $settings['class'];
                $formConfigPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $settings['class'] . '.xml';
                if (file_exists($formConfigPath)) {
                    $configContent = new Zend_Config_Xml($formConfigPath);
                    $form = new $formName($configContent);
                } else {
                    $form = new $formName();
                }
                
                //Set timeout
                $db->query("SET statement_timeout TO {$this->timeout};");
                
                try {
                    $records = $table->getByField($field, $text, array(
                        'bbox' => $bbox ,
                        'limit' => $this->limit ,
                        'form' => $form
                    ));
    
                    $result['records'] = $records['records'];
                    $result['scale'] = $records['scale'];
                    $result['defaultField'] = $table->getDefaultField();
    
                    foreach($table->getPublicFields() as $field) {
                        $result['fields'][] = $field['name'];    
                    }
                    
                } catch (Exception $e) {
                    $result['replyCode'] = 500;
                    $result['error'] = $e->getMessage();
                }
                
                //Unset timeout
                $db->query("SET statement_timeout TO 0;");    
            } else {
                $result['form'] = (string) $form;
            }
        }
        
        $this->_helper->json($result);
    }

    public function getForm ($fieldId = null)
    {
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if(! empty($userId)) {
            $form = new Giscuit_Form_Search();
        } else {
            $form = new Giscuit_Form_Publish_Search();
        }
        
        return $form;
    }

    public function fieldsAction ()
    {
        $request = $this->getRequest();
        $layerId = intval($request->getParam('layer_id'));
        
        if (empty($layerId)) {
            $result['replyCode'] = 500;
            $result['replyError'] = 'Invalid input';
        } else {
            $form = $this->getForm($layerId);
            $result = $form->getFields($layerId);
            $result['replyCode'] = 200;
        }
        
        $this->_helper->json($result);
    }

}