--
-- PostgreSQL database dump
--

SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

--
-- Name: rastercut; Type: TABLE;
--

CREATE TABLE rastercut (
    gid integer NOT NULL,
    user_id integer,
    comment character varying(100),
    the_geom public.geometry,
    date timestamp with time zone DEFAULT now(),
    area numeric,
    scale integer,
    CONSTRAINT enforce_dims_the_geom CHECK ((public.ndims(the_geom) = 2)),
    CONSTRAINT enforce_geotype_the_geom CHECK (((public.geometrytype(the_geom) = 'MULTIPOLYGON'::text) OR (the_geom IS NULL)))
);

CREATE SEQUENCE rastercut_gid_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE rastercut_gid_seq OWNED BY rastercut.gid;

ALTER TABLE rastercut ALTER COLUMN gid SET DEFAULT nextval('rastercut_gid_seq'::regclass);

ALTER TABLE ONLY rastercut ADD CONSTRAINT rastercut_pkey PRIMARY KEY (gid);

CREATE INDEX rastercut_the_geom_gist ON rastercut USING gist (the_geom);

--
-- Name: world_boundaries; Type: TABLE;
--

CREATE TABLE world_boundaries (
    gid integer NOT NULL,
    name character varying(36),
    iso_a2 character varying(2),
    iso_a3 character varying(3),
    dial integer,
    the_geom public.geometry,
    people integer,
    CONSTRAINT enforce_dims_the_geom CHECK ((public.ndims(the_geom) = 2)),
    CONSTRAINT enforce_geotype_the_geom CHECK (((public.geometrytype(the_geom) = 'MULTIPOLYGON'::text) OR (the_geom IS NULL)))
);


CREATE SEQUENCE world_boundaries_gid_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER SEQUENCE world_boundaries_gid_seq OWNED BY world_boundaries.gid;

ALTER TABLE world_boundaries ALTER COLUMN gid SET DEFAULT nextval('world_boundaries_gid_seq'::regclass);

ALTER TABLE ONLY world_boundaries ADD CONSTRAINT world_boundaries_pkey PRIMARY KEY (gid);

CREATE INDEX world_boundaries_the_geom_gist ON world_boundaries USING gist (the_geom);