<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Base.php 2896 2012-08-01 15:28:28Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for base table 
 *
 * @copyright  Copyright (c) 2012 VEC
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 */
class Giscuit_Db_Table_Base extends Giscuit_Db_Table_Multilang
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'base';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Contains the names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array(
        'name'
    );

    /**
     * Get classes
     * 
     * @param array $options Available options:
     * <ul><li>withMultilangFields bool - Add multilanguage fields based on current language</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'active' ,
            'minscaledenom' ,
            'maxscaledenom' ,
            'selected' , 
            'format' ,
            'parent_id' , 
            'order'
        );
        
        $select->from($this, $fields);
        
        $this->whereId($select, $options);
        $this->whereActive($select, $options);
        
        $this->withMultilangFields($select, $options);
        if (empty($options['withMultilangFields'])) {
            $this->joinMultilangField($select, 'name');
        }
        
        //Order
        $select->order($this->_name . '.order asc');
        
        $result = $this->fetchAll($select);
        
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * 
     * @return integer The id of the new element
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Multilang text
        $this->manageMultilangText($this->getMultilangPostData($data->getValues()), $row, Giscuit_Db_Table_Multilang::UPDATE);
        
        $row->selected = $data->getValue('selected');
        $row->format = $data->getValue('format');
        $row->active = $data->getValue('active');
        $row->parent_id = $data->getValue('parent_id');
        
        //minscaledenom
        $minscaledenom = $data->getValue('minscaledenom');
        if (empty($minscaledenom)) {
            $row->minscaledenom = NULL;
        } else {
            $row->minscaledenom = $data->getValue('minscaledenom');
        }
        
        //maxscaledenom
        $maxscaledenom = $data->getValue('maxscaledenom');
        if (empty($maxscaledenom)) {
            $row->maxscaledenom = NULL;
        } else {
            $row->maxscaledenom = $data->getValue('maxscaledenom');
        }
        
        if ($row->selected == 't') {
            $this->unselectAll();
        }
        
        Giscuit_Db_Table_LayerPublish::updateTilecacheConfig();
        Giscuit_Map_Map::clearLink();
        
        return $row->save();
    
    }

    /**
     * Add item
     * 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return Zend_Db_Table_Row
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        
        $multilangPostData = $this->getMultilangPostData($data->getValues());
        $this->manageMultilangText($multilangPostData, $newRow);
        
        $newRow->selected = $data->getValue('selected');
        $newRow->format = $data->getValue('format');
        $newRow->active = $data->getValue('active');
        $newRow->parent_id = $data->getValue('parent_id');
        
        //minscaledenom
        $minscaledenom = $data->getValue('minscaledenom');
        if (empty($minscaledenom)) {
            $row->minscaledenom = NULL;
        } else {
            $row->minscaledenom = $data->getValue('minscaledenom');
        }
        
        //maxscaledenom
        $maxscaledenom = $data->getValue('maxscaledenom');
        if (empty($maxscaledenom)) {
            $row->maxscaledenom = NULL;
        } else {
            $row->maxscaledenom = $data->getValue('maxscaledenom');
        }
        
        if ($newRow->selected == 't') {
            $this->unselectAll();
        }
        
        $pkey = $newRow->save();
        
        $newRow->order = $pkey;
        $newRow->save();
        
        Giscuit_Db_Table_LayerPublish::updateTilecacheConfig();
        Giscuit_Map_Map::clearLink();
        
        return $newRow;
    }

    /**
     * Set the value of selected field in all rows to false
     * 
     * @return void
     */
    public function unselectAll ()
    {
        $query = "UPDATE \"{$this->_schema}\".\"{$this->_name}\" 
            SET selected = 'f'
            WHERE parent_id = NULL";
        
        $this->getDefaultAdapter()->query($query);
    
    }
}
