<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Files.php 2897 2012-08-02 11:41:08Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for files table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Files extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'files';
    
    /**
     * List of file extensions that will ne treated as an image 
     * 
     * @var array
     */
    public static $imageExtensions = array(
        'jpeg',
        'jpg' ,
        'png',
        'gif'
    );

    /**
     * Get items
     *   
     * @param array $options Available options:
     * <ul><li>whereId integer - Fetch rows that match the id</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' ,
            'oid' ,
            //'type' ,
            'name' ,
            'for_layer_id'
        );
                        
        $select->from($this, $fields);
        
        $this->whereId($select, $options);
        //$select->where($this->_name . '.for_user_id IS NULL');
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        /*
        if(! empty($userId)) {
            $select->orWhere($this->_name . '.for_user_id = ?', $userId);
        }
        */    
        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @return integer The id of the new element
     */
    public function addItem (array $data)
    {
        $file = $data['file'];
        
        if (! file_exists($file)) {
            throw new Giscuit_Db_Table_Exception('File does not exists');
        }
        
        //Verify filename length
        if (strlen($data['name']) > 255) {
            throw new Giscuit_Db_Table_Exception('Filename is more than 255 characters long');
        }
        
        //Add file
        $pdo = $this->getAdapter()->getConnection();
        $pdo->beginTransaction();
        $oid = $pdo->pgsqlLOBCreate();
        $stream = $pdo->pgsqlLOBOpen($oid, 'w');
        $local = fopen($file, 'rb');
        stream_copy_to_stream($local, $stream);
        $pdo->commit();
        
        //Clean up
        $local = $stream = null;
        unlink($file);
        
        //Add row
        $newRow = $this->createRow();
        $newRow->oid = $oid;

        $newRow->name = $data['name'];
        
        if(! empty($data['layer_id'])) {
            $newRow->for_layer_id = $data['layer_id'];
        }
        
        //By user id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        if (! empty($userId)) {
            $newRow->by_user_id = $userId;
        }
        
        //For user id
        if(! empty($data['for_layer_id'])) {
            $newRow->for_user_id = $data['for_layer_id'];
        }
        
        //Date
        $newRow->date = new Zend_Db_Expr('now()');
        
        return $newRow->save();

    }
    
    /**
     * Delete item
     * 
	 * @param integer $id File id
     * @return void
     */
    public function deleteItem($id) 
    {
        $this->find($id)->current()->delete();
    }

}
