<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Joins.php 2897 2012-08-02 11:41:08Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for fields table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Joins extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'joins';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Private Joins
     * 
     * @var array
     */
    /*
    protected $_privateJoins = array(
        'gid' , 
        'the_geom'
    );
    */

    /**
     * Get Joins
     * 
     * @param array $options Available options:
     * <ul><li>whereStm string - Fetch rows that match the statement</li>
     * <li>whereLayerId integer - Fetch rows that match the layer id</li>
     * <li>asArray bool - Get results as array</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $Joins = array(
            'id' , 
            'layer_id' ,
            'expression' ,
            'table' , 
            'order'
            //'name' , 
            //'join_Join' , 
            //'join_to' , 
            //'for_render' , 
            //'visibility' ,
        );
        
        $select->from($this, $Joins);
        
        if (isset($options['whereStm'])) {
            $select->where($options['whereStm']);
        }
        
        if (isset($options['whereLayerId'])) {
            $select->where($this->_name . '.layer_id = ?', $options['whereLayerId']);
        }
        
        $this->whereId($select, $options);
        
        $select->order($this->_name . '.order asc');
        
        $result = $this->fetchAll($select);
        
        //Convert to array
        if (isset($options['asArray']) && $options['asArray'] == true) {
            $result = $result->toArray();
        }
        
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Get column
        $column = $data->column;
        
        if ($column == 'expression') {
            $row->$column = $data->getUnescaped('data');
        } else {
            $row->$column = $data->data;
        }
        
        return $row->save();
    
    }

    /**
     * Add item
     * 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return Zend_Db_Table_Row
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        $newRow->layer_id = $data->layer_id;
        
        $id = $newRow->save();
        
        $newRow->order = $id;
        $newRow->save();
        
        return $newRow;
    }
    
    /**
     * Check if join belongs to current user
     * 
     * @param integer $id Join id
     * @return void
     */
    public function checkCurrUserOwnJoin($id)
    {
        $rows = $this->getItems(array(
        	'whereId' => $id
        ))->toArray();
        if(count($rows) == 0) {
            throw new Exception("Invalid join id ({$id})");
        }
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $layersTable->checkCurrUserOwnLayer($rows[0]['layer_id']);
    }
    
}
