<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Layers.php 3017 2012-10-04 20:11:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for layers table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Layers extends Giscuit_Db_Table_Multilang
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'layers';

    /**
     * User role
     * 
     * @var string
     */
    public $defaultResource = 'registered';

    /**
     * Contains the names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array(
        'name'
    );
    
        
    /**
     * Fetch layer settings data
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>withLayerSettings bool - Fetch rows with layer settings for current user </li>
     * <li>whereView string - Fetch rows with a specific "view" value (t,f)</li>
     * <li>whereSelect string - Fetch rows with a specific "select" (information) value (t,f)</li>
     * <li>whereEdit string - Fetch rows with a specific "edit" value (t,f) </li>
     * <li>orderDirection string - Order direction (ASC, DESC)</li></ul>
     * @return void
     */
    public function withLayerSettings ($select, array $options = array())
    {
        
        if (empty($options['withLayerSettings'])) {
            
            $select->joinLeft(array(
                'layer_defaults' => $this->_schema . '.layer_defaults'
            ), "{$this->_name}.id = layer_defaults.id AND ({$this->_name}.class <> '')", array(
                'view' => 'layer_defaults.view' , 
                'select' => 'layer_defaults.select' , 
                'edit' => 'layer_defaults.edit' , 
                'label' => 'layer_defaults.label' , 
                'label_size' => 'layer_defaults.label_size' , 
                'label_color' => 'layer_defaults.label_color' , 
                'label_fields' => 'layer_defaults.label_fields' , 
                'order' => 'layer_defaults.order'
            ));
            
            if (! empty($options['orderDirection'])) {
                $select->order('layer_defaults.order ' . $options['orderDirection']);
            }
            
            return;
        }
        
        //Get current user id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $select->joinLeft(array(
            'layer_settings' => $this->_schema . '.layer_settings'
        ), "{$this->_name}.id = layer_settings.layer_id AND layer_settings.user_id = $userId AND ({$this->_name}.class <> '')", array(
            'layer_setting_id' => 'layer_settings.id' , 
            'view' => 'layer_settings.view' , 
            'select' => 'layer_settings.select' , 
            'edit' => 'layer_settings.edit' , 
            'label' => 'layer_settings.label' , 
            'label_size' => 'layer_settings.label_size' , 
            'label_color' => 'layer_settings.label_color' , 
            'label_fields' => 'layer_settings.label_fields' ,
            'order' => 'layer_settings.order'
        ));
        
        //Where view
        if (! empty($options['whereView'])) {
            $select->where('layer_settings.view = ? OR layer_settings.id IS NULL', $options['whereView']);
        }
        
        //Where select
        if (! empty($options['whereSelect'])) {
            $select->where('layer_settings.select = ? OR layer_settings.id IS NULL', $options['whereSelect']);
        }
        
        //Where edit
        if (! empty($options['whereEdit'])) {
            $select->where('layer_settings.edit = ?', $options['whereEdit']);
        }
        
        //Order direction
        if (! empty($options['orderDirection'])) {
            $select->order('layer_settings.order ' . $options['orderDirection']);
        }
    
    }

    /**
     * Fetch container data
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>withGroups bool - Fetch with container rows</li></ul>
     * @return void
     */
    public function withGroups ($select, array $options = array())
    {
        
        if (empty($options['withGroups'])) {
            $select->where("({$this->_name}.class <> '')");
            
            return;
        }
        
        $parentIdField = "COALESCE({$this->_name}.parent_id, 0)";
        $select->order("$parentIdField ASC, \"order\" ASC");
    
    }

    /**
     * Fetch publish data
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>withPublish bool - Fetch publish data</li>
     * <li>whereWmsPublic string - Fetch rows with a specific "wms_public" value (t,f)</li>
     * <li>whereWmsPrivate string - Fetch rows with a specific "wms_private" value (t,f)</li>
     * <li>whereWfsPublic string - Fetch rows with a specific "wfs_public" value (t,f)</li>
     * <li>whereWfsPrivate string - Fetch rows with a specific "wfs_private" value (t,f)</li>
     * <li>whereWcsPublic string - Fetch rows with a specific "wcs_public" value (t,f)</li>
     * <li>whereWcsPrivate string - Fetch rows with a specific "wcs_private" value (t,f)</li></ul>
     * @return void
     */
    public function withPublish ($select, array $options = array())
    {
        
        if (empty($options['withPublish'])) {
            return;
        }
        
        $fields = array(
            'format' => 'layer_publish.format' ,
            'destination' => 'layer_publish.destination' ,
            'selected' => 'layer_publish.selected',
            'wms_private' => 'layer_publish.wms_private' , 
            'wms_public' => 'layer_publish.wms_public' , 
            'wfs_private' => 'layer_publish.wfs_private' , 
            'wfs_public' => 'layer_publish.wfs_public' , 
            'wcs_private' => 'layer_publish.wcs_private' , 
            'wcs_public' => 'layer_publish.wcs_public' , 
            'wcs_formats' => 'layer_publish.wcs_formats' , 
            'publish' => 'layer_publish.publish'
        );
        
        $select->joinLeft(array(
            'layer_publish' => $this->_schema . '.layer_publish'
        ), "{$this->_name}.id = layer_publish.id AND {$this->_name}.class <> ''", $fields);
        
        //Where publish
        if (isset($options['wherePublish'])) {
            $select->where('layer_publish.publish IS NULL OR layer_publish.publish = ?', $options['wherePublish']);
        }

        //Where wms public
        if (isset($options['whereWmsPublic'])) {
            $select->where('layer_publish.wms_public IS NULL OR layer_publish.wms_public = ?', $options['whereWmsPublic']);
        }
        
        //Where wms private
        if (isset($options['whereWmsPrivate'])) {
            $select->where('layer_publish.wms_private IS NULL OR layer_publish.wms_private = ?', $options['whereWmsPrivate']);
        }
        
        //Where wfs public
        if (isset($options['whereWfsPublic'])) {
            $select->where('layer_publish.wfs_public IS NULL OR layer_publish.wfs_public = ?', $options['whereWfsPublic']);
        }
        
        //Where wfs private
        if (isset($options['whereWfsPrivate'])) {
            $select->where('layer_publish.wfs_private IS NULL OR layer_publish.wfs_private = ?', $options['whereWfsPrivate']);
        }
        
        //Where wcs public
        if (isset($options['whereWcsPublic'])) {
            $select->where('layer_publish.wcs_public IS NULL OR layer_publish.wcs_public = ?', $options['whereWcsPublic']);
        }
        
        //Where wcs private
        if (isset($options['whereWcsPrivate'])) {
            $select->where('layer_publish.wcs_private IS NULL OR layer_publish.wcs_private = ?', $options['whereWcsPrivate']);
        }
    
    }

    /**
     * Fetch rows with the specified ids
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereIds array - Fetch rows with specific ids</li></ul>
     * @return void
     */
    public function whereIds ($select, array $options = array())
    {
        if (empty($options['whereIds'])) {
            return;
        }
        
        $whereStm = $this->getAdapter()->quoteIdentifier($this->_name . '.id') . ' = ' . implode(' OR ' . $this->getAdapter()->quoteIdentifier($this->_name . '.id') . ' = ', $options['whereIds']);
        $select->where($whereStm);
    }

    /**
     * Add default layer settings to the result
     * 
     * @param Zend_Db_Select $select
     * @param array $result Result
     * @param array $options Available options:
     * <ul><li>withLayerSettings bool - Add default layer settings to the result</li></ul>
     * @return boolean
     */
    public function addDefaultSettings ($select, $result, array $options = array())
    {
        $addLayersFlag = false;
        
        if (empty($options['withLayerSettings'])) {
            return false;
        }
         
        foreach ($result as $row) {
            if ($row['active'] == 't' && (! empty($row['class'])) && $row['layer_setting_id'] == null) {
                $addLayersFlag = true;
                break;
            }
        }
        
        if (empty($addLayersFlag)) {
            return false;
        }
        
        $layerSettingsTable = new Giscuit_Db_Table_LayerSettings();
        
        foreach ($result as $row) {
            if ($row['layer_setting_id'] == null && (! empty($row['class']))) {
                $layerSettingsTable->addItem($row);
            }
        }
        
        return true;
    
    }

    /**
     * Fetch routing data
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>withRouting bool - Fetch routing data</li>
     * <li>whereRoutingActive string - Fetch rows with a specific "routing_active" value (t,f)</li></ul>
     * @return void
     */
    public function withRouting ($select, array $options = array())
    {
        if (empty($options['withRouting'])) {
            return;
        }
        
        //
        if (isset($options['whereRoutingActive'])) {
            $select->where('layer_routing.active = ?', $options['whereRoutingActive']);
        }
        
        //
        $fields = array (
            'routing_id' => 'layer_routing.id',
            'routing_active' => 'layer_routing.active',
            'routing_source' => 'layer_routing.source', 
            'routing_reverse_source' => 'layer_routing.reverse_source', 
            'routing_algorithm' => 'layer_routing.algorithm', 
            'routing_directed' => 'layer_routing.directed', 
            'routing_reverse_cost' => 'layer_routing.reverse_cost',
            'routing_processing' => 'layer_routing.processing'
        );
        
        $select->joinLeft(
            array(
                'layer_routing' => $this->_schema . '.layer_routing'
            ), 
            "{$this->_name}.id = layer_routing.id", 
            $fields
        );
    }
    
    /**
     * Fetch routing cost coefficient data
     * 
     * @param Zend_Db_Select $select
      * @param array $options Available options:
     * <ul><li>withRoutingCostCoefColumn string - Fetch rows with a specific cost coefficient column name</li></ul>
     * @return void
     */
    public function withRoutingCostCoefColumn ($select, array $options = array())
    {
        if (empty($options['withRoutingCostCoefColumn'])) {
            return;
        }
        
        $fields = array (
            'cost_coef_column' => 'routing_cost_columns.column',
            'reverse_cost_coef_column' => 'routing_cost_columns.reverse_column',
        );
            
        $select->joinLeft(
            array(
                'routing_cost_columns' => $this->_schema . '.routing_cost_columns'
            ), 
            "{$this->_name}.id = routing_cost_columns.layer_id AND " . "routing_cost_columns.id = {$options['withRoutingCostCoefColumn']}", 
            $fields
        );
        
    }
    
    /**
     * Fetch access list data
     * 
     * @param Zend_Db_Select $select
     * @param array $result Result
     * @param array $options Available options:
     * <ul><li>withAcl bool - Fetch acl data</li></ul>
     * @return void
     */
    public function withAcl ($select, &$result, array $options = array())
    {
        if (empty($options['withAcl'])) {
            return;
        }
        
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        foreach ($result as $key => $row) {
        	if($row['permissions_user_id'] == $userId) {
        		
        	} else if(strstr(',' . $row['permissions_view_ids'], ',' . $userId . ',')) {

        	} else if($row['permissions_user_id'] && $isAllowed->direct('admin')) {
        		unset($result[$key]);
        	} else if ($isAllowed->direct($row['resource'], $row['priv_view']) == false) {
                unset($result[$key]);
            }
        
        }
    
    }

    /**
     * Fetch OWS access list data
     * 
     * @param Zend_Db_Select $select
     * @param array $result Result
     * @param array $options Available options:
     * <ul><li>withOwsAcl bool - Fetch OWS access list data</li></ul>
     * @return void
     */
    public function withOwsAcl ($select, &$result, array $options = array())
    {
        if (empty($options['withOwsAcl'])) {
            return;
        }
        
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        
        foreach ($result as $key => $row) {
        	
        	if($row['permissions_user_id'] == $userId) {
        	
        	} else if(strstr(',' . $row['permissions_ows_ids'], ',' . $userId . ',')) {
        		
        	} else if($row['permissions_user_id'] && $isAllowed->direct('admin')) {
        		unset($result[$key]);
        	} else if ($isAllowed->direct($row['resource'], $row['priv_ows']) == false) {
                unset($result[$key]);
            }
        }
    
    }

    /**
     * Parse the result
     * 
     * @param Zend_Db_Select $select
     * @param array $result Result
     * @param array $options Available options
     * @return void
     */
    public function parse ($select, &$result, array $options = array())
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        foreach ($result as $key => $row) {
            
            if (empty($result[$key]['resource'])) {
                $result[$key]['resource'] = $this->defaultResource;
            }
            
            if (empty($result[$key]['name'])) {
                $result[$key]['name'] = $translate->_('nameEmpty');
            }
            
            //Explode destination
            if (! empty($row['destination'])) {
                $result[$key]['destination'] = explode(',', $result[$key]['destination']);
            }
            
            //Explode wcs_formats
            if (! empty($row['wcs_formats'])) {
                $result[$key]['wcs_formats'] = explode(',', $result[$key]['wcs_formats']);
            }
            
            //Explode label fields
            if (! empty($row['label_fields'])) {
                $result[$key]['label_fields'] = explode(',', $result[$key]['label_fields']);
            }
            
        }
        
    }

    /**
     * Fetch classes and styles data
     * 
     * @param Zend_Db_Select $select
     * @param array $result
     * @param array $options Available options:
     * <ul><li>withClassesAndStyles bool - Fetch classes and styles data</li>
     * <li>withPublish bool - Fetch additional publish data</li></ul>
     * @return void
     */
    public function withClassesAndStyles ($select, &$result, array $options = array())
    {
        
        if (empty($options['withClassesAndStyles'])) {
            return;
        }
        
        $ids = array();
        $classesIds = array();
        
        foreach ($result as $key => $row) {
            $ids[] = $row['id'];

            $result[$key]['classes'] = array();
        }
        
        //Get classes
        if (! empty($ids)) {
            
            //Fetch additional publish data
            if(empty($options['withPublish'])) {
                $withClassLabels = false;
            } else {
                $withClassLabels = true;
            }
            
            $classesTable = new Giscuit_Db_Table_Classes();
            $classesTableName = $classesTable->info('name');
            $whereStm = "{$classesTableName}.layer_id = " . implode(" OR {$classesTableName}.layer_id = ", $ids);
            $classes = $classesTable->getItems(array(
                'whereStm' => $whereStm ,
                'withClassLabels' => $withClassLabels
            ))->toArray();
        
        }
        
        if (! empty($classes)) {
            
            foreach ($classes as $class) {
                $classesIds[] = $class['id'];
                foreach ($result as $layerKey => $layer) {
                    
                    if ($class['layer_id'] == $layer['id']) {
                        $class['styles'] = array();
                        $result[$layerKey]['classes'][] = $class;
                    }
                }
            }
            
            //Get styles
            $stylesTable = new Giscuit_Db_Table_Styles();
            $stylesTableName = $stylesTable->info('name');
            $whereStm = "{$stylesTableName}.class_id = " . implode(" OR {$stylesTableName}.class_id = ", $classesIds);
            $styles = $stylesTable->getItems(array(
                'whereStm' => $whereStm
            ))->toArray();
            
            foreach ($result as $layerKey => $layer) {
                foreach ($layer['classes'] as $classKey => $class) {
                    foreach ($styles as $style) {
                        if ($class['id'] == $style['class_id']) {
                            $result[$layerKey]['classes'][$classKey]['styles'][] = $style;
                        }
                    }
                }
            }
        
        }
    
    }

    /**
     * Fetch fields data
     * 
     * @param Zend_Db_Select $select
     * @param array $result Result
     * @param array $options Available options:
     * <ul><li>withFields bool - Fetch fields data</li></ul>
     * @return void
     */
    public function withFields ($select, &$result, array $options = array())
    {
        if (empty($options['withFields'])) {
            return;
        }
        
        if (! is_array($result)) {
            $result = $result->toArray();
        }
        
        $ids = array();
        $fieldsIds = array();
        
        foreach ($result as $key => $row) {
            $ids[] = $row['id'];
            
            $result[$key]['fields'] = array();
        }
        
        if (! empty($ids)) {
            
            //Fields
            $fieldsTable = new Giscuit_Db_Table_Fields();
            $fieldsLayerIdField = $this->getAdapter()->quoteIdentifier($fieldsTable->info('name') . '.layer_id');
            $whereStm = "$fieldsLayerIdField = " . implode(" OR $fieldsLayerIdField = ", $ids);
            
            $fields = $fieldsTable->getItems(array(
                'whereStm' => $whereStm
            ))->toArray();
            
            foreach ($fields as $field) {
                $fieldsIds[] = $field['id'];
                foreach ($result as $layerKey => $layer) {
                    // || (isset($layer['history_layer_id']) && $field['layer_id'] == $layer['history_layer_id']))
                    if ($field['layer_id'] == $layer['id']) {
                        $result[$layerKey]['fields'][] = $field;
                    }
                }
            }
            
            //Joins
            $joinsTable = new Giscuit_Db_Table_Joins();
            $joinsLayerIdField = $this->getAdapter()->quoteIdentifier($joinsTable->info('name') . '.layer_id');
            $whereStm = "$joinsLayerIdField = " . implode(" OR $joinsLayerIdField = ", $ids);
            $joins = $joinsTable->getItems(array(
                'whereStm' => $whereStm
            ))->toArray();
            
            foreach ($joins as $join) {
                $joinsIds[] = $join['id'];
                foreach ($result as $layerKey => $layer) {
                    // || (isset($layer['history_layer_id']) && $join['layer_id'] == $layer['history_layer_id'])
                    if ($join['layer_id'] == $layer['id']) {
                        $result[$layerKey]['joins'][] = $join;
                    }
                }
            }
        
        }
    
    }

    /**
     * Fetch metadata data
     * 
     * @param Zend_Db_Select $select
     * @param array $result
     * @param array $options Available options:
     * <ul><li>withMetadata bool - Fetch metadata data</li></ul>
     * @return void
     */
    public function withMetadata ($select, &$result, array $options = array())
    {
        if (empty($options['withMetadata'])) {
            return;
        }
        
        if (! is_array($result)) {
            $result = $result->toArray();
        }
        
        $config = Zend_Registry::get('config');
        $configOws = $config->ows->toArray();
        $locale = Zend_Registry::get('Zend_Translate')->getLocale();
        
        foreach ($result as $key => $row) {
            $metaData = $row['metadata'];
            
            if(empty($configOws[$locale])) {
                $configOws[$locale] = array();
            }
            
            if (empty($metaData)) {
                $result[$key]['metadata'] = $configOws[$locale];
            } else {
                $dbOws = Zend_Json::decode($metaData);
                                
                if (isset($dbOws[$locale])) {
                    $result[$key]['metadata'] = array_merge($configOws[$locale], $dbOws[$locale]);
                } else {
                    $result[$key]['metadata'] = $configOws[$locale];
                }
            }
            $result[$key]['metadata']['ows_title'] = $row['name'];
            
            foreach ($result[$key]['metadata'] as $keyword => $value) {
                if (empty($value)) {
                    unset($result[$key]['metadata'][$keyword]);
                }
            }
            
        }

    }
    
    /**
     * Get items
     * 
     * @param array $options Available options:
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $select->from($this, array(
            '*'
        ));
        
        $this->withLayerSettings($select, $options);
        $this->withPublish($select, $options);
        $this->withOwsclient($select, $options);
        $this->withCurrUserOwnLayer($select, $options);
        $this->whereIds($select, $options);
        $this->withRouting($select, $options);
        $this->withRoutingCostCoefColumn($select, $options);
        $this->withPermissions($select, $options);
        
        //Where class
        if (! empty($options['whereClass'])) {
            $select->where($this->_name . '.class = ?', $options['whereClass']);
        }
        
        $this->whereActive($select, $options);
        $this->whereId($select, $options);
        $this->withMultilangFields($select, $options);
        
        $this->withGroups($select, $options);
        
        if (empty($options['withMultilangFields'])) {
            $this->joinMultilangField($select, 'name');
        }
        
        $result = $this->getAdapter()->fetchAssoc($select);
        
        $hasNewSettings = $this->addDefaultSettings($select, $result, $options);
        
        //If new settings where added refetch the data
        if ($hasNewSettings == true) {
            $result = $this->getAdapter()->fetchAssoc($select);
        }
        
        $this->parse($select, $result, $options);
        
        $this->withAcl($select, $result, $options);
        $this->withOwsAcl($select, $result, $options);
        
        $this->withClassesAndStyles($select, $result, $options);
        $this->withFields($select, $result, $options);
        $this->withMetadata($select, $result, $options);
        
        //
        $this->updateWfsLayers($result);
        
        return $result;
    }
    
    /**
     * Update wfs layers
     * 
     * @param array $rows Layers
     * @return void  
     */
    public function updateWfsLayers($rows) 
    {
    	$owsclientTable = new Giscuit_Db_Table_LayerOwsclient();
    	
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            $scriptExt = '.bat';
        } else {
            $scriptExt = '.sh';
        }
        
        foreach($rows as $row) {
            if($row['owsclient_update_date']) {
                $lastUpdate = intval($row['owsclient_update_date']);
                $interval = intval($row['owsclient_update_interval']) * 60;
                
                if(! $interval) {
                    continue;
                }
                
                if($lastUpdate + $interval < time()) {
                    //
                    $data = array(
                        'owsclient_update_date' => new Zend_Db_Expr('now()')
                    );
                    $where = $owsclientTable->getAdapter()->quoteInto('id = ?', $row['id']);
                    $owsclientTable->update($data, $where);
                    
                    //
                    $convertCmd = '"' . GT_TMP_PATH . DIR_SEP . $row['table'] . $scriptExt . '"';                    
                    Giscuit_Exec::background($convertCmd);
                }
            }
            
        }
    } 

    /**
     * Edit item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Form $data A form object populated with request data
     * @return void
     */
    public function edit ($id, $data)
    {
        //Update cache
        file_put_contents(GT_TMP_PATH . DIR_SEP . 'offline_timestamp', time());
        
        $row = $this->find($id)->current();
        
        $layerDefaultsTable = new Giscuit_Db_Table_LayerDefaults();
        $defaultRow = $layerDefaultsTable->find($id)->current();
        
        $layerPublishTable = new Giscuit_Db_Table_LayerPublish();
        $publishRow = $layerPublishTable->find($id)->current();
        
        $layerOwsclientTable = new Giscuit_Db_Table_LayerOwsclient();
        $owsclientRow = $layerOwsclientTable->find($id)->current();
        
        //Multilang text
        if (empty($row->name_id)) {
            $method = Giscuit_Db_Table_Multilang::INSERT;
        } else {
            $method = Giscuit_Db_Table_Multilang::UPDATE;
        }
        $this->manageMultilangText($this->getMultilangPostData($data->getValues()), $row, $method);

        $foreignRows = array(
            'default' => $defaultRow, 
            'publish' => $publishRow, 
            'owsclient' => $owsclientRow
        );
        
        //Check for routing plugin
        $routingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Routing');
        if ($routingPlugin != NULL && $routingPlugin->isLayerLine('Giscuit_Map_Layer_' . $row->class)) {
            $layerRoutingTable = new Giscuit_Db_Table_LayerRouting();
            $routingRow = $layerRoutingTable->find($id)->current();
            
            if ($routingRow == NULL) {
                $routingRow = $layerRoutingTable->createRow();
                $routingRow->id = $id;
            }
            
            $foreignRows['routing'] = $routingRow;
        }
        
        $this->_populateRow($row, $data, $foreignRows);
        
        //Only one layer with destination 'base' can be selected
        if($data->getValue('selected') == 't'
            && $data->getValue('destination') != 'overlay') {
            $layerPublishTable->update(array(
                'selected' => 'f',
            ), $layerPublishTable->getAdapter()->quoteInto('destination <> ?', 'overlay'));                
        }
        
        $row->save();
        
        if (empty($row->class)) {
            $data = array('parent_id' => new Zend_Db_Expr('parent_id'));
            $this->update($data, '');
        }
        
        if (! empty($defaultRow)) {
            $defaultRow->save();
        }
        
        if (! empty($publishRow)) {
            $publishRow->save();
            
            Giscuit_Db_Table_LayerPublish::updateTilecacheConfig();
        }
        
        if (! empty($owsclientRow)) {
            $owsclientRow->save();
        }
        
        if (! empty($routingRow)) {
            $routingRow->save();
        }
        
        
        //
        $sql = "UPDATE {$this->_schema}.layer_settings 
        SET label_fields = layer_defaults.label_fields
        FROM {$this->_schema}.layers, {$this->_schema}.layer_defaults
        WHERE layers.id = {$id} AND layers.classlabel_active = 't' AND layers.id = layer_settings.layer_id AND layers.id = layer_defaults.id";
        $this->getAdapter()->query($sql);

    }

    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @param array $options Available options:
     * <ul><li>type string - Layer type (vector, wfs, raster, javascript, wms)</li>
     * <li>layerType string - Layer geometry type (Point, Line, Polygon)</li>
     * <li>parentId integer - Parent container id (root if empty)</li></ul>
     * @return integer Id of the new element
     */
    public function add ($data, array $options = array())
    {
        //Update cache
        file_put_contents(GT_TMP_PATH . DIR_SEP . 'offline_timestamp', time());
        
        $config = Zend_Registry::get('config');
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        $row = $this->createRow();
        
        $type = $options['type'];
        $layerType = $options['layerType'];
        
        //Get multilang fields
        $multilangArr = array();
        $fields = $this->_multilangTextColumns;
        $languages = Zend_Registry::get('config')->language->toArray();
        
        foreach ($languages as $key => $language) {
            foreach ($fields as $field) {
                $multilangArr[$field][$key] = $data->getValue($field . '_' . $key);
            }
        }
        $this->manageMultilangText($multilangArr, $row, Giscuit_Db_Table_Multilang::INSERT);
        
        //Parent id
        if (empty($options['parentId'])) {
            $options['parentId'] = NULL;
        }
        $row->parent_id = $options['parentId'];
        
        $row->active = 'f';
        
        //Table
        if ($type == 'vector' || $type == 'wfs') {
            
            $row->table = $data->getValue('table');
            
            $row->edit_maxscale = $config->map->defaultEditScale;
            $row->snap_maxscale = $config->map->defaultSnapScale;
            
            if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
                $row->label_font = 'arial';
            } else {
                $row->label_font = 'FreeSans';
            }
        
        } elseif ($type == 'raster') {
            //Data
            $row->data = $data->getValue('data');
            
            //Tileindex
            $row->tileindex = $data->getValue('tileindex');
            
            //Tileitem
            $row->tileitem = $data->getValue('tileitem');
        } elseif ($type == 'javascript') {

        }
        
        $row->class = $data->getValue('class');
        
        //Default resource
        if (! $isAllowed->direct('admin')) {
            $row->resource = 'admin_layer';
        }
        
        //Save layer
        $pkey = $row->save();
        
        //Add default
        $layersDefaultsTable = new Giscuit_Db_Table_LayerDefaults();
        $defaultRow = $layersDefaultsTable->createRow();
        
        if ($type == 'vector' || $type == 'wfs') {
            if ($layerType == 'Point') {
                $row->label_position = 107;
            }
            
            if ($layerType == 'Line') {
                $row->label_position = 109;
            }
            
            $mapOptions = array(
                'orderDirection' => 'desc' , 
                'withFields' => true
            );
            
            $table = Giscuit_Db_Table_Layers::getLayerTableById($pkey, $mapOptions);
            
            //Add initial fields
            $fieldsTable = new Giscuit_Db_Table_Fields();
            $defaultRow->label_fields = $row->default_field = $fieldsTable->addInitial($table);
            
            //Add initial class
            $classesTable = new Giscuit_Db_Table_Classes();
            $classId = $classesTable->addInitial($table)->id;
            
            //Add initial style
            $stylesTable = new Giscuit_Db_Table_Styles();
            $stylesTable->addInitial($classId, $layerType);
        } 
        
        if ($type == 'wms' || $type == 'wfs') {
            $layerOwsclientTable = new Giscuit_Db_Table_LayerOwsclient();
            $owsclientRow = $layerOwsclientTable->createRow();
            
            //owsclient_uri
            $owsclientRow->owsclient_uri = $data->getValue('owsclient_uri');
            
            //owsclient_layers
            $owsclientRow->owsclient_layers = $data->getValue('owsclient_layers');
            
            //owsclient_auth_type
            $owsclientRow->owsclient_auth_type = $data->getValue('owsclient_auth_type');
            
            //owsclient_auth_username
            $owsclientRow->owsclient_auth_username = $data->getValue('owsclient_auth_username');
            
            //owsclient_auth_password
            $owsclientRow->owsclient_auth_password = $data->getValue('owsclient_auth_password');
            
            if($type == 'wms') {
                //owsclient_version
                $owsclientRow->owsclient_version = '1.1.1';
                
            } else if($type == 'wfs') {
                //owsclient_version
                $owsclientRow->owsclient_version = '1.0.0';
                
                //owsclient_update_interval
                $owsclientRow->owsclient_update_interval = 0;
            
                //owsclient_update_date
                $owsclientRow->owsclient_update_date = new Zend_Db_Expr('now()');
                
            } 
            
            $owsclientRow->id = $pkey;
            $owsclientRow->save();
            
        } 
        
        $defaultRow->id = $pkey;
        $defaultRow->order = $pkey * -1;
        $defaultRow->save();
        
        //Publish row
        $layersPublishTable = new Giscuit_Db_Table_LayerPublish();
        $publishRow = $layersPublishTable->createRow();
        $publishRow->id = $pkey;
        $publishRow->save();
        
        //Attach layer to current user
        $layersPermissionsTable = new Giscuit_Db_Table_LayerPermissions();
        $permissionsRow = $layersPermissionsTable->createRow();
        $permissionsRow->id = $pkey;
        if (! $isAllowed->direct('admin')) {
            $permissionsRow->user_id = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        } else {
            $permissionsRow->user_id = NULL;
        }
        $permissionsRow->save();
        
        //Save
        $row->save();
        
        return $pkey;
    
    }

    /**
     * Get default search scale for point layer
     * 
     * @return integer
     */
    protected function _getSearchScale ()
    {
        $config = Zend_Registry::get('config');
        
        $resArr = explode(',', $config->map->resolutions);
        $resIndex = round(count($resArr) * 0.8);
        $res = $resArr[$resIndex];
        return intval(round(Giscuit_Map_Map::getScaleFromResolution($res))) - 1;
    }

    /**
     * Pupulate with generic information the row from the $data form object
     * 
     * @param Zend_Db_Table_Row $row Row
     * @param Zend_Form $data A form object populated with request data
     * @param array $rows Foreign rows
     * @return void
     */
    protected function _populateRow ($row, $data, array $rows = array())
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        $row->active = $data->getValue('active') == 't' ? 't' : 'f';

        //parent id
        $parentId = $data->getValue('parent_id');
        
        if ($parentId != $row->parent_id) {
            if (!empty($parentId)) {
                $row->parent_id = $parentId;
            } else {
                $row->parent_id = NULL;
            }
        }
        
        //minscaledenom
        $minscaledenom = $data->getValue('minscaledenom');
        if (empty($minscaledenom)) {
            $row->minscaledenom = NULL;
        } else {
            $row->minscaledenom = $data->getValue('minscaledenom');
        }
        
        //maxscaledenom
        $maxscaledenom = $data->getValue('maxscaledenom');
        if (empty($maxscaledenom)) {
            $row->maxscaledenom = NULL;
        } else {
            $row->maxscaledenom = $data->getValue('maxscaledenom');
        }
        
        $row->resource = $data->getValue('resource');
        $row->priv_view = $data->getValue('priv_view');
        $row->priv_editgeom = $data->getValue('priv_editgeom');
        $row->priv_editattr = $data->getValue('priv_editattr');
        $row->priv_ows = $data->getValue('priv_ows');
        
        if (! empty($rows['default'])) {
            
            if ($isAllowed->direct('admin')) {
                //view
                $rows['default']->view = $data->getValue('view') == 't' ? 't' : 'f';
                
                //select
                $rows['default']->select = $data->getValue('select') == 't' ? 't' : 'f';
                
                //label
                $rows['default']->label = $data->getValue('label') == 't' ? 't' : 'f';
            }
                        
            $rows['default']->label_size = $data->getValue('label_size');
            $rows['default']->label_color = $data->getValue('label_color');
            
            $fields = $data->getValue('label_fields');
            if (! empty($fields)) {
                $rows['default']->label_fields = implode(',', $fields);
            } else {
                $rows['default']->label_fields = '';
            }
        
        }
        
        if (! empty($rows['publish'])) {
            
            //destination
            $destination = $data->getValue('destination');
            
            if ($isAllowed->direct('admin')) {
                if (! empty($destination)) {
                    $rows['publish']->destination = implode(',', $destination);
                } else {
                    $rows['publish']->destination = NULL;
                }
            }
            
            //publish
            if(! empty($rows['publish']->destination)) {
                $rows['publish']->publish = 't';    
            } else {
                $rows['publish']->publish = 'f';
            }

            //wms_private
            $rows['publish']->wms_private = $data->getValue('wms_private') == 't' ? 't' : 'f';
            
            //wms_public
            if ($isAllowed->direct('admin')) {
                $rows['publish']->wms_public = $data->getValue('wms_public') == 't' ? 't' : 'f';
            }

            //wfs_private
            $rows['publish']->wfs_private = $data->getValue('wfs_private') == 't' ? 't' : 'f';
            
            //wfs_public
            if ($isAllowed->direct('admin')) {
                $rows['publish']->wfs_public = $data->getValue('wfs_public') == 't' ? 't' : 'f';
            }
            
            //wcs_private
            $rows['publish']->wcs_private = $data->getValue('wcs_private') == 't' ? 't' : 'f';
            
            //wcs_public
            $rows['publish']->wcs_public = $data->getValue('wcs_public') == 't' ? 't' : 'f';
            
            //wcs_formats
            if($data->getValue('wcs_formats')) {
                $rows['publish']->wcs_formats = implode(',', $data->getValue('wcs_formats'));
            } else {
                $rows['publish']->wcs_formats = NULL;
            }
            
            //format
            $rows['publish']->format =  $data->getValue('format');
            
            //selected
            $rows['publish']->selected = $data->getValue('selected') == 't' ? 't' : 'f';
            
        }
        
        if (! empty($rows['owsclient'])) {
            if(empty($rows['owsclient']->owsclient_update_date)) { 
                //owsclient_uri
                $rows['owsclient']->owsclient_uri = $data->getValue('owsclient_uri');
            
                //owsclient_layers
                $rows['owsclient']->owsclient_layers = $data->getValue('owsclient_layers');
                
                //owsclient_version
                $rows['owsclient']->owsclient_version = $data->getValue('owsclient_version');
            } else {
                //owsclient_update_interval
                $rows['owsclient']->owsclient_update_interval = $data->getValue('owsclient_update_interval');    
            }
            
            //owsclient_image_format
            $rows['owsclient']->owsclient_image_format = $data->getValue('owsclient_image_format');
            
            //owsclient_info_format
            $rows['owsclient']->owsclient_info_format = $data->getValue('owsclient_info_format');
            
            //owsclient_auth_username
            $rows['owsclient']->owsclient_auth_username = $data->getValue('owsclient_auth_username');
            
            //owsclient_auth_password
            $rows['owsclient']->owsclient_auth_password = $data->getValue('owsclient_auth_password');
            
            //owsclient_auth_type
            $rows['owsclient']->owsclient_auth_type = $data->getValue('owsclient_auth_type');
        }
        
        if (! empty($rows['routing'])) {
            //active
            $rows['routing']->active = $data->getValue('routing_active') == 't' ? 't' : 'f';
            
            //source
            $rows['routing']->source = $data->getValue('routing_source');
            
            //reverse_source
            $rows['routing']->reverse_source = $data->getValue('routing_reverse_source');
            
            //directed
            $rows['routing']->directed = $data->getValue('routing_directed') == 't' ? 't' : 'f';
            
            //reverse_cost
            $rows['routing']->reverse_cost = $data->getValue('routing_reverse_cost') == 't' ? 't' : 'f';

            //algorithm
            $rows['routing']->algorithm = implode(',', $data->getValue('routing_algorithm'));
        }
        
        //label_font
        $row->label_font = $data->getValue('label_font');

        //label_buffer
        $row->label_buffer = $data->getValue('label_buffer');
        
        //label_separator
        $row->label_separator = $data->getValue('label_separator');
        
        //label_position
        $row->label_position = $data->getValue('label_position');
        
        //label_angle
        $row->label_angle = $data->getValue('label_angle');
        
        //label_backgroundcolor
        $row->label_backgroundcolor = $data->getValue('label_backgroundcolor');
        
        //label_minfeaturesize
        $label_minfeaturesize = $data->getValue('label_minfeaturesize');
        if (empty($label_minfeaturesize)) {
            $row->label_minfeaturesize = NULL;
        } else {
            $row->label_minfeaturesize = $label_minfeaturesize;
        }
        
        //label_repeatdistance
        $label_repeatdistance = $data->getValue('label_repeatdistance');
        if (empty($label_repeatdistance)) {
            $row->label_repeatdistance = NULL;
        } else {
            $row->label_repeatdistance = $label_repeatdistance;
        }
        
        //label_align
        $row->label_align = $data->getValue('label_align');
        
        //label_maxlength
        $label_maxlength = $data->getValue('label_maxlength');
        if (empty($label_maxlength)) {
            $row->label_maxlength = NULL;
        } else {
            $row->label_maxlength = $label_maxlength;
        }
                
        //label_priority
        $label_priority = $data->getValue('label_priority');
        if (empty($label_priority)) {
            $row->label_priority = NULL;
        } else {
            $row->label_priority = $label_priority;
        }
        
        //label_wrap
        $specialchars = array("&amp;", "&quot;", "&#039;", "&lt;", "&gt;");
        $row->label_wrap = str_replace($specialchars, "", $data->getValue('label_wrap'));
        
        //Filter
        if ($isAllowed->direct('admin')) {
            $row->filter = $data->getValue('filter');
        }
        
        //Opacity
        $row->opacity = $data->getValue('opacity');
        
        //Edit maxscale
        $edit_maxscale = $data->getValue('edit_maxscale');
        if (empty($edit_maxscale)) {
            $row->edit_maxscale = NULL;
        } else {
            $row->edit_maxscale = $edit_maxscale;
        }
        
        //Edit maxscale
        $snap_maxscale = $data->getValue('snap_maxscale');
        if (empty($snap_maxscale)) {
            $row->snap_maxscale = NULL;
        } else {
            $row->snap_maxscale = $snap_maxscale;
        }
        
        //Tolerance
        $tolerance = $data->getValue('tolerance');
        if (empty($tolerance)) {
            $row->tolerance = NULL;
        } else {
            $row->tolerance = $tolerance;
        }
        
        //Search scale
        $search_scale = $data->getValue('search_scale');
        if (empty($search_scale)) {
            $row->search_scale = NULL;
        } else {
            $row->search_scale = $search_scale;
        }
        
        //Offsite
        $offsite = $data->getValue('offsite');
        if (empty($offsite)) {
            $row->offsite = NULL;
        } else {
            $row->offsite = $offsite;
        }
        
        //Default field
        $row->default_field = $data->getValue('default_field');
        
        //Data
        $row->data = $data->getValue('data');
        
        //Tileindex
        $row->tileindex = $data->getValue('tileindex');
        
        //Tileitem
        $row->tileitem = $data->getValue('tileitem');
        
        //Srid
        $srid = $data->getValue('srid');
        if (empty($srid)) {
            $row->srid = NULL;
        } else {
            $row->srid = $srid;
        }
        
        //Raster query
        $row->query = $data->getValue('query') == 't' ? 't' : 'f';
        
        //Javascript is base layer
        $row->js_isbase = $data->getValue('js_isbase') == 't' ? 't' : 'f';
        
        //Javascript file
        $row->js_file = $data->getValue('js_file');
        
        //Class label active
        $row->classlabel_active = $data->getValue('classlabel_active') == 't' ? 't' : 'f';

        Giscuit_Map_Map::clearLink();
    }
    
    /**
     * Get editable tables for the current user
     * 
     * @param array $options Available options
     * @return Zend_Db_Table
     */
    public function getEditTables(array $options = array()) 
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $tables = array();
        
        $mapOptions = array_merge($options, array(
            'whereActive' => 't' , 
            'withLayerSettings' => true ,
            'withFields' => true ,
            'withAcl' => true
        ));
        
        if(isset($mapOptions['whereHistory'])) {
            unset($mapOptions['whereHistory']);    
        }
        
        $rows = $this->getItems($mapOptions);    
        
        foreach($rows as $row) {
            
            if (
           		//Check user_id
           		$row['permissions_user_id'] == $userId ||
            		
           		//Check priv_editgeom
           		strstr(',' . $row['permissions_editgeom_ids'], ',' . $userId . ',') ||
            		
           		//Check resource
            	! empty($row['priv_editgeom']) && $isAllowed->direct($row['resource'], $row['priv_editgeom'])
            	) {
                
                $tableName = 'Giscuit_Db_Table_Layer_' . $row['class'];
                $table = new $tableName(array(
                    'name' => $row['table']
                ));
                $table->setSettings($row);

                if(isset($options['whereHistory']) && $options['whereHistory'] == true) {
                    if($table instanceof Giscuit_Db_Table_Layer_History) {
                       $tables[] = $table;      
                    }
                } else {
                   $tables[] = $table;    
                }
            }

        }
        
        return $tables;
    }

    /**
     * Get layer table by layer id
     * 
     * @param integer $id Layer id
     * @param array $options Available options
     * @return Zend_Db_Table
     */
    public static function getLayerTableById ($id, array $options = array())
    {

        $mapOptions = $options;
        $mapOptions['whereId'] = $id;
        
        $layers = new self();
        $result = array_values($layers->getItems($mapOptions));
        
        if (empty($result)) {
            throw new Giscuit_Db_Table_Exception('Invalid layer id');
        }
        
        $result = $result[0];
        
        //Create table 
        $tableName = 'Giscuit_Db_Table_Layer_' . $result['class'];
        
        if(! @class_exists($tableName)) {
            return false;
        }
        
        $tableOptions = array(
            'name' => $result['table']
        );
        
        $table = new $tableName($tableOptions);
        $table->setSettings($result);
        
        return $table;
    }

    /**
     * Get the current editable layer
     * 
     * @return array
     */
    public function getEditLayer ()
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $mapOptions = array(
            'orderDirection' => 'desc' , 
            'whereView' => 't' , 
            'whereEdit' => 't' , 
            'whereActive' => 't' , 
            'withLayerSettings' => true , 
            'withAcl' => true ,
            'withClassesAndStyles' => true ,
            'withFields' => true ,
        );
        
        $result = $this->getItems($mapOptions);
        
        $result = array_values($result);
        
        $returnArr = array(
            'editgeom' => false , 
            'editattr' => false
        );
        
        if (! empty($result)) {
            $result = $result[0];
            
            //Check user_id
            if($result['permissions_user_id'] == $userId) {
            	$returnArr['editgeom'] = true;
            	
            //Check priv_editgeom
            } else if(strstr(',' . $result['permissions_editgeom_ids'], ',' . $userId . ',')) {
            	$returnArr['editgeom'] = true;
            	
            //Check resource
            } else if (! empty($result['priv_editgeom']) && $isAllowed->direct($result['resource'], $result['priv_editgeom'])) {
                $returnArr['editgeom'] = true;
            } else if (! empty($result['priv_editattr']) && $isAllowed->direct($result['resource'], $result['priv_editattr'])) {
                $returnArr['editattr'] = true;
            }
            
            $returnArr['settings'] = $result;
        }
        
        return $returnArr;
    
    }
    
    /**
     * Fetch ows client data
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options
     * @return void
     */
    public function withOwsclient ($select, array $options = array())
    {
        $fields = array(
            'owsclient_uri' => 'layer_owsclient.owsclient_uri' , 
            'owsclient_image_format' => 'layer_owsclient.owsclient_image_format' , 
            'owsclient_info_format' => 'layer_owsclient.owsclient_info_format' ,
            'owsclient_layers' => 'layer_owsclient.owsclient_layers' ,
            'owsclient_auth_username' => 'layer_owsclient.owsclient_auth_username' ,
            'owsclient_auth_password' => 'layer_owsclient.owsclient_auth_password' ,
            'owsclient_auth_type' => 'layer_owsclient.owsclient_auth_type' ,
            'owsclient_version' => 'layer_owsclient.owsclient_version',
            'owsclient_update_interval' => 'layer_owsclient.owsclient_update_interval',
            'owsclient_update_date' => new Zend_Db_Expr("extract(epoch from layer_owsclient.owsclient_update_date)")
        );
       
        $select->joinLeft(array(
            'layer_owsclient' => $this->_schema . '.layer_owsclient'
        ), "{$this->_name}.id = layer_owsclient.id AND {$this->_name}.class <> ''", $fields);
    
    }
    
    /**
     * Fetch rows owned by current user
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>withCurrUserOwnLayer bool - Fetch rows owned by current user</li></ul>
     * @return void
     */
    public function withCurrUserOwnLayer ($select, array $options = array())
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');

        if ($isAllowed->direct('admin') || empty($options['withCurrUserOwnLayer'])) {
            return;
        }
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
    
        $select->where("layer_permissions.user_id = ?", $userId);
    }
    
    /**
     * Fetch permission data
     *
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereAdminLayers bool - Fetch rows that belong to admin role</li>
     * <li>whereUserLayers bool - Fetch rows that DO NOT belong to admin role</li></ul>
     * @return void
     */
    public function withPermissions ($select, array $options = array())
    {
    	$fields = array(
			'permissions_user_id' => 'layer_permissions.user_id' ,
    		'permissions_view_ids' => 'layer_permissions.view_ids' ,
    		'permissions_editgeom_ids' => 'layer_permissions.editgeom_ids' ,
    		'permissions_ows_ids' => 'layer_permissions.ows_ids' ,
    	);
    	 
    	$select->joinLeft(
    		array(
    			'layer_permissions' => $this->_schema . '.layer_permissions'
    		), 
    		"{$this->_name}.id = layer_permissions.id", 
    		$fields
    	);
    	
    	if (isset($options['whereAdminLayers']) && $options['whereAdminLayers'] == true) {
    	    $select->where('"layer_permissions"."user_id" IS NULL');
    	} else if (isset($options['whereUserLayers']) && $options['whereUserLayers'] == true) {
    	    $select->where('"layer_permissions"."user_id" IS NOT NULL');
    	}
    }
    
    /**
     * Check if layer belongs to current user
     * 
     * @param integer $id Layer id
     * @return void
     */
    public function checkCurrUserOwnLayer($id)
    {
        $rows = $this->getItems(array(
        	'whereId' => $id,
                'withGroups' => true
        ));
        if(count($rows) == 0) {
            throw new Exception("Invalid layer id ({$id})");
        }
    }
    
    /**
     * Get all containers of a specific layer id
     * 
     * @param integer $id Layer id
     * @return array
     */
    public static function getContainers ($id)
    {
        $result = array();
        $result[0] = '/';
        
        $layersTable = new Giscuit_Db_Table_Layers();
        
        $rows = $layersTable->getItems(array(
            'orderDirection' => 'asc' , 
            'withGroups' => true
        ));
        
        foreach ($rows as $row) {
            if ($row['id'] == $id) {
                $layerPath = $row['path'];
            }
        }

        foreach ($rows as $row) {
            if ($layerPath != substr($row['path'], 0, strlen($layerPath)) && empty($row['class'])) {
                $path = $row['path'];
                $path = trim($path, '/');
                $path = explode('/', $path);
                foreach ($path as $key => $value) {
                    $path[$key] = $rows[$path[$key]]['name'];
                }
                $row['name'] = implode('/', $path);
                $result[$row['id']] = '/' . $row['name'];
            }
        }
        
        return $result;
    }
    
    /**
     * Get table XML definition
     * 
     * @param integer $id Layer id
     * @return string
     */
    public static function getTableDefinition ($id)
    {
        $result = new Zend_Config(array(), true);
        $result->elements = array();
        
        $mapOptions = array(
            'orderDirection' => 'desc' , 
            'withFields' => true
        );
        
        $table = Giscuit_Db_Table_Layers::getLayerTableById($id, $mapOptions);
        $pkeyName = $table->getPrimaryKey(array(
            'name' => true
        ));
        
        $info = $table->info('metadata');
        
        foreach ($info as $key => $col) {
            $name = $col['COLUMN_NAME'];
            
            if ($name == $pkeyName || $col['DATA_TYPE'] == 'geometry') {
                continue;
            }

            $result->elements->$name = Giscuit_Db_Table_Layers::getColumnDefinition($col);
        }
                
        if(count($result->elements) == 0) {
            unset($result->elements);
        }
                
        $writer = new Zend_Config_Writer_Xml(array(
            'config' => $result
        ));
        
        return $writer->render();
    
    }
    
    /**
     * Get column definition
     * 
     * @param array $col Columns
     * @return mixed
     */
    public static function getColumnDefinition (array $col = array())
    {
        //Types
        $stringTypes = array(
            'varchar' , 
            'char' , 
            'text'
        );
        
        $floatTypes = array(
            'float4' , 
            'float8' , 
            'decimal' , 
            'numeric'
        );
        
        $dateTypes = array(
            'timestamp' , 
            'timestamptz' , 
            'date'
        );
        
        $name = $col['COLUMN_NAME'];
        
        $result = new Zend_Config(array(), true);
        $result->elements = array();
        
        $result->elements->$name = array();
        $el = $result->elements->$name;
        
        $el->type = 'text';
        $el->options = array();
        
        //Label
        $el->options->label = $name;
        
        //Required
        if ($col['NULLABLE'] == false) {
            $el->options->required = true;
        }
        
        //Validators and filters
        $el->options->validators = array();
        $validators = $el->options->validators;
        
        $el->options->filters = array();
        $filters = $el->options->filters;
        
        if (substr($col['DATA_TYPE'], 0, 3) == 'int') {
            $length = (pow(2, 8 * (integer) $col['LENGTH']) / 2);
            if (! is_int($length)) {
                $length = PHP_INT_MAX;
            }
            $validators->digits = array();
            $validators->digits->validator = "Int";
            $validators->digits->breakChainOnFailure = 1;
            
            $validators->between = array();
            $validators->between->validator = "Between";
            $validators->between->options = array();
            $validators->between->options->min = ($length - 1) * - 1;
            $validators->between->options->max = ($length - 1);
        
        } elseif (in_array($col['DATA_TYPE'], $stringTypes)) {
            
            if ($col['LENGTH'] > 0) {
                $validators->strlen = array();
                $validators->strlen->validator = "StringLength";
                $validators->strlen->options = array();
                $validators->strlen->options->max = $col['LENGTH'];
            }
            
            if ($col['DATA_TYPE'] == 'text') {
            }
        
        } elseif (in_array($col['DATA_TYPE'], $floatTypes)) {
            
            $validators->float = array();
            $validators->float->validator = "Float";
        
        } elseif (in_array($col['DATA_TYPE'], $dateTypes)) {
            $el->type = 'date';
        }
        
        if (count($validators) == 0) {
            unset($el->options->validators);
        }
        
        if (count($filters) == 0) {
            unset($el->options->filters);
        }
        
        return $el;
    }

}
