<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Abstract.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Generic form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Form_Abstract extends Zend_Form
{

    /**
     * Checkbox validator
     * 
     * @var array
     */
    public $checkboxValidator = array(
        array(
            'InArray' , 
            false , 
            array(
                array(
                    't' , 
                    'f'
                )
            )
        )
    );
    
    /**
     * Constructor
     *
     * @param mixed $options 
     * @return void
     */
    public function __construct ($options = array())
    {
        //Filters
        $this->addElementPrefixPath('Giscuit_Filter', 'Giscuit/Filter/', 'filter');
        
        //Validators
        $this->addElementPrefixPath('Giscuit_Validate', 'Giscuit/Validate', 'validate');
        
        //Decorators
        $this->addElementPrefixPath('Giscuit_Form_Decorator', 'Giscuit/Form/Decorator', 'decorator');
        
        //Elements
        $this->addPrefixPath('Giscuit_Form_Element', GT_LIBRARY_PATH . '/Giscuit/Form/Element/', 'element');
        
        parent::__construct($options);
    
    }
    
    public function applyJqmFixes()
    {
        $this->multiFix();
        $this->checkboxFix();
    }
    
    public function multiFix() 
    {
        foreach ($this->getElements() as $element) {
            
            if($element instanceof Zend_Form_Element_Multi) {
                $element->setSeparator('');
                $element->setDecorators(array(
                    'ViewHelper' , 
                    'Errors' , 
                    array(
                        'HtmlTag' , 
                        array(
                            'tag' => 'dd'
                        )
                    ) , 
                    array(
                        'HtmlTag' , 
                        array(
                            'tag' => 'fieldsset' , 
                            'data-role' => 'controlgroup'
                        )
                    ) , 
                    array(
                        'Label' , 
                        array(
                            'tag' => 'dt'
                        )
                    )
                ));
            }
        }
    }
    
    public function checkboxFix()
    {
        foreach ($this->getElements() as $element) {
            if($element instanceof Zend_Form_Element_Checkbox) {
                $arr = $element->getDecorators();
                $arr['Zend_Form_Decorator_ViewHelper']->setHelper('formJqmCheckbox');
            }
        }    
    }

}