<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayersEditContainer.php 2891 2012-07-31 14:39:01Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Edit container form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_LayersEditContainer extends Giscuit_Form_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        //Submit
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' , 
            'id' => 'layersSubmit'
        ));
        
        //Form view helper
        $this->addDecorators(array(
            array(
                'viewScript' , 
                array(
                    'viewScript' => 'forms' . DIR_SEP . 'layersEditContainer' . GT_SCRIPT_EXT
                )
            )
        ));
    
    }

    /**
     * Add general section
     * 
     * @param integer $id Layer id
     * @return void
     */
    public function addGeneralSection ($id)
    {
        //Active
        $this->addElement('checkbox', 'active', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        $containers = Giscuit_Db_Table_Layers::getContainers($id);
        
        $this->addElement('select', 'parent_id', array(
            'required' => true , 
            'label' => 'container' , 
            'multiOptions' => $containers 
        ));
     
    }

    /**
     * Add name section
     * 
     * @return void
     */
    public function addNameSection ()
    {
        $languages = Zend_Registry::get('config')->language->toArray();
        foreach ($languages as $key => $language) {
            $this->addElement('text', 'name_' . $key, array(
                'filters' => array(
            		'HtmlSpecialChars'
            	) , 
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
					),
                ) , 
                'required' => false , 
                'label' => $language
            ));
        }
    }
    
    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate (array $values)
    {
        $result = $this->setDefaults($values);
        
        return $result;
       
    }
}