<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayersEditJavascript.php 2891 2012-07-31 14:39:01Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Edit javascript layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_LayersEditjavascript extends Giscuit_Form_Abstract
{
    /**
     * Rules table
     * 
     * @var Giscuit_Db_Table_Rules
     */
    protected $_rulesTable;

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        //Submit
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' , 
            'id' => 'layersSubmit'
        ));
        
        //Form view helper
        $this->addDecorators(array(
            array(
                'viewScript' , 
                array(
                    'viewScript' => 'forms' . DIR_SEP . 'layersEditJavascript' . GT_SCRIPT_EXT
                )
            )
        ));
    
    }

    /**
     * Add general section
     * 
     * @param integer $id Layer id
     * @return void
     */
    public function addGeneralSection ($id)
    {
        //Active
        $this->addElement('checkbox', 'active', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        $containers = Giscuit_Db_Table_Layers::getContainers($id);
        $this->addElement('select', 'parent_id', array(
            'required' => true , 
            'label' => 'container' , 
            'multiOptions' => $containers 
        ));
        
        //File
        $this->addElement('text', 'js_file', array(
            'required' => false , 
            'label' => 'js_file'
        ));
        
        //Is base layer
        $this->addElement('checkbox', 'js_isbase', array(
            'validators' => $this->checkboxValidator ,                 
            'required' => true , 
            'label' => 'js_isbase' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Minscaledenom
        $this->addElement('text', 'minscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'minscaledenom'
        ));
        
        //Maxscaledenom
        $this->addElement('text', 'maxscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'maxscaledenom'
        ));
        
        //Opacity
        $this->addElement('text', 'opacity', array(
            'validators' => array(
                'Digits' , 
                array(
                    'Between' , 
                    false , 
                    array(
                        0 , 
                        100
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'opacity'
        ));
       
        //Tolerance
        $this->addElement('text', 'tolerance', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'tolerance'
        ));
                
        //Raster query
        $this->addElement('checkbox', 'query', array(
            'validators' => $this->checkboxValidator ,                 
            'required' => true , 
            'label' => 'query' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
     
    }

    /**
     * Add name section
     * 
     * @return void
     */
    public function addNameSection ()
    {
        $languages = Zend_Registry::get('config')->language->toArray();
        foreach ($languages as $key => $language) {
            $this->addElement('text', 'name_' . $key, array(
                'filters' => array(
            		'HtmlSpecialChars'
            	) ,
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
                    ),
                ) , 
                'required' => false , 
                'label' => $language
            ));
        }
    }

    /**
     * Add default settings section
     * 
     * @return void
     */
    public function addSettingsSection ()
    {
        //View
        $this->addElement('checkbox', 'view', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'view' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Select
        $this->addElement('checkbox', 'select', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'information' ,
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    }
    
    /**
     * Add permissions section
     * 
     * @return void
     */
    public function addPermissionsSection ()
    {
        //Resource
        $this->_rulesTable = new Giscuit_Db_Table_Rules();
        $resources = $this->_rulesTable->getItems(array(
            'getResourceList' => 'layer'
        ));
        $resources = $resources->toArray();
        $resourcesArr = array(
            '' => ''
        );
        foreach ($resources as $resource) {
            $resourcesArr[$resource['resource']] = $resource['resource'];
        }
        $this->addElement('select', 'resource', array(
            'required' => false , 
            'label' => 'resource' , 
            'attribs' => array(
                'id' => 'menuResource'
            ) , 
            'multiOptions' => $resourcesArr
        ));
        
        //View privilege
        $this->addElement('select', 'priv_view', array(
            'required' => false , 
            'label' => 'priv_view' , 
            'attribs' => array(
                'id' => 'menuViewPrivilege'
            )
        ));

    }
    
    /**
     * Add publish section
     * 
     * @return void
     */
    public function addPublishSection()
    {
        //Selected
        $this->addElement('checkbox', 'selected', array(
            'validators' => $this->checkboxValidator , 
            'value' => 'f' , 
            'required' => true , 
            'label' => 'publish_selected' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Destination
        $destinationArr = array(
            'overlay' => 'overlay' ,
            'base' => 'base' 
        );
        
        $this->addElement('select', 'destination', array(
            'required' => false , 
            'label' => 'publish_destination' , 
            'isArray' => true , 
            'multiOptions' => $destinationArr
        ));
        
    }
    
    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate (array $values)
    {
        //Privledges
        if (! empty($values['resource'])) {
            $privileges = $this->_rulesTable->getItems(array(
                'getPrivilegeList' => $values['resource']
            ));
            $privilegesArr = array(
                '' => ''
            );
            foreach ($privileges as $privilege) {
                $privilegesArr[$privilege['privilege']] = $privilege['privilege'];
            }
            
            $this->getElement('priv_view')->setMultiOptions($privilegesArr);
        }
        
        $result = $this->setDefaults($values);
        
        return $result;
       
    }
}