<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Menu.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Menu form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_Menu extends Giscuit_Form_Abstract
{
    /**
     * Rules table
     * 
     * @var Giscuit_Db_Table_Rules
     */
    protected $_rulesTable;

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        
        $languages = Zend_Registry::get('config')->language->toArray();
        
        foreach ($languages as $key => $language) {
            
            $this->addElement('text', 'title_' . $key, array(
                'filters' => array(
                    'StringTrim'
                ) , 
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        2
                    )
                ) , 
                'required' => false , 
                'label' => 'title'
            ));
            
            $this->addElement('text', 'tags_' . $key, array(
                'filters' => array(
                    'StringTrim'
                ) , 
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
                    )
                ) , 
                'required' => false , 
                'label' => 'tags'
            ));
            
            $this->addElement('textarea', 'text_' . $key, array(
                'filters' => array(
                    'StringTrim'
                ) , 
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
                    )
                ) , 
                'required' => false , 
                'label' => 'text' , 
                'rows' => 5
            ));
        }
        
        $this->addElement('checkbox', 'active', array(
            'validators' =>  $this->checkboxValidator ,
            'required' => true , 
            'label' => 'active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        $this->addElement('select', 'type', array(
            'filters' => array(
                'Digits'
            ) , 
            'validators' => array() , 
            
            'required' => true , 
            'label' => 'type' , 
            'multiOptions' => array(
                1 => 'container' , 
                2 => 'text' , 
                3 => 'link'
            )
        ));
        
        //TODO: Add url validator
        $this->addElement('text', 'url', array(
            'filters' => array(
                'StringTrim'
            ) ,
            'validators' => array(
            ) , 
            'required' => false , 
            'label' => 'url'
        ));
        
        //Resource
        $this->_rulesTable = new Giscuit_Db_Table_Rules();
        $resources = $this->_rulesTable->getItems(array(
            'getResourceList' => 'menu'
        ));
        $resources = $resources->toArray();
        $resourcesArr = array(
            '' => ''
        );
        foreach ($resources as $resource) {
            $resourcesArr[$resource['resource']] = $resource['resource'];
        }
        $this->addElement('select', 'resource', array(
            'required' => false , 
            'label' => 'resource' , 
            'attribs' => array(
                'id' => 'menuResource'
            ) ,
            'multiOptions' => $resourcesArr
        ));

        //Privilege
        $this->addElement('select', 'privilege', array(
            'required' => false , 
            'label' => 'privilege' ,
            'attribs' => array(
                'id' => 'menuPrivilege'
            ) ,
        ));
        
        //Open in new window
        $this->addElement('checkbox', 'target', array(
            'validators' =>  $this->checkboxValidator ,
            'required' => true , 
            'label' => 'openInNewWindow' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
       
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' , 
        ));
        
        $this->addDecorators(array(
            array(
                'viewScript' , 
                array(
                    'viewScript' => 'forms' . DIR_SEP . 'menu' . GT_SCRIPT_EXT
                )
            )
        ));

    }

    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate (array $values)
    {
        
        if (! empty($values['resource'])) {
            $privileges = $this->_rulesTable->getItems(array(
                'getPrivilegeList' => $values['resource']
            ));
            $privilegesArr = array(
                '' => ''
            );
            foreach ($privileges as $privilege) {
                $privilegesArr[$privilege['privilege']] = $privilege['privilege'];
            }
            
            $privilegeElement = $this->getElement('privilege');
            $privilegeElement->setMultiOptions($privilegesArr);
        }
        
        $result = $this->setDefaults($values);
        
        return $result;
    }
}