<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Copyright.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Giscuit_Map_Layer_Abstract
 */
require_once 'Giscuit/Map/Layer/Abstract.php';

/**
 * Class for copyright layer object
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Map_Layer_Copyright extends Giscuit_Map_Layer_Abstract
{
    /**
     * Text
     * 
     * @var string
     */
    private $_text = NULL;
    
     /**
     * Class constructor
     *
     * @param mixed $map Map object
     * @param array $settings Settings
     * @return void
     */
    public function __construct ($map, $settings)
    {
        $this->_text = $settings['watermark'];
        
        parent::__construct($map, $settings);
            
        //$this->set('name', 'copyright');
        $this->set('status', MS_DEFAULT);
        $this->set('type', MS_LAYER_ANNOTATION);
        $this->set('sizeunits', MS_PIXELS);
        $this->set('transform', MS_FALSE);
        
        $class = new Giscuit_Map_Class($this, 'copyright_default');
        
        //Label
        $label = $class->label;
        $label->outlinecolor->setrgb(100, 100, 100);
        $label->set('type', MS_TRUETYPE);
        $label->set('partials', MS_FALSE);
        $label->set('force', MS_TRUE);
        
        //FIXME: Hardcoded font names are used
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            $label->set('font', 'arial');
        } else {
            $label->set('font', 'FreeSans');    
        }
                
        $label->set('size', 8);
        $label->color->setrgb(200, 200, 200);
        
        //Get text
        $date = new Zend_Date();
        $auth = Zend_Auth::getInstance();
        if ($auth->hasIdentity() && isset($auth->getIdentity()->fname) && isset($auth->getIdentity()->lname)) {
            $name = $auth->getIdentity()->fname . ' ' . $auth->getIdentity()->lname;
        } else {
            $name = '';
        }
        
        if($this->_text === NULL) {
            $text = $name . "\n" . $date->toString(GT_DATE_STYLE_PHP);
        } else {
            $text = $this->_text;
        }
        
        $mapSize = $map->getSize();
        $w = $mapSize['width'];
        $h = $mapSize['height'];
                
        //Add feature
        $this->addWatermark($w * 0.25, $h * 0.25, $text);
        $this->addWatermark($w * 0.75, $h * 0.25, $text);
        $this->addWatermark($w * 0.25, $h * 0.75, $text);
        $this->addWatermark($w * 0.75, $h * 0.75, $text);
    }
    
    /**
     * Add text
     * 
     * @param $x X pixel coordinate
     * @param $y Y pixel coordinate
     * @param $text Text
     * @return void
     */
    public function addWatermark($x, $y, $text)
    {
        $point = ms_newPointObj();
        $point->setXY($x, $y);

        $line = ms_newLineObj();
        $line->add($point);

        $shape = ms_newShapeObj(MS_SHAPE_POINT);
        $shape->set('text', $text);
        $shape->add($line);
        
        $this->addFeature($shape);    
    }
 
}