<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Navigation.php 1346 2011-05-24 19:28:03Z ue $
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for hierarchic navigation menu
 * 
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Menu_Mobile_Offline extends Giscuit_Menu
{
    /**
     * 
     * 
     * @var string
     */
    //protected $_baseUrl;

    /**
     * Build a menu using html lists template
     * 
     * @param array $data
     * @return string
     */
    public function buildMenu (array &$data)
    {
        if (empty($data)) {
            return '';
        }
        
        if(! $data[0]['parent_id']) {
            $html = '<ul data-role="listview" data-split-icon="delete" id="gtOfflineMenu">';
        } else {
            $html = "<ul data-split-icon=\"delete\">";
        }
        
        foreach ($data as $row) {
            $html .= $this->_buildRow($row);
            if (! empty($row['child'])) {
                $html .= $this->buildMenu($row['child']) . "</li>";
            } else {
                $html .= "</li>\n";
            }
        }
        
        $html .= "</ul>";
        
        return $html;
    }

    /**
     * Build menu element
     * 
     * @param array $row Element
     * @return string
     */
    protected function _buildRow ($row)
    {
        $count = $delete = $nameClass = '';

        //
        if(! empty($row['class'])) {
            $nameClass = 'gtOfflineMenuName ';
            $layerClassName = 'Giscuit_Map_Layer_' . $row['class'];
                        
            $count = "<span class=\"ui-li-count\">0</span>";
            $delete = "<a  class=\"gtOfflineMenuDelete\"></a>";
        }

        $text = "<li><a data-layer-id=\"{$row['id']}\" class=\"{$nameClass}\">{$row['name']}{$count}</a>" .
            $delete;

        //Disable javascript layers
        if(! empty($row['class'])) {
            $layerClassName = 'Giscuit_Map_Layer_' . $row['class'];
            
            if (class_exists('Giscuit_Map_Layer_Vector') 
                && class_exists($layerClassName) 
                && is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Javascript') ) {

                $text = '<li style="display: none;">';
            }
            
            if($row['class'] == 'Rastercut') {
                $text = '<li style="display: none;">';
            }
        }
            
        return $text;
    }
    
    /**
     * Formats menu elements
     * 
     * @param array $child Element of the menu
     * @return array
     */
    protected function _parse ($child)
    {
        $child['name'] = htmlspecialchars($child['name'], ENT_QUOTES);
        
        return $child;
    }

}