<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Navigation.php 1346 2011-05-24 19:28:03Z ue $
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for hierarchic navigation menu
 * 
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Menu_Mobile_Privatelayers extends Giscuit_Menu
{
    /**
     * 
     * 
     * @var string
     */
    protected $_baseUrl;

    /**
     * Build a menu using html lists template
     * 
     * @param array $data
     * @return string
     */
    public function buildMenu (array &$data)
    {
        if (empty($data)) {
            return '';
        }
        
        if(! $data[0]['parent_id']) {
            $html = '<ul data-role="listview" data-split-icon="gear" id="gtLayersMenu">';
        } else {
            $html = "<ul data-split-icon=\"gear\">";
        }
        
        foreach ($data as $row) {
            $html .= $this->_buildRow($row);
            if (! empty($row['child'])) {
                $html .= $this->buildMenu($row['child']) . "</li>";
            } else {
                $html .= "</li>\n";
            }
        }
        
        $html .= "</ul>";
        
        return $html;
    }

    /**
     * Build menu element
     * 
     * @param array $row Element
     * @return string
     */
    protected function _buildRow ($row)
    {
        $settings = $settingsClass = $nameClass = $liClass = '';

        //
        if(! empty($row['class'])) {
            $nameClass = 'gtLayersMenuName ';
            $settingsClass = "gtLayersMenuSettings";
            $layerClassName = 'Giscuit_Map_Layer_' . $row['class'];
            
            if(isset($row['view']) && $row['view'] == true) {
                $settingsClass .= " ui-btn-active";
            }
            
            if (class_exists('Giscuit_Map_Layer_Vector') 
                && class_exists($layerClassName) 
                && !is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Vector') ) {

                $settingsClass .= " info-only";
            }

            $settings = "<a  class=\"$settingsClass\"></a>";
        } else {
            $nameClass = 'gtLayersMenuContainer ';
        }

        //
        if(isset($row['view']) && $row['view'] == true) {
            $liClass = 'ui-btn-active ';
        }
        
        $text = "<li class=\"$liClass\">" . 
            "<a data-layer-id=\"{$row['id']}\" class=\"{$nameClass}\">{$row['name']}</a>" .
            $settings;
        
        return $text;
    }
    
    /**
     * Formats menu elements
     * 
     * @param array $child Element of the menu
     * @return array
     */
    protected function _parse ($child)
    {
        $child['name'] = htmlspecialchars($child['name'], ENT_QUOTES);
        
        return $child;
    }

}