<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Navigation.php 1346 2011-05-24 19:28:03Z ue $
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for hierarchic navigation menu
 * 
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Menu_Mobile_Publiclayers extends Giscuit_Menu
{
    /**
     * 
     * 
     * @var string
     */
    protected $_baseUrl;

    /**
     * Build a menu using html lists template
     * 
     * @param array $data
     * @return string
     */
    public function buildMenu (array &$data)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        if (empty($data)) {
            return '';
        }
        
        if(! $data[0]['parent_id']) {
            $html = '<ul data-role="listview" id="gtLayersMenu">';
        } else {
            $html = "<ul>";
        }
        
        
        foreach ($data as $row) {
            $match = //Container
                empty($row['class']) ||
                
                //Overlay
                (
                    isset($row['destination']) && 
                    (in_array('overlay', $row['destination']))
                ) ||
                
                //Merged overlay
                (
                    ! isset($row['destination']) &&
                    $row['parent_id'] != -1 &&
                    substr($row['id'], 0, 5) == '_base'
                );
            
            if($match) {
                $html .= $this->_buildRow($row, false);
            }
            
            
            if (! empty($row['child'])) {
                $html .= $this->buildMenu($row['child']) . '</li>';
            } else {
                if($match) {
                    $html .= "</li>\n";
                }
            }
        }
                
        if(! $data[0]['parent_id']) {
            $html .= '<li data-role="list-divider">' . $translate->_('baseLayers') . '</li>';

            foreach ($this->_data as $row) {
                if(
                    //Javascript
                    (isset($row['destination']) && in_array('base', $row['destination'])) ||
                    
                    //Merged base
                    (
                        ! isset($row['destination']) &&
                        $row['parent_id'] == -1 &&
                        substr($row['id'], 0, 5) == '_base'
                    )
                ) {
                    $html .= $this->_buildRow($row, true) . '</li>';
                }
            }

        }
        
        $html .= "</ul>";
                
        return $html;
    }

    /**
     * Build menu element
     * 
     * @param array $row Element
     * @return string
     */
    protected function _buildRow ($row, $base = false)
    {
        $dataIcon = $nameClass = $liClass = '';

        //
        if(! (isset($row['child']) && count($row['child'])) > 0) {
            $dataIcon = 'data-icon="false"';
        }
        
        //
        if(! empty($row['class'])) {
            if($base == false) {
                $nameClass = 'class="gtLayersMenuName"';
            } else {
                $nameClass = 'class="gtLayersMenuName gtLayersMenuBase"';
            }
        } else {
            $nameClass = 'class="gtLayersMenuContainer"';
        }
        
        //
        $visibleLayers = Zend_Controller_Front::getInstance()->getRequest()->getCookie('gtLayers');
        
        if($visibleLayers !== NULL) {
            $visibleLayers = explode(',', $visibleLayers);
            if(! empty($row['class']) && in_array($row['class'], $visibleLayers)) {
                $liClass = 'ui-btn-active ';                    
            }
        } else {
            if(isset($row['selected']) && $row['selected'] == 't') {
                $liClass = 'ui-btn-active ';
            }
        }
        
        $text = "<li $dataIcon class=\"$liClass\">" . 
            "<a data-layer-id=\"{$row['id']}\" $nameClass>{$row['name']}</a>";
        
        return $text;
    }
    
    /**
     * Formats menu elements
     * 
     * @param array $child Element of the menu
     * @return array
     */
    protected function _parse ($child)
    {
        $child['name'] = htmlspecialchars($child['name'], ENT_QUOTES);
        
        return $child;
    }

}