<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Classname.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class name validator
 * 
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Validate_Classname extends Zend_Validate_Abstract
{

    const CLASS_EXISTS = 'classExists';

    const INVALID_CLASS_NAME = 'invalidClassName';

    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = array(
        self::CLASS_EXISTS => "Class already exists" , 
        self::INVALID_CLASS_NAME => "Class contains invalid characters"
    );

    /**
     * Defined by Zend_Validate_Interface
     * 
     * Returns true if and only if class with name as $value exists
     * 
     * @param string $value
     * @return boolean
     */
    public function isValid ($value)
    {
        /*
        The @(suppress error operator) is used because the class gets autoladed 
        and could not be found if $values in an invalid class name
        */
        
        if (@class_exists('Giscuit_Map_Layer_' . $value)) {
            $this->_error(self::CLASS_EXISTS);
            return false;
        }
        
        if (! preg_match('/^([0-9a-zA-Z])+$/i', $value)) {
            $this->_error(self::INVALID_CLASS_NAME);
            return false;
        }
        
        return true;
    }
}