<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: TablenameAvailable.php 2182 2012-01-20 12:01:13Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Validator for checking table name availability
 * 
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Validate_TablenameAvailable extends Zend_Validate_Abstract
{

    const TABLE_NOT_AVAILABLE = 'tableNameUnavailable';

    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = array(
        self::TABLE_NOT_AVAILABLE => 'Table name unavailable'
    );

    /**
     * Defined by Zend_Validate_Interface
     *
     * Returns true if table does not exist or belongs to current user
     *
     * @param string $value 
     * @return boolean
     */
    public function isValid ($value)
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        if (! $isAllowed->direct('admin')) {
            $config = Zend_Registry::get('config');
            $geometryTablesTable = new Giscuit_Db_Table_GeometryTables();
            
            //        
            $table = $geometryTablesTable->getItems(array('whereName' => $value));
            if (count($table) != 0) {
                $usersTable = new Giscuit_Db_Table_Users();
                
                $select = $usersTable->select();
                $select->from($usersTable, array('id'));
                $select->where('id = ?', Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct());
                $select->where("tables LIKE '%{$value},%'");
                
                $result = $usersTable->fetchRow($select);
        
                if (count($result) == 0) {
                    $this->_error(self::TABLE_NOT_AVAILABLE);
                    return false;
                }
            }
        }
        return true;
    }
}