/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editgeom.js 2573 2012-04-25 16:35:06Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        if(gt.config.layout != 'desktop') {
            return;
        }
        
        gt.control.historymanage = new Giscuit.Control.Historymanage();
        gt.control.historydata = new Giscuit.Control.Historydata();
        
        gt.control.editgeom = new Giscuit.Control.Editgeom();
    }
});

/**
 * Class: Giscuit.Control.Editgeom
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Editgeom = OpenLayers.Class(Giscuit.Control, {

    button: null,
    
    controls: null,

    controlPanel: null,
    
    secondaryControlPanel: null,
    
    editLayer: null,
    
    loaded: false,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        if(! gt.pluginParams.editingAvailable) {
            return;
        }
        
        var onClick = function (type, args, item) {
            var label = item.cfg.getProperty('text');
            self.button.set('label', label);
            
            if(this.id == 'gtDatamanageDownloadControl') {
                if(gt.control.historydata.active) {
                    gt.control.historydata.deactivate();
                } else {
                    gt.control.historydata.downloadActivate();
                }
            } else if(this.id == 'gtDatamanageUploadControl') {
                gt.control.historydata.uploadActivate();
            } else if(this.id == 'gtDatamanageActivateControl') {
                gt.control.historydata.activateActivate();
            } else if(this.id == 'gtDatamanageHistoryControl') {
                gt.control.historymanage.activate();
            } else if(this.id == 'gtEditgeomControl') {
                if(self.active) {
                    self.deactivate();
                } else {
                    self.activate();
                }
            } 
        };
        
        var menu = [];
        menu.push({
            id: 'gtEditgeomControl' ,
            text: gt.map.getControlIcon(6, gt.translate._('editing')),
            onclick: {fn: onClick}
        });
        
        if(gt.pluginParams.historymanageIsAllowed) {
            menu.push({
                id: 'gtDatamanageHistoryControl' ,
                text: gt.map.getControlIcon(9, gt.translate._('history')),
                onclick: {fn: onClick}
            });
            
            menu.push({
                id: 'gtDatamanageDownloadControl' ,
                text: gt.map.getControlIcon(2, gt.translate._('download')),
                onclick: {fn: onClick}
            });
            
            menu.push({
                id: 'gtDatamanageUploadControl' ,
                text: gt.map.getControlIcon(21, gt.translate._('upload')),
                onclick: {fn: onClick}
            });
        }
        
        if(gt.pluginParams.historydataActivatesubmitIsAllowed) {
            menu.push({
                id: 'gtDatamanageActivateControl' ,
                text: gt.map.getControlIcon(0, gt.translate._('activate')),
                onclick: {fn: onClick}
            });
        }
        
        this.button = new YAHOO.widget.Button({
            type: 'menu', 
            label: menu[0].text,
            'menu': menu,
            title: gt.translate._('editing'), 
            container: 'gtMapControls',
            lazyloadmenu: false
        });  
        
    },
        
    activate: function () {
        gt.map.deactivateAll();
        
        gt.map.globalSelectControl.deactivate();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        if (activated) {
            this.enableEditing();
        }
        
        return activated;
    },
    
    deactivate: function (sendDisableRequest) {
        if(sendDisableRequest == undefined) {        
            sendDisableRequest = true;
        }
        
        gt.map.globalSelectControl.activate();
        
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
                
        if (deactivated) {
            if(this.editLayer && this.editLayer.map) {
                this.editLayer.events.un({
                    'loadend': this.loadend,
                    'loadstart': this.loadstart,
                    scope: this
                });
            }
            
            //Deactivate snap
            if (typeof(gt.snap.vector) != 'undefined') {
                gt.snap.vector.deactivate();
            }
            if(typeof(gt.layer.snap) != 'undefined') {
                gt.layer.snap.setVisibility(false);
            }
            
            if(sendDisableRequest) {
                this.disableEditing();
            } else {
                if(this.editLayer && this.editLayer.map) {
                    this.editLayer.setVisibility(false);
                }
            }
            
            if (gt.control.editattr) {
                gt.control.editattr.deactivate();
                gt.control.editattr.destroy();
                gt.control.editattr = null;
            }

            for (var obj in this.controls) {
                this.controls[obj].deactivate();
                this.controls[obj].destroy();
            }
           
            if (this.controlPanel) {
                try {
                    this.controlPanel.destroy();
                    this.controlPanel = null;
                } catch(e) {
                    debug.warn(e.message);
                }
            }
            
            if (this.secondaryControlPanel) {
                try {
                    this.secondaryControlPanel.destroy();
                    this.secondaryControlPanel = null;
                } catch(e) {
                    debug.warn(e.message);
                }
            }
            
            //Uncheck button
            if(this.button) {
                YAHOO.util.Dom.removeClass(
                    this.button.get('element'), 
                    'yui-checkbox-button-checked'
                );
            }
        }
                
        return deactivated;
    },
    
    enableEditing: function() {
        //Check button
        this.button.set("label", Giscuit.Util.getMenuItemById(this.button, 'gtEditgeomControl').cfg.getProperty('text'));
        YAHOO.util.Dom.addClass(
            this.button.get('element'),
            'yui-checkbox-button-checked'
        );
        
        var enableCallback = {
            success: function(o) {
                try {
                    var responseText = YAHOO.lang.JSON.parse(o.responseText);
                } catch(e) {
                    Giscuit.Util.parseFailure(o, e);
                    return;
                }
                
                if (responseText.replyCode != 200) {
                    Giscuit.Util.invalidReplyCode(responseText);
                    return;
                }
                
                this.initializeControls(responseText);
            } ,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
                    
        if (gt.control.layers.getEditLayer()) {
            this.editLayer = gt.layer.edit;
            var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/privatelayers/editenable/rand/" + Math.random(), enableCallback);
        } else {
            Giscuit.Util.message(gt.translate._('noEditLayerSelected'));
            this.deactivate(false);
        }
    },
    
    disableEditing: function() {
        var disableCallback = {
            success: function() {
                if(this.editLayer) {
                    this.editLayer.setVisibility(false);
                    gt.map.refresh();
                }
            },
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/privatelayers/editdisable/rand/" + Math.random(), disableCallback);
    },
    
    insertPointFirstXY: function (x, y, handler) {
        //Create feature
        var geometry = new OpenLayers.Geometry.Point(x, y);
        handler.point = new OpenLayers.Feature.Vector(geometry);
        handler.callback("create", [handler.point.geometry, handler.point]);
        handler.point.geometry.clearBounds();
        handler.layer.addFeatures([handler.point], {silent: true});
        
        handler.callback("modify", [handler.point.geometry, handler.point, false]);
        handler.point.geometry.clearBounds();
        handler.drawFeature();
        
        handler.finalize();
    },
    
    insertLineFirstXY: function (x, y, handler) {
        //Create feature
        var geometry = new OpenLayers.Geometry.Point(x, y);
        handler.point = new OpenLayers.Feature.Vector(geometry);
        handler.line = new OpenLayers.Feature.Vector(
            new OpenLayers.Geometry.LineString([handler.point.geometry])
        );
        handler.callback("create", [handler.point.geometry, handler.getSketch()]);
        handler.point.geometry.clearBounds();
        handler.layer.addFeatures([handler.line, handler.point], {silent: true});
        
        //Add point
        handler.layer.removeFeatures([handler.point]);
        handler.point = new OpenLayers.Feature.Vector(
            new OpenLayers.Geometry.Point(x, y)
        );
        handler.line.geometry.addComponent(
            handler.point.geometry, handler.line.geometry.components.length
        );
        handler.layer.addFeatures([handler.point]);
        handler.callback("point", [handler.point.geometry, handler.getGeometry()]);
        handler.callback("modify", [handler.point.geometry, handler.getSketch()]);
        handler.drawFeature();
        delete handler.redoStack;
    } ,
    
    //FIXME: No hole support 
    insertPolygonFirstXY: function (x, y, handler) {
        //Create feature
        var geometry = new OpenLayers.Geometry.Point(x, y);
        handler.point = new OpenLayers.Feature.Vector(geometry);
        handler.line = new OpenLayers.Feature.Vector(
            new OpenLayers.Geometry.LinearRing([handler.point.geometry])
        );
        handler.polygon = new OpenLayers.Feature.Vector(
            new OpenLayers.Geometry.Polygon([handler.line.geometry])
        );
        handler.callback("create", [handler.point.geometry, handler.getSketch()]);
        handler.point.geometry.clearBounds();
        handler.layer.addFeatures([handler.polygon, handler.point], {silent: true});
        
        //Add point
        handler.layer.removeFeatures([handler.point]);
        handler.point = new OpenLayers.Feature.Vector(
            new OpenLayers.Geometry.Point(x, y)
        );
        handler.line.geometry.addComponent(
            handler.point.geometry, handler.line.geometry.components.length
        );
        handler.layer.addFeatures([handler.point]);
        handler.callback("point", [handler.point.geometry, handler.getGeometry()]);
        handler.callback("modify", [handler.point.geometry, handler.getSketch()]);
        handler.drawFeature();
        delete handler.redoStack;
    } ,
    
    /**
     * Add
     */
    addControl: function(type) {
        //Add polygon
        if (type == 'Polygon') {
            this.controls.addControl = new OpenLayers.Control.DrawFeature(this.editLayer, OpenLayers.Handler.Polygon, {
                'displayClass': 'olControlAddPolygon',
                title: gt.translate._('add')
            });
            
            this.controls.addControl.insertXY = Giscuit.Util.bind(function(x, y) {
                var handler = this.controls.addControl.handler;
                
                //Default action
                if (handler && handler.line) {
                    handler.insertXY(x, y);
                    
                //Hack to fix insertXY when adding the first point using touch
                } else if (handler) {
                    this.insertPolygonFirstXY(x, y, handler);
                }
            }, this);
            
        //Add Line
        } else if (type == 'Line') {
            this.controls.addControl = new OpenLayers.Control.DrawFeature(this.editLayer, OpenLayers.Handler.Path, {
                'displayClass': 'olControlAddPath',
                title: gt.translate._('add')
            });
            
            this.controls.addControl.insertXY = Giscuit.Util.bind(function(x, y) {
                var handler = this.controls.addControl.handler;
                
                //Default action
                if (handler && handler.line) {
                    handler.insertXY(x, y);
                    
                //Hack to fix insertXY when adding the first point using touch
                } else if (handler) {
                    this.insertLineFirstXY(x, y, handler);
                }
            }, this);
        
        //Add Point
        } else if (type == 'Point') {
            this.controls.addControl = new OpenLayers.Control.DrawFeature(this.editLayer, OpenLayers.Handler.Point, {
                'displayClass': 'olControlAddPoint',
                title: gt.translate._('add')
            });
        
            this.controls.addControl.insertXY = Giscuit.Util.bind(function(x, y) {
                var handler = this.controls.addControl.handler;
                this.insertPointFirstXY(x, y, handler);
            }, this);
        }
        
        //Events
        this.controls.addControl.events.on({
            "featureadded": this.addAction,
            "activate": function() {
                if(gt.control.editattr) {
                    gt.control.editattr.destroy();
                    gt.control.editattr = null;
                }
                this.clearInsertedFeatures();
            },
            scope: this
        });
        
        //
        this.controlPanel.addControls(this.controls.addControl);
    },
    
    addAction: function (evt) {
        if(! this.loaded) {
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        //Check if valid Polygon
        if(evt.feature.geometry instanceof OpenLayers.Geometry.Polygon
            && evt.feature.geometry.getVertices().length < 3) {
        
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        //Check if valid Line
        if(evt.feature.geometry instanceof OpenLayers.Geometry.LineString
            && evt.feature.geometry.getVertices().length == 2
            && evt.feature.geometry.components[0].distanceTo(evt.feature.geometry.components[1]) == 0
            ) {
        
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        if(gt.control.editattr) {
            gt.control.editattr.destroy();
            gt.control.editattr = null;
        }
        
        gt.control.editattr = new Giscuit.Control.Editattr(evt.feature, 0);
        
        this.controls.addControl.deactivate();
    },
    
    /*
     * Edit
     */
    editControl: function() {
       this.controls.editControl = new OpenLayers.Control.ModifyFeature(this.editLayer, {
           'displayClass': 'olControlEdit',
           clickout: false,
           mode: OpenLayers.Control.ModifyFeature.RESHAPE | OpenLayers.Control.ModifyFeature.DRAG,
           title: gt.translate._('edit')
       });
           
        //Events
        this.controls.editControl.events.on({
            'activate': function() {
                if(gt.control.editattr) {
                    gt.control.editattr.destroy();
                    gt.control.editattr = null;
                }
                this.clearInsertedFeatures();
            },
            scope: this
        });
        
      
        this.controls.editControl.selectControl.events.on({
            'featureunhighlighted': this.editFeatureUnhighlighted,
            'featurehighlighted': this.editAction,
            scope: this
        });
    
        
        this.editLayer.events.on({
            'loadend': this.loadend,
            'loadstart': this.loadstart,
            'featuremodified': function() {
                this.featuremodified();
            },
            scope: this
        });
        
        //Set move vertex color
        this.controls.editControl.collectDragHandle = function() {
            var geometry = this.feature.geometry;
            var center = geometry.getBounds().getCenterLonLat();
            var originGeometry = new OpenLayers.Geometry.Point(
                center.lon, center.lat
            );
                            
            var origin = new OpenLayers.Feature.Vector(
                originGeometry,
                null,
                {
                    pointRadius: gt.map.pointRadius, 
                    fillColor: "yellow", 
                    fillOpacity: 0.4, 
                    strokeColor: "yellow", 
                    strokeWidth: 3
                }
            );
            
            originGeometry.move = function(x, y) {
                OpenLayers.Geometry.Point.prototype.move.call(this, x, y);
                geometry.move(x, y);
            };
            origin._sketch = true;
            this.dragHandle = origin;
            
            this.layer.addFeatures([this.dragHandle], {silent: true});
        };
        
        //
        if(this.editLayer.styleMap.styles['default'].rules.length) {
            this.controls.editControl.collectVertices = function() {
                //FIXME:
                delete this.virtualStyle.display;
                
                this.vertices = [];
                this.virtualVertices = [];        
                var control = this;
                function collectComponentVertices(geometry) {
                    var i, vertex, component, len;
                    if(geometry.CLASS_NAME == "OpenLayers.Geometry.Point") {
                        vertex = new OpenLayers.Feature.Vector(geometry);
                        vertex._sketch = true;
                        vertex.renderIntent = control.vertexRenderIntent;
                        control.vertices.push(vertex);
                    } else {
                        var numVert = geometry.components.length;
                        if(geometry.CLASS_NAME == "OpenLayers.Geometry.LinearRing") {
                            numVert -= 1;
                        }
                        for(i=0; i<numVert; ++i) {
                            component = geometry.components[i];
                            if(component.CLASS_NAME == "OpenLayers.Geometry.Point") {
                                vertex = new OpenLayers.Feature.Vector(component);
                                vertex._sketch = true;
                                vertex.renderIntent = control.vertexRenderIntent;
                                control.vertices.push(vertex);
                            } else {
                                collectComponentVertices(component);
                            }
                        }
                        
                        // add virtual vertices in the middle of each edge
                        if(geometry.CLASS_NAME != "OpenLayers.Geometry.MultiPoint") {
                            for(i=0, len=geometry.components.length; i<len-1; ++i) {
                                var prevVertex = geometry.components[i];
                                var nextVertex = geometry.components[i + 1];
                                if(prevVertex.CLASS_NAME == "OpenLayers.Geometry.Point" &&
                                   nextVertex.CLASS_NAME == "OpenLayers.Geometry.Point") {
                                    var x = (prevVertex.x + nextVertex.x) / 2;
                                    var y = (prevVertex.y + nextVertex.y) / 2;
                                    var point = new OpenLayers.Feature.Vector(
                                        new OpenLayers.Geometry.Point(x, y),
                                        null, control.virtualStyle
                                    );
                                    // set the virtual parent and intended index
                                    point.geometry.parent = geometry;
                                    point._index = i + 1;
                                    point._sketch = true;
                                    control.virtualVertices.push(point);
                                }
                            }
                        }
                    }
                }
                
                collectComponentVertices.call(this, this.feature.geometry);
                
                //FIXME:
                var vertexStyle = OpenLayers.Util.applyDefaults({
                    pointRadius: gt.map.pointRadius ,
                    strokeWidth: 3
                }, OpenLayers.Feature.Vector.style['default']);
                for(var i=0; i < this.vertices.length; i++) {
                    this.vertices[i].style = vertexStyle;
                }
                
                this.layer.addFeatures(this.virtualVertices, {silent: true});
                this.layer.addFeatures(this.vertices, {silent: true});
            };
        }
        
        this.controlPanel.addControls(this.controls.editControl);
    },
    
    selectByBboxHandler: function(selectControl) {
    },
    
    editAction: function(evt) {
        if(evt.feature == this.controls.editControl.feature) {
            return;
        }
        
        if (gt.control.editattr) {
            gt.control.editattr.destroy();
            gt.control.editattr = null;
        }
        
        evt.feature.originalGeometry = evt.feature.geometry.clone();
        
        gt.control.editattr = new Giscuit.Control.Editattr(evt.feature, 0);
        
    },
    
    editFeatureUnhighlighted: function(evt) {
        if(evt.feature.state == OpenLayers.State.UPDATE 
            && evt.feature.originalGeometry != undefined) {

            this.editLayer.removeFeatures(evt.feature, true);

            //Revert unsaved geometry
            evt.feature.geometry.destroy();
            evt.feature.geometry = evt.feature.originalGeometry.clone();
            evt.feature.state = null;
            
            this.editLayer.addFeatures(evt.feature, true);
        }
        
        //
        if(this.controls.editsaveControl != undefined) {
            this.controls.editsaveControl.panel_div.className = 'olControlEditsaveItemInactive';
        }
    },
    
    /**
     * Delete
     */
    deleteControl: function() {
        this.controls.deleteControl = new OpenLayers.Control.SelectFeature(this.editLayer, {
            'displayClass': 'olControlDelete',
            title: gt.translate._('delete')
        });
        this.controls.deleteControl.events.on({
            "activate": function() {
                if(gt.control.editattr) {
                    gt.control.editattr.destroy();
                    gt.control.editattr = null;
                }
                this.clearInsertedFeatures();
            },
            "featurehighlighted": this.deleteAction,
            scope: this
        });
        
        //
        this.controlPanel.addControls(this.controls.deleteControl);
    },
    
    deleteAction: function (evt) {
        var deleteCallback = {
            success: function() {
                this.backActivate();
            },
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        setTimeout(Giscuit.Util.bind(function() {
            var answer = confirm(gt.translate._('deleteConfirm'));
            if (answer) {
                this.editLayer.removeFeatures(evt.feature);
                
                var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom/delete/id/" + evt.feature.attributes.gid, deleteCallback);
            } else {
                this.controls.deleteControl.unselect(evt.feature);
            }
        }, this), 100);
    },
    
    /**
     * Back
     */
    backControl: function(history, changesCount) {
        if(! history) {
            return;
        }
        
        this.controls.backControl = new OpenLayers.Control({
            'displayClass': 'olControlBack',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                if(this.controls.backControl.panel_div.className == 'olControlBackItemActive olButton') {
                    this.backAction();
                }
            }, this),
            title: gt.translate._('undo')
        });
        
        //
        this.controlPanel.addControls(this.controls.backControl);
    },
    
    backAction: function() {
        var backCallback = {
            success: this.backSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom/back", backCallback);
    } ,
    
    backActivate: function() {
        if(this.controls.backControl == undefined) {
            return false;
        }
        
        this.controls.backControl.panel_div.className = 'olControlBackItemActive olButton';
        
        return true;
    },
    
    backDeactivate: function() {
        if(this.controls.backControl == undefined) {
            return false;
        }
        
        this.controls.backControl.panel_div.className = 'olControlBackItemInactive olButton';
        
        return true;
    },
    
    backSuccess: function (o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        if(responseText.changesCount == 0) {
            this.backDeactivate();
        } else {
            this.backActivate();
        }
        
        this.editLayer.mergeNewParams( {
            'rand' : Math.random()
        });
    },
    
    /**
     * Layer name
     */
    layerNameControl: function() {
        this.controlPanel.addLayerNameControl = function() {
            if(! this.div.children.length) {
                return;
            }
            
            var layerNamePH = document.createElement('span');
            layerNamePH.innerHTML = gt.control.layers.getEditLayer().name.replace(/ /g,'&nbsp;');
            layerNamePH.style.display = 'inline-block';
            //layerNamePH.style.float = 'left';
            layerNamePH.style.backgroundColor = '#EEE';
            layerNamePH.style.fontWeight = 'bold';
            
            if(gt.config.layout == 'desktop') {
                layerNamePH.style.padding = '8px 0 0 3px';
                this.div.appendChild(layerNamePH);
            } else {
                layerNamePH.style.padding = '13px 0 0 3px';

                //Do not show layer name if delete control is not at the top line
                if($('.olControlDeleteItemInactive').position().top == 0) {
                    $(this.div).prepend(layerNamePH);
                }
            }
            
        };
        this.controlPanel.redraw = function() {
            OpenLayers.Control.Panel.prototype.redraw.apply(this, arguments);
            this.addLayerNameControl();
        };
        this.controlPanel.draw = function() {
            OpenLayers.Control.Panel.prototype.draw.apply(this, arguments);
            this.addLayerNameControl();
            return this.div;
        };
    } ,
    
    invalidEditScale: function(editMaxscale) {
        gt.tooltip.cfg.setProperty('text', gt.translate._('layerIsEditableFromScale') + ' 1:' + editMaxscale);
        gt.tooltip.showTooltip();
    },
    
    //Initialize Controls
    initializeControls: function (data) {
        //Activate snap
        if(typeof(gt.layer.snap) != 'undefined') {
            gt.layer.snap.setVisibility(true);
        }
        if (typeof(gt.snap.edit) != 'undefined') {
            gt.snap.edit.activate();
        }
        
        //Style
        this.editLayer.styleMap.styles['default'].rules = [];
        this.editLayer.styleMap.styles['default'].addRules(gt.map.getStyleRules(data.sld, true));
        
        //        
        this.controls = {};

        //Invalid edit scale message
        if(gt.map.getScale() > data.edit_maxscale) {
            this.invalidEditScale(data.edit_maxscale);
        }
        
        //Initialize panels
        this.controlPanel = new OpenLayers.Control.Panel({
            'displayClass': 'gtEditgeomToolbar'
        });
        
        //Primary panel controls
        if (data.editgeom) {
            this.addControl(data.type);
            this.editControl();
            this.deleteControl();
            this.backControl(data.history, data.changesCount);
        } else {
            this.editControl();
        }
        
        //
        if(gt.layer.edit) {
            this.editLayer.setVisibility(true);
            gt.map.refresh();
        }
        
        this.layerNameControl();
        
        //Add panels
        gt.map.addControl(this.controlPanel);
        
        //
        var mapControls = YAHOO.util.Dom.getRegion('gtMapControls');
        var region = YAHOO.util.Dom.getRegion(this.controlPanel.div);
        
        this.controlPanel.div.style.left = mapControls.width - region.width - 2 + 'px' ;
        
        //
        if(data.changesCount > 0) {
            this.backActivate();
        }
    },
    
    loadend: function() {
        this.loaded = true;
        
        gt.control.editgeom.controls.editControl.selectControl.unselectAll();
        
        if(gt.control.editattr) {
            gt.control.editattr.destroy();
            gt.control.editattr = null;
        }
    },
    
    loadstart: function() {
        this.loaded = false;
    },
    
    featuremodified: function() {
    },
    
    clearInsertedFeatures: function() {
        var features = this.editLayer.features;
        for(var i = 0; i < features.length; i++) {
            if(features[i].state == OpenLayers.State.INSERT) {
                this.editLayer.removeFeatures(features[i], true);
            }
        }
    },
    
    CLASS_NAME: "Giscuit.Control.Editgeom"
});