/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Querybuilder.js 1133 2011-03-14 16:12:16Z ue $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.querybuilder = new Giscuit.Control.Querybuilder();
    }
});

/**
 * Class: Giscuit.Control.Querybuilder
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Querybuilder = OpenLayers.Class(Giscuit.Control, {

    dt: null,
    
    resultsDt: null,
    
    queriesDt: null,
    
    resultsDtHeightCoef: 85,
    
    queriesPerPage: 10,
    
    dtRecords : {},
    
    matchRecords: {},
    
    
    operators: [],
    
    functs: [], 
    
    measurementFuncts: [],
    
    measurementOperators: [],
    
    
    layerArr: [],
    
    fieldArr: [],
    
    layerGeomArr: [''],
    

    panel: null,
    
    layerDropdown: null,
    
    layerDropdownLastIndex: null,
    
    nameDialog: null,
    
    
    selected: null,
        
    
    limit: null,
    
    
    submitButton: null,
        
    resetButton: null,
       
    backButton: null,
    
    downloadButton: null,
    
    saveButton: null,
    
    loadButton: null,
    
    
    cleared: false,

    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        if(! gt.pluginParams.querybuilderIsAllowed) {
            return;
        }       
        
        var self = this;
        
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon(28), 
            id: "gtControlQuerybuilder", 
            onclick: {
                fn: function() {
                    self.activate();
                }
            },
            title: gt.translate._('querybuilder'), 
            container: "gtMapControls"
        });
    },
    
    activate: function () {
        var activated = Giscuit.Control.prototype.activate.call(this);

        if (activated == true) {
            this.create();
        } else {
            gt.accord.openPanel(this.getPanelIndex());
        }
        
        return activated;
    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        return deactivated;
    },
    
    clear: function() {
        this.deactivate();

        gt.accord.unsubscribe('afterPanelOpen', this.updateDtHeight);
        
        if(this.dt != null) {
            this.dt.destroy();
            this.dt = null;
        }
        
        if(this.resultsDt != null) {
            this.resultsDt.destroy();
            this.resultsDt = null;
        }
       
        //Buttons
        this.submitButton.destroy();
        this.submitButton = null;
        
        this.resetButton.destroy();
        this.resetButton = null;
        
        this.backButton.destroy();
        this.backButton = null;
        
        this.downloadButton.destroy();
        this.downloadButton = null;
        
        //
        YAHOO.util.Event.purgeElement(this.layerDropdown, true);
        this.layerDropdown = null;
        this.layerDropdownLastIndex = null;
        
        YAHOO.util.Event.purgeElement('gtQuerybuilderAdd', true);
                
        //Properties
        this.dtRecords = {};
        this.matchRecords = {};
        this.layerArr = [];
        this.fieldArr = [];
        this.layerGeomArr = [];
        
        this.cleared = true;
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    actionFormatter: function(elCell, oRecord, oColumn, oData) {
        var image = document.createElement('img');
        image.style.cursor = 'pointer';
        image.src = gt.config.httpImagePath + '/list_remove.png';
        
        YAHOO.util.Event.addListener(image, "click", function(oArgs) {
            YAHOO.util.Event.purgeElement(this);
            gt.control.querybuilder.dt.deleteRow(this.parentNode.parentNode.parentNode);
            
            //
            gt.control.querybuilder.conditionChange();
        });
        
        YAHOO.util.Dom.get('gtQuerybuilderError');
        
        elCell.style.textAlign = 'center';
        elCell.appendChild(image);
    },
    
    fieldFormatter: function(elCell, oRecord, oColumn, oData) {
        var field = oRecord.getData('field');
        var label = null;
        
        for(var i=0; i < this.fieldArr.length; i++) {
            if(field == this.fieldArr[i].value) {
                label = this.fieldArr[i].label;
                break;
            }
        }
        
        if(label) {
            elCell.innerHTML = label;
        } else {
            elCell.innerHTML = '';
        }
    },
    
    operatorFormatter: function(elCell, oRecord, oColumn, oData) {
        var operator = oRecord.getData('operator');
        var funct = oRecord.getData('funct');
        var label = null;
        
        if(funct == 'ST_DWithin') {
            for(var i=0; i < this.layerGeomArr.length; i++) {
                if(operator == this.layerGeomArr[i].value) {
                    label = this.layerGeomArr[i].label;
                    break;
                }
            }
            
            elCell.innerHTML = label;
        } else {
            elCell.innerHTML = oData;
        }

    },
    
    value0Formatter: function(elCell, oRecord, oColumn, oData) {
        var tmpRecord = oRecord.getData('field').split(',');
        var layer = tmpRecord[0];
        var field = tmpRecord[1];
        var value0 = oRecord.getData('value0');
        var funct = oRecord.getData('funct');
        
        var label = null;
               
        if((field != 'the_geom') || 
            (field == 'the_geom' && Giscuit.Util.inArray(gt.control.querybuilder.measurementFuncts, funct)) ||
            (field == 'the_geom' && funct == 'ST_DWithin')
            ) {
            
            elCell.innerHTML = oData;
        }  else {
            for(var i=0; i < this.layerGeomArr.length; i++) {
                if(value0 == this.layerGeomArr[i].value) {
                    label = this.layerGeomArr[i].label;
                    break;
                }
            }
        
            if(label) {
                elCell.innerHTML = label;
            } else {
                elCell.innerHTML = '';
            }
        }
    },
    
    conditionChange: function() {
        var affectedLayers = [];
        
        //Get selected layer id
        var selectedLayerId = this.layerDropdown.options[this.layerDropdown.selectedIndex].value;
        
        //Update data table records
        this.dtRecords[selectedLayerId] = [];
        var dtRecords = this.dt.getRecordSet().getRecords();
        for (i = 0; i < dtRecords.length; i++) {
            this.dtRecords[selectedLayerId].push(dtRecords[i].getData());
        }
           
        //Get current layers
        var currLayers = gt.control.layers.dt.getRecordSet().getRecords();
           
        //Get affected layers 
        for (layerId in this.dtRecords) {
            var records = this.dtRecords[layerId];
            
            for (var a=0; a < records.length; a++) {
                var record = records[a];
                
                if (record.field && record.operator) {
                    affectedLayers.push(layerId);
                    break;
                }
            }
        }
        
        //
        var save = YAHOO.util.Dom.get('gtQuerybuilderSave');
        
        for(var a=0; a < affectedLayers.length; a++) {
            for(var b=0; b < currLayers.length; b++) {
                var affectedLayersData = currLayers[b].getData();
                
                if(affectedLayersData.id == affectedLayers[a]) {
                    if(affectedLayersData.view == true || save.checked == false) {
                        affectedLayers[a] = affectedLayersData.name;
                    } else {
                        affectedLayers[a] = '<span style="color: #CC1100;">' + affectedLayersData.name + '</span>';
                    }
                }
                
            }
        }
        
        var affectedEl = YAHOO.util.Dom.get('gtQuerybuilderAffected');
        if(affectedLayers.length == 0) {
            affectedEl.innerHTML = gt.translate._('none');
        } else {
            affectedEl.innerHTML = affectedLayers.join(", ");
        }
        
    },
    
    reset: function() {
        YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/querybuilder/clear",
        {
            success: function(o) {
                this.submitButton.set('disabled', false);
                this.resetButton.set('disabled', false);
                
                try {
                    var responseText = YAHOO.lang.JSON.parse(o.responseText);
                } catch (e) {
                    Giscuit.Util.parseFailure(o, e);
                    return;
                }
                
                if (responseText.replyCode != 200) {
                    Giscuit.Util.invalidReplyCode(responseText);
                    return;
                }
                
                //Hide error
                var errorPH = YAHOO.util.Dom.get('gtQuerybuilderError').style.display = 'none';
                
                //
                gt.map.refresh();
                
                //dt
                var totalRecords = this.dt.getState().totalRecords;
                if(totalRecords > 0) {
                    this.dt.deleteRows(0, totalRecords);
                    this.dt.addRow({field: "", operator: "", value0: "", action: "", funct: ""}, 0);
                    
                    this.dtRecords = {};
                }
                
                //Match
                var matchDropdown = YAHOO.util.Dom.get('gtQuerybuilderMatch');
                this.matchRecords = {};
                matchDropdown.selectedIndex = 0;
                
                //Affected
                YAHOO.util.Dom.get('gtQuerybuilderAffected').innerHTML = gt.translate._('none');
                
            } ,
            failure: function(o) {
                Giscuit.Util.callbackFailure(o);   
            } , 
            scope: this
        });

    },
    
    back: function() {
        YAHOO.util.Dom.get('gtQuerybuilderLimitText').style.display = 'none';
        
        //Visibility
        YAHOO.util.Dom.get('gtQuerybuilderPH').style.display = 'block';
        YAHOO.util.Dom.get('gtQuerybuilderResultsPH').style.display = 'none';
    },
    
    collectQueryData: function() {
        //Get selected layer id
        var selectedLayerId = this.layerDropdown.options[this.layerDropdown.selectedIndex].value;
           
        //Save match records
        var matchDropdown = YAHOO.util.Dom.get('gtQuerybuilderMatch');
        this.matchRecords[selectedLayerId] = matchDropdown.options[matchDropdown.selectedIndex].value;
           
        //id
        var id = this.layerDropdown.options[this.layerDropdown.selectedIndex].value;
        
        //data
        var dataJson = YAHOO.lang.JSON.stringify(this.dtRecords, function(key, value) {
            if(key == 'value0' || key == 'operator') {
                return encodeURIComponent(value);
            } else {
                return value;
            }
        });
           
        //match
        var matchJson = YAHOO.lang.JSON.stringify(this.matchRecords);
        
        var save = 'f';
        if(YAHOO.util.Dom.get('gtQuerybuilderSave').checked) {
            save = 't';
        }
        
        var collectedData = {
        	id: id ,
        	save: save,
        	matchJson: matchJson,
        	dataJson: dataJson
        };
        
        return collectedData;
    },

    submit: function() {
        var collectedData = this.collectQueryData();
    	
        //Send data
        YAHOO.util.Connect.asyncRequest(
            'POST', 
            gt.config.httpToolPath + "/querybuilder/submit/id/" + collectedData.id + "/save/" + collectedData.save,
            {
                success: this.submitSuccess ,
                failure: function(o) {
                    Giscuit.Util.callbackFailure(o);   
                } , 
                scope: this
            }, 'data=' + collectedData.dataJson + '&match=' + collectedData.matchJson
        );
        
        this.submitButton.set('disabled', true);
        this.resetButton.set('disabled', true);
    },
    
    
    submitSuccess: function (o) {
        this.submitButton.set('disabled', false);
        this.resetButton.set('disabled', false);
        
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        var errorPH = YAHOO.util.Dom.get('gtQuerybuilderError');
        if(responseText.error) {
               errorPH.innerHTML = responseText.error;
               errorPH.style.display = 'block';
            
            return;
        } else {
            errorPH.style.display = 'none';
        }
        
        if(responseText.records.length == this.limit) {
            YAHOO.util.Dom.get('gtQuerybuilderLimitText').style.display = 'block';
        } else {
            YAHOO.util.Dom.get('gtQuerybuilderLimitText').style.display = 'none';
        }
        
        //Visibility
        YAHOO.util.Dom.get('gtQuerybuilderPH').style.display = 'none';
        YAHOO.util.Dom.get('gtQuerybuilderResultsPH').style.display = 'block';
        
        var fields = responseText.fields;
        var scale = responseText.scale;
        
        //
        if(this.resultsDt != null) {
            this.resultsDt.destroy();
            this.resultsDt = null;
        }
                    
        //Create data source
        var ds = new YAHOO.util.DataSource(responseText.records);
        ds.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
        ds.responseSchema = {
            "fields": fields.concat(['the_geom'])
        };
        
        //Create column definitions
        var columnDefs = [];
        var maxLength = (fields.length < gt.config.maxFieldsAmount) ? fields.length : gt.config.maxFieldsAmount;
        for(var i=0; i < maxLength; i++) {
            var column = {};
            column.key = fields[i];
            column.label = gt.translate._(fields[i]);
            column.sortable = true;
            column.resizeable = true;
            columnDefs.push(column);
        }
        
        //Show column limitation tooltip
        if (maxLength == gt.config.maxFieldsAmount) {
        	var firstNFieldsDisplayed = gt.translate._('firstNFieldsDisplayed').replace("%1$s", gt.config.maxFieldsAmount);
            gt.tooltip.cfg.setProperty('text', firstNFieldsDisplayed);
            gt.tooltip.showTooltip();
        }
        
        //Create table
        this.resultsDt = new YAHOO.widget.DataTable("gtQuerybuilderResultsDt", columnDefs, ds, {
            scrollable: true,
            selectionMode: "single",
            width: '99.5%',
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });
        
        //
        gt.accord.unsubscribe('afterPanelOpen', this.updateDtHeight);
        gt.accord.on('afterPanelOpen', this.updateDtHeight, null, this);
        this.setOptimalDatatableHeight(this.resultsDt, this.resultsDtHeightCoef);
        
        this.resultsDt.subscribe("rowMouseoverEvent", this.dt.onEventHighlightRow);
        this.resultsDt.subscribe("rowMouseoutEvent", this.dt.onEventUnhighlightRow);
        
        //Select row
        this.resultsDt.subscribe("cellClickEvent", function (oArgs) {
            this.resultsDt.onEventSelectRow(oArgs);
            
            var row = this.resultsDt.getSelectedRows();
            var geometry = OpenLayers.Geometry.fromWKT(this.resultsDt.getRecord(row[0]).getData('the_geom'));
            
            if(this.selected != null) {
                gt.layer.vector.destroyFeatures([this.selected]);
                this.selected = null;
            }
            
            this.selected = new OpenLayers.Feature.Vector(geometry);
            
            gt.layer.vector.addFeatures([this.selected]);
            
            gt.map.zoomToExtent(geometry.getBounds());
            
            //Zoom to scale if available
            if(scale != null) {
                gt.map.zoomToScale(parseInt(scale));
            }
            
        }, null, this);
        
        //Refresh the map
        gt.map.refresh();
    },
    
    download: function(oType, oArgs) {
        var menuItem = oArgs[1];
        
        if (menuItem) {
            this.downloadButton.set('disabled', true);
            
            //id
            var id = this.layerDropdown.options[this.layerDropdown.selectedIndex].value;
            
            YAHOO.util.Connect.asyncRequest(
                'GET' , 
                gt.config.httpToolPath + "/querybuilder/downloadcreate/id/" + id + "/ext/" + menuItem.value, {
                    success: function(o) {
                        this.downloadButton.set('disabled', false);
                    
                        try {
                            var responseText = YAHOO.lang.JSON.parse(o.responseText);
                        } catch (e) {
                            Giscuit.Util.parseFailure(o, e);
                            return;
                        }
                        
                        if (responseText.replyCode != 200) {
                            Giscuit.Util.invalidReplyCode(responseText);
                            return;
                        }
                        
                        window.location = gt.config.httpToolPath + '/querybuilder/download/file/' + responseText.file;
                        
                    },
                    failure: function(o) {
                       Giscuit.Util.callbackFailure(o);   
                    } , 
                    scope: this
                }
            );
        }
    },
    
    createSuccess: function (o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch (e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        //Clear
        if(this.cleared == true) {
            //Refresh the map
            gt.map.refresh();
            
            this.cleared = false;
        }
        
        //Limit
        this.limit = responseText.limit;
        var firstNResultsDisplayed = gt.translate._('firstNResultsDisplayed').replace("%1$s", this.limit);
                   
        //Html
        var html =  
            '<form><dl id="gtQuerybuilderPH" class="zend_form">' +
            '<p id="gtQuerybuilderError" style="color: #CC1100; display: none;"></p>' +
            '<dt><label>' +  gt.translate._('layer') + '</label></dt>' + 
            '<dd><select id="gtQuerybuilderLayer"></select></dd>' +
            
            '<dt><label>' +  gt.translate._('showResultsOnMap') + '</label></dt>' +
            '<dd><input type="checkbox" id="gtQuerybuilderSave" value="t" checked="checked"></dd>' +
            
            '<dt><label>' +  gt.translate._('affectedLayers') + '</label></dt>' +
            '<dd id="gtQuerybuilderAffected">' + gt.translate._('none') + '</dd>' +
            
            '<hr noshade size="1">' +
            
            '<dt><label>' +  gt.translate._('match') + '</label></dt>' +
            '<dd><select id="gtQuerybuilderMatch">' +
            '<option value="1">' + gt.translate._('all') + '</option>' +
            '<option value="2">' + gt.translate._('any') + '</option>' +
            '</select></dd>' +
            
            '<div id="gtQuerybuilderDt" style="margin-top: 10px;"></div>' +
            
            '<p style="margin-top: 5px;"><img style="cursor: pointer;" id="gtQuerybuilderAdd" src="' 
            + gt.config.httpImagePath + '/list_add.png"> ' + gt.translate._('addCondition') + '</p>' +
            
            '<button type="button" id="gtQuerybuilderSubmit">' + gt.translate._('submit') + '</button>' +
            '<button type="button" id="gtQuerybuilderClear">' + gt.translate._('clear') + '</button>' +
            '<button type="button" id="gtQuerybuilderShowQueries">' + gt.translate._('loadQuery') + '</button>' +
            '</dl></form>' +
            
            '<form><dl class="zend_form" id="gtQuerybuilderQueriesPH" style="display: none;">' +
            '<div id="gtQuerybuilderQueriesDt"></div>' +
            '<p></p>' +
            '<button type="button" id="gtQuerybuilderQueriesLoad">' + gt.translate._('load') + '</button>' +
            '<button type="button" id="gtQuerybuilderQueriesDelete">' + gt.translate._('delete') + '</button>' +
            '<button type="button" id="gtQuerybuilderQueriesBack">' + gt.translate._('back') + '</button>' +
            '</dl></form>' +
            
            '<form><dl class="zend_form" id="gtQuerybuilderResultsPH" style="display: none;">' +
            
            '<button type="button" id="gtQuerybuilderBack">' + gt.translate._('back') + '</button>' +
            '<input id="gtQuerybuilderDownloadButton" value="' + gt.translate._('download') + '">' +
            '<select id="gtQuerybuilderDownloadMenu"></select>' +
            '<button type="button" id="gtQuerybuilderSaveQuery">' + gt.translate._('saveQuery') + '</button>' +
            '<div id="gtQuerybuilderNameDialog"></div>' + 
            
            '<div id="gtQuerybuilderResultsDt" style="margin-top: 10px;"></div>' +
            '<div id="gtQuerybuilderLimitText" style="display: none;"><i>' + firstNResultsDisplayed +  '</i></div>' +
            '</dl></form>';
        
        gt.accord.setBody(this.getPanelIndex(), html);
        
        //Add condition
        var image = YAHOO.util.Dom.get('gtQuerybuilderAdd');
        YAHOO.util.Event.addListener(image, "click", function(oArgs) {
            this.dt.addRow({field: "", operator: "", value0: "", action: "", funct: ""}, this.dt.getRecordSet().getLength());
            
            //
            this.conditionChange();
        } , this, true);
        
        //
        this.operators = responseText.operators;
        this.functs = responseText.functs;
        this.measurementFuncts = responseText.measurementFuncts;
        this.measurementOperators = responseText.measurementOperators;
        
        //Formats dropdown
        var formatsDropdown = YAHOO.util.Dom.get('gtQuerybuilderDownloadMenu');
        for(obj in responseText.formats) {
            var elOptNew = document.createElement('option');
            elOptNew.text = responseText.formats[obj];
            elOptNew.value = obj;
            
            try {
                formatsDropdown.add(elOptNew, null);
            } catch(ex) {
                formatsDropdown.add(elOptNew);
            }
        }
        
        //
        var layers = responseText.records;
        for(var a=0; a < layers.length; a++) {
            var fields = layers[a].fields;
            
            //layerArr
            this.layerArr.push({
                value: layers[a].id ,
                label: layers[a].path + layers[a].name 
            });
            
            //layerGeomArr
            this.layerGeomArr.push({
                value: layers[a].id ,
                label: '"' + layers[a].name + '"."' + gt.translate._('the_geom') + '"'
            });
            
            //fieldArr
            for(var b=0; b < fields.length; b++) {
                this.fieldArr.push({
                    label: '"' + layers[a].name + '"."' + gt.translate._(fields[b].name) + '"',
                    value: [layers[a].id, fields[b].name]
                });
            }
            if(a != layers.length - 1) {
                this.fieldArr.push("");
            }
        }
        
        //Layer dropdown
        this.layerDropdown = YAHOO.util.Dom.get('gtQuerybuilderLayer');
        this.layerDropdown.style.width = '90%';
        
        for (var i in this.layerArr) {
            var elOptNew = document.createElement('option');
            elOptNew.text = this.layerArr[i].label;
            elOptNew.value = this.layerArr[i].value;
            
            try {
                this.layerDropdown.add(elOptNew, null);
            } catch(ex) {
                this.layerDropdown.add(elOptNew);
            }
        }
        YAHOO.util.Event.addListener(this.layerDropdown, "change", function (e) {
            this.createDatatable();
        }, null, this);
        
        //
        YAHOO.util.Event.addListener('gtQuerybuilderSave', "change", function (e) {
            this.conditionChange();
        }, null, this);
        
        this.createDatatable();
        
        //
                    
        //Create buttons
        this.submitButton = new YAHOO.widget.Button('gtQuerybuilderSubmit');
        this.submitButton.on("click", this.submit, null, this);
        
        this.resetButton = new YAHOO.widget.Button('gtQuerybuilderClear');
        this.resetButton.on("click", this.reset, null, this);
        
        this.loadButton = new YAHOO.widget.Button('gtQuerybuilderShowQueries');
        this.loadButton.on("click", function() {
        	if (this.queriesDt == null) {
        		this.createQueriesDt();
        	} else {
        		this.queriesDt.ds.sendRequest('', {
                    success : this.queriesDt.dt.onDataReturnInitializeTable,
                    failure : this.queriesDt.dt.onDataReturnInitializeTable,
                    scope   : this.queriesDt.dt,
                    argument: this.queriesDt.dt.getState()
                });
        	}
        	
        	YAHOO.util.Dom.get('gtQuerybuilderQueriesPH').style.display = 'block';
        	YAHOO.util.Dom.get('gtQuerybuilderPH').style.display = 'none';
        }, null, this);
        
        this.backButton = new YAHOO.widget.Button('gtQuerybuilderBack');
        this.backButton.on("click", this.back, null, this);
        
        this.downloadButton = new YAHOO.widget.Button("gtQuerybuilderDownloadButton", { 
            type: "menu", 
            menu: "gtQuerybuilderDownloadMenu"
        });
        this.downloadButton.getMenu().subscribe("click", this.download, null, this);
        
        this.saveButton = new YAHOO.widget.Button('gtQuerybuilderSaveQuery');
        this.saveButton.on("click", this.saveQuery, null, this);
    },
    
    saveQuery: function() {
        var handleNo = function() {
            this.nameDialog.hide();
        };
        
        var handleYes = function() {
        	this.nameDialog.hide();
        	var name = YAHOO.util.Dom.get('gtQuerybuilderQueryName').value;

            var collectedData = this.collectQueryData();
            var url = gt.config.httpToolPath + 
            	"/querybuilder/savequery/id/" + collectedData.id + 
            	"/save/" + collectedData.save +
            	"/name/" + name;
        	
            //Send data
            YAHOO.util.Connect.asyncRequest('POST', url, {
                success: function(o) {
                    try {
                        var responseText = YAHOO.lang.JSON.parse(o.responseText);
                    } catch (e) {
                        Giscuit.Util.parseFailure(o, e);
                        return;
                    }
                        
                    if (responseText.replyCode != 200) {
                        Giscuit.Util.invalidReplyCode(responseText);
                        return;
                    }
                        
                    this.saveButton.set('disabled', false);
                } ,
                failure: function(o) {
                    Giscuit.Util.callbackFailure(o);   
                } , 
                scope: this
            }, 'data=' + collectedData.dataJson + '&match=' + collectedData.matchJson);
            
            this.saveButton.set('disabled', true);
        }
    	
        var dialogText = '<input value="" type="text" name="query_name" id="gtQuerybuilderQueryName" style="width: 275px;">';
    	if(this.nameDialog == null) {  
	    	this.nameDialog = new YAHOO.widget.SimpleDialog("nameDialog", { 
	    		width: "300px",
				fixedcenter: true,
				visible: true,
				draggable: true,
				close: false,
				text: dialogText,
				constraintoviewport: true,
				buttons: [{ text: gt.translate._("save"), handler: Giscuit.Util.bind(handleYes, this), isDefault: true},
					{ text: gt.translate._("cancel"), handler: Giscuit.Util.bind(handleNo, this) }]
			});
	    	
	    	this.nameDialog.setHeader(gt.translate._('queryName')); 
	    	this.nameDialog.render("gtQuerybuilderNameDialog");
    	} else {
    		this.nameDialog.setBody(dialogText);
    		this.nameDialog.show();
    	}
    },
    
    populateData: function(queryData) {
        //Set selected layer
        var layerOptions = YAHOO.util.Dom.get('gtQuerybuilderLayer').options;
        for (var i = 0; i <  layerOptions.length; i++) {
        	if (layerOptions[i].value == queryData.id) {
        		layerOptions[i].selected = true;
        		break;
        	}
        }
        
        //Set show results
        if (queryData.save == 't') {
        	YAHOO.util.Dom.get('gtQuerybuilderSave').checked = true;
        } else {
        	YAHOO.util.Dom.get('gtQuerybuilderSave').checked = false;
        }
        
        //Set match
        YAHOO.util.Dom.get('gtQuerybuilderMatch').options[queryData.match[queryData.id] - 1].selected = true;
        this.matchRecords[this.layerDropdownLastIndex] = queryData.match[queryData.id];
        
        //Set conditions
        this.dtRecords = queryData.data;
        this.layerDropdownLastIndex = null;
        
        this.createDatatable();
        this.conditionChange();
    },
    
    createQueriesDt: function() {
        var queriesDtOptions = {
            fields : ["id", "name"] ,
            columnDefs : [
                {key: "id", resizeable: true, sortable: true, label: gt.translate._("id")},
                {key: "name", resizeable: true, sortable: true, label: gt.translate._("queryName")}
            ],
            dataUrl : gt.config.httpToolPath + "/querybuilder/getqueries" ,
            deleteDataUrl : gt.config.httpToolPath + "/querybuilder/deletequeries",
            container : 'gtQuerybuilderQueriesDt' , 
            rowsPerPage :  this.queriesPerPage ,
            keyColumn : 'id' ,
            selectionMode: 'single' ,
            paginatorAlwaysVisible: false
        };  

        this.queriesDt = new Giscuit.Datatable(queriesDtOptions);
        this.queriesDt.deleteButton = new YAHOO.widget.Button('gtQuerybuilderQueriesDelete');

        var queryLoadButton = new YAHOO.widget.Button('gtQuerybuilderQueriesLoad');
        queryLoadButton.on("click", function(o) {
            var rows = this.queriesDt.dt.getSelectedRows();
            var id = this.queriesDt.dt.getRecord(rows[0]).getData('id');
            	
            queryLoadButton.set('disabled', true);
            YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/querybuilder/loadquery/id/" + id, {
                success: function(o) {
                    try {
                        var responseText = YAHOO.lang.JSON.parse(o.responseText);
                    } catch (e) {
                        Giscuit.Util.parseFailure(o, e);
                        return;
                    }
                        
                    if (responseText.replyCode != 200) {
                        Giscuit.Util.invalidReplyCode(responseText);
                        return;
                    }
                    
                    queryLoadButton.set('disabled', false);
                    this.populateData(responseText.queryData);
                    
                	YAHOO.util.Dom.get('gtQuerybuilderPH').style.display = 'block';
                	YAHOO.util.Dom.get('gtQuerybuilderQueriesPH').style.display = 'none';
                } ,
                failure: function(o) {
                    Giscuit.Util.callbackFailure(o);   
                } , 
                scope: this
            });
        }, null, this);
        
        var backButton = new YAHOO.widget.Button('gtQuerybuilderQueriesBack');
        backButton.on("click", function(o) {
        	YAHOO.util.Dom.get('gtQuerybuilderPH').style.display = 'block';
        	YAHOO.util.Dom.get('gtQuerybuilderQueriesPH').style.display = 'none';
        });

        this.queriesDt.initialize();
    },
    
    createDatatable: function() {
        //Get selected layer id
        var selectedLayerId = this.layerDropdown.options[this.layerDropdown.selectedIndex].value;
        
        //
        var matchDropdown = YAHOO.util.Dom.get('gtQuerybuilderMatch');
        
        //Save previous conditions
        if(this.layerDropdownLastIndex != null) {
            
            //dt
            this.dtRecords[this.layerDropdownLastIndex] = [];
            var records = this.dt.getRecordSet().getRecords();
            
            for(i = 0; i < records.length; i++) {
                this.dtRecords[this.layerDropdownLastIndex].push(records[i].getData());
            }
            
            //match
            this.matchRecords[this.layerDropdownLastIndex] = matchDropdown.options[matchDropdown.selectedIndex].value;
        } 
        
        //Default dt records
        if(selectedLayerId in this.dtRecords == false) {
            this.dtRecords[selectedLayerId] = [{field: "", operator: "", value0: "", action: "", funct: ""}];
        }
        
        //Default match records
        if(selectedLayerId in this.matchRecords == false) {
            this.matchRecords[selectedLayerId] = 1;
            matchDropdown.selectedIndex = 0;
        } else {
            for(var i=0; i < matchDropdown.options.length; i++) {
                if(matchDropdown.options[i].value == this.matchRecords[selectedLayerId]) {
                    matchDropdown.selectedIndex = i;
                    break;
                }
            }
        }

        //Create datasource
        var ds = new YAHOO.util.DataSource(this.dtRecords[selectedLayerId]);
        ds.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
        ds.responseSchema = {
            fields: ["field", "operator", "value0", "funct"]
        };
           
        //Define data table columns
        var functsArr = this.functs.slice(0);
        functsArr.unshift('');
        var columnDefs = [
            {        
                key: "action" ,
                label: '' ,
                formatter: Giscuit.Util.bind(this.actionFormatter, this)
            },
            {
                key: "field",
                label: gt.translate._("field") ,
                editor: new YAHOO.widget.QuerybuilderFieldEditor({
                    disableBtns: false ,
                    dropdownOptions: this.fieldArr
                }) ,
                formatter: Giscuit.Util.bind(this.fieldFormatter, this)
            } ,
            {
                key: "funct" ,
                label: gt.translate._("function") ,
                editor: new YAHOO.widget.DropdownCellEditor({
                    dropdownOptions: functsArr ,
                    disableBtns: false
                }) 
            } ,
            {
                key: "operator" ,
                label: gt.translate._("operator") ,
                editor: new YAHOO.widget.QuerybuilderOperatorEditor({
                    disableBtns: false
                }) ,
                formatter: Giscuit.Util.bind(this.operatorFormatter, this)
            } ,
            {
                key: "value0" ,
                label: gt.translate._("value0") ,
                editor: new YAHOO.widget.QuerybuilderValueEditor({
                    disableBtns: false
                }) ,
                formatter: Giscuit.Util.bind(this.value0Formatter, this)
            }
        ];
        
        if(this.dt != null) {
            this.dt.destroy();
            this.dt = null;
        }
                    
        //Create data table
        this.dt = new YAHOO.widget.ScrollingDataTable("gtQuerybuilderDt", columnDefs, ds, {
            width: '99.5%',
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });
        
        this.dt.subscribe("cellClickEvent", function (oArgs) {
            var fieldColumnValue = this.getRecord(oArgs.target)._oData.field.split(',')[1];
            var columnType = this.getColumn(oArgs.target).key;
            var funct = this.getRecord(oArgs.target)._oData.funct;
            var operator = this.getRecord(oArgs.target)._oData.operator;
            
            if (columnType == 'field') {
                this.onEventShowCellEditor(oArgs);
                
            } else if ((columnType == 'funct' && fieldColumnValue == 'the_geom') || 
                (columnType == 'operator' && fieldColumnValue != 'the_geom') || 
                (columnType == 'operator' && fieldColumnValue == 'the_geom' && Giscuit.Util.inArray(gt.control.querybuilder.measurementFuncts, funct)) ||
                (columnType == 'operator' && fieldColumnValue == 'the_geom' && funct == 'ST_DWithin')
                ) {
                
                this.onEventShowCellEditor(oArgs);
            } else if (columnType == 'value0') {
                if (funct != 'ST_IsValid' && funct != 'NOT ST_IsValid' &&
                    operator != 'IS NULL' && operator != 'IS NOT NULL') {
                    
                    this.onEventShowCellEditor(oArgs);
                }
            }
            
        }, null, this.dt);
        
        this.dt.subscribe("editorSaveEvent", function(oArgs) {
            //Field change
            if(oArgs.editor._oColumn.key == 'field') {
                this.dt.updateCell(oArgs.editor._oRecord, 'operator', '');
                this.dt.updateCell(oArgs.editor._oRecord, 'value0', '');
                this.dt.updateCell(oArgs.editor._oRecord, 'funct', '');
            }
            
            //Funct change
            if(oArgs.editor._oColumn.key == 'funct' && oArgs.newData != oArgs.oldData) {
                this.dt.updateCell(oArgs.editor._oRecord, 'operator', '');
                this.dt.updateCell(oArgs.editor._oRecord, 'value0', '');
            }
            
            this.conditionChange();
        }, null, this);
        
        this.dt.subscribe("editorBlurEvent", function (oArgs) {
            this.cancelCellEditor();
        });
        
        //
        this.layerDropdownLastIndex = selectedLayerId;
    }, 

    create: function () {

        if (this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('querybuilder'),
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.querybuilder.pkg.html');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }

        gt.accord.openPanel(this.getPanelIndex());

        var createCallback = {
            success: this.createSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/querybuilder", createCallback);

    },
    
    updateDtHeight: function(args) {
        if(this.panel == args.panel && this.resultsDt) {
            this.setOptimalDatatableHeight(this.resultsDt, this.resultsDtHeightCoef);
        }
    },
    
    CLASS_NAME: "Giscuit.Control.Querybuilder"
});

(function () {

    var lang = YAHOO.lang,
        util = YAHOO.util,
        widget = YAHOO.widget,
        ua = YAHOO.env.ua,
        Dom = util.Dom,
        Ev = util.Event,
        DT = widget.DataTable;

    var BCE = widget.BaseCellEditor;
    var DCE = widget.DropdownCellEditor;

    widget.QuerybuilderFieldEditor = function (oConfigs) {
        this._sId = "yui-dropdownceditor" + BCE._nCount++;
        DCE.superclass.constructor.call(this, "querybuilderField", oConfigs);
    };

    lang.extend(widget.QuerybuilderFieldEditor, DCE, {

        focus : function() {
            var layerDropdown = gt.control.querybuilder.layerDropdown;
            var selectedLayerId = layerDropdown.options[layerDropdown.selectedIndex].value;

            if(this.getInputValue() == '') {
                for(var i=0; i < this.dropdown.options.length; i++) {
                    var option = this.dropdown.options[i];
                    var layerId = option.value.split(',')[0];
                                    
                    if(layerId == selectedLayerId) {
                        this.dropdown.selectedIndex = i;
                        break;
                    }
                }
            }
            
            //Focus on element
            this.getDataTable()._focusEl(this.dropdown);
        }
    });

    lang.augmentObject(widget.QuerybuilderFieldEditor, DCE);

})();

(function () {

    var lang = YAHOO.lang,
        util = YAHOO.util,
        widget = YAHOO.widget,
        ua = YAHOO.env.ua,
        Dom = util.Dom,
        Ev = util.Event,
        DT = widget.DataTable;

    var BCE = widget.BaseCellEditor;
    var DCE = widget.DropdownCellEditor;
    
    widget.QuerybuilderOperatorEditor = function (oConfigs) {
        this._sId = "yui-dropdownceditor" + BCE._nCount++;
        DCE.superclass.constructor.call(this, "querybuilderOperator", oConfigs);
    };

    lang.extend(widget.QuerybuilderOperatorEditor, DCE, {

        focus : function() {
            //
            var tmpRecord = this._oRecord.getData('field').split(',');
            this.layer = tmpRecord[0];
            this.field = tmpRecord[1];
            this.funct = this._oRecord.getData('funct');
            
            //Clear
            var containerEl = this.getContainerEl();
            if(containerEl) {
                Ev.purgeElement(containerEl, true);
                containerEl.innerHTML = '';
            }
            
            //Populate
            if(this.field != 'the_geom') {
                this.dropdownOptions = gt.control.querybuilder.operators;
            } else if (this.field == 'the_geom' && 
                Giscuit.Util.inArray(gt.control.querybuilder.measurementFuncts, this.funct)) {
                
                this.dropdownOptions = gt.control.querybuilder.measurementOperators;
            } else if (this.field == 'the_geom' && this.funct == 'ST_DWithin') {
                this.dropdownOptions = gt.control.querybuilder.layerGeomArr;
            }
            
            //Render
            this.renderForm();
            this.renderBtns();
            
            //FIXME: Hack for ie "<" symbol
            if(this.field != 'the_geom') {
                for(var i=0; i < this.dropdown.options.length; i++) {
                    if(this.dropdown.options[i].value == '<') {
                        this.dropdown.options[i].text = '< ';
                    }
                }
            }
            
            //Set value
            var value = this.getRecord().getData(this.getColumn().key);
            if(value) {
                for(var i=0; i < this.dropdown.options.length; i++) {
                    if(this.dropdown.options[i].value == value) {
                        this.dropdown.selectedIndex = i;
                        break;
                    }
                }
            }
            
            //Focus
            this.getDataTable()._focusEl(this.dropdown);
        }
    });

    lang.augmentObject(widget.QuerybuilderOperatorEditor, DCE);

})();

(function () {

    var lang = YAHOO.lang,
        util = YAHOO.util,
        widget = YAHOO.widget,
        ua = YAHOO.env.ua,
        Dom = util.Dom,
        Ev = util.Event,
        DT = widget.DataTable;

    var BCE = widget.BaseCellEditor;
    var DCE = widget.DropdownCellEditor;
    var TCE = widget.TextboxCellEditor;

    widget.QuerybuilderValueEditor = function (oConfigs) {
        this._sId = "yui-dropdownceditor" + BCE._nCount++;
        DCE.superclass.constructor.call(this, "querybuilderValue", oConfigs);
    };

    lang.extend(widget.QuerybuilderValueEditor, DCE, {

        textbox: null,
        
        isGeometryDropdown: function() {
            if(this.field == 'the_geom' &&
                ! Giscuit.Util.inArray(gt.control.querybuilder.measurementFuncts, this.funct) &&
                this.funct != 'ST_DWithin') {
                    
                return true;
            } else {
                return false;
            }
        },
        
        renderForm: function() {
            if(this.isGeometryDropdown()) {
                DCE.prototype.renderForm.call(this);
            } else {
                TCE.prototype.renderForm.call(this);
            }
        },
        
        move: function() {
            if(this.isGeometryDropdown()) {
                DCE.prototype.move.call(this);
            } else {
                TCE.prototype.move.call(this);
            }
        },
        
        resetForm: function() {
            if(this.isGeometryDropdown()) {
                DCE.prototype.resetForm.call(this);
            } else {
                TCE.prototype.resetForm.call(this);
            }
        },

        renderBtns: function() {
          
            var elBtnsDiv = this.getContainerEl().appendChild(document.createElement("div"));
            elBtnsDiv.className = DT.CLASS_BUTTON;
            
            // Save button
            var elSaveBtn = elBtnsDiv.appendChild(document.createElement("button"));
            elSaveBtn.className = DT.CLASS_DEFAULT;
            elSaveBtn.innerHTML = this.LABEL_SAVE;
            Ev.addListener(elSaveBtn, "click", function(oArgs) {
                this.save();
            }, this, true);
            this._elSaveBtn = elSaveBtn;
            
            if (this.field != 'the_geom') {
                // Data button
                var elDataBtn = elBtnsDiv.appendChild(document.createElement("button", gt.translate._('uniqueValues')));
                elDataBtn.innerHTML = gt.translate._('uniqueValues');
                elDataBtn.style.width = '8em';
                
                var populateCallback = {
                    success: function (o) { 
                        elDataBtn.disabled = false;
                        
                        try {
                            var responseText = YAHOO.lang.JSON.parse(o.responseText);
                        } catch (e) {
                            Giscuit.Util.parseFailure(o, e);
                            return;
                        }
                        
                        if (responseText.replyCode != 200) {
                            Giscuit.Util.invalidReplyCode(responseText);
                            return;
                        }
                        
                        var records = responseText.records;
                        var select = document.createElement("select");

                        for (var i = 0; i < records.length; i++) {
                            select.options.add(new Option(records[i], records[i]));
                        }
                        
                        var previousSelect = this.getContainerEl().getElementsByTagName("select");
                        if (previousSelect.length != 0) {
                            this.getContainerEl().removeChild(previousSelect[0]);
                        } 
                        
                        var textInput = this.getContainerEl().firstChild;
                        this.textbox.style.display = 'none';
                        this.getContainerEl().insertBefore(select, textInput);
                        this.textbox.value = select.value;
                        
                        var saveButton = this.getContainerEl().lastChild.lastChild.previousSibling;
                        Ev.addListener(select, "change", function(oArgs) {
                            this.textbox.value = select.value;
                        }, this, true);
                        
                    } , 
                    failure: function(o) {
                        Giscuit.Util.callbackFailure(o);   
                    } , 
                    scope: this
                };
                
                Ev.addListener(elDataBtn, "click", function(oArgs) {
                    var record = this._oRecord.getData('field').split(',');
                    if (record.length == 2) {
                        elDataBtn.disabled = true;
                        var layer = record[0];
                        var field = record[1];
                        var url = gt.config.httpToolPath + '/querybuilder/getcolumnvalues/id/' + layer + '/field/' + field;
                        YAHOO.util.Connect.asyncRequest('GET', url, populateCallback); 
                    }
                }, this, true);
            }

            // Cancel button
            var elCancelBtn = elBtnsDiv.appendChild(document.createElement("button"));
            elCancelBtn.innerHTML = this.LABEL_CANCEL;
            Ev.addListener(elCancelBtn, "click", function(oArgs) {
                this.cancel();
            }, this, true);
            this._elCancelBtn = elCancelBtn; 
        },
        
        focus: function() {
            //
            var tmpRecord = this._oRecord.getData('field').split(',');
            this.layer = tmpRecord[0];
            this.field = tmpRecord[1];
            this.funct = this._oRecord.getData('funct');
                        
            //Clear
            var containerEl = this.getContainerEl();
            if(containerEl) {
                Ev.purgeElement(containerEl, true);
                containerEl.innerHTML = '';
            }

            //Populate
            if(this.isGeometryDropdown()) {
                  this.dropdownOptions = gt.control.querybuilder.layerGeomArr;
            }
            
            //Render
            this.renderForm();
            this.renderBtns();
            
            //Set value
            var value = this.getRecord().getData(this.getColumn().key);
            
            if(this.isGeometryDropdown()) {
                for(var i=0; i < this.dropdown.options.length; i++) {
                    if(this.dropdown.options[i].value == value) {
                        this.dropdown.selectedIndex = i;
                        break;
                    }
                }
            }

            if(value && this.field == 'the_geom' && 
                (
                    Giscuit.Util.inArray(gt.control.querybuilder.measurementFuncts, this.funct) ||
                    this.funct == 'ST_DWithin'
                )) {
                
                this.textbox.value = value;
            }
            
            if(value && this.field != 'the_geom') {
                this.textbox.value = value;
            }
                        
            //Focus
            if(this.isGeometryDropdown()) {
                DCE.prototype.focus.call(this);
            } else {
                TCE.prototype.focus.call(this);
            }
        },
        
        getInputValue: function() {
            if(this.isGeometryDropdown()) {
                return DCE.prototype.getInputValue.call(this);
            } else {
                return TCE.prototype.getInputValue.call(this);
            }
        }
        
    });

    lang.augmentObject(widget.QuerybuilderValueEditor, DCE);

})();