/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Search.js 2946 2012-08-17 15:10:49Z vlad.bruma $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.search = new Giscuit.Control.Search();
    }
});

/**
 * Class: Giscuit.Control.Search
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Search = OpenLayers.Class(Giscuit.Control, {

    dt: null,
    
    dtHeightCoef: 135,

    panel: null,
    
    button: null,

    searchButton: null,

    backButton: null,

    
    loadImg: null,
    
    searchLayer : null,
        
    selected: null,
    
    layers: [],
    
    elements: ['gtSearchLayer-label' , 
        'gtSearchLayer-element' ,
        
        'gtSearchField-label' ,
        'gtSearchField-element' ,
        
        'gtSearchWhere-label' ,
        'gtSearchWhere-element'
    ],
    
    timeout: 0 ,
    
    limit: 100 ,
    
    textLabel: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon(18), 
            id: "gtControlSearch", 
            onclick: {
                fn: function() {
                    self.activate();
                }
            },
            title: gt.translate._('search'), 
            container: "gtMapControls"
        });
    },
    
    activate: function () {
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        if (activated == true) {
            this.create();
        } else {
             gt.accord.openPanel(this.getPanelIndex());
        }
        
        return activated;
    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if (deactivated == true) {
            //Destroy features
            if(this.selected != null) {
                gt.layer.vector.destroyFeatures([this.selected]);
                this.selected = null;
            }
            
            gt.control.layers.events.unregister('changelayer', this, this.filterByVisibility);
        }
        
        return deactivated;
    },
    
    clear: function() {
        this.deactivate();
        
        gt.accord.unsubscribe('afterPanelOpen', this.updateDtHeight);
        
        if(this.dt != null) {
            this.dt.destroy();
            this.dt = null;
        }
        
        this.searchButton.destroy();
        this.searchButton = null;
        
        this.backButton.destroy();
        this.backButton = null;
        
        this.searchLayer = null;
        this.loadImg = null;
        
        YAHOO.util.Event.purgeElement('gtSearchField', true);
        YAHOO.util.Event.purgeElement('gtSearchLayer', true);
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    submitSuccess: function (o) {
        this.searchButton.set('disabled', false);
        
        try {
            //
            if(typeof(o.responseText) == 'string') {
                var responseText = YAHOO.lang.JSON.parse(o.responseText);

                if (responseText.replyCode != 200) {
                    Giscuit.Util.invalidReplyCode(responseText);
                    return;
                }
            } else {
                var responseText = o.responseText;
            }

        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        var fields = responseText.fields;
        var scale = parseInt(responseText.scale);
        
        if(responseText.records.length == this.limit) {
            YAHOO.util.Dom.get('gtSearchLimitText').style.display = 'block';
        } else {
            YAHOO.util.Dom.get('gtSearchLimitText').style.display = 'none';
        }
            
        //
        if(this.dt != null) {
            this.dt.destroy();
            this.dt = null;
        }

        //Create data source
        var ds = new YAHOO.util.DataSource(responseText.records);
        ds.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
        ds.responseSchema = {
            fields: fields.concat(['the_geom'])
        };
                    
        //Create column definitions
        var columnDefs = [];
        var maxLength = (fields.length < gt.config.maxFieldsAmount) ? fields.length : gt.config.maxFieldsAmount;
        for(var i=0; i < maxLength; i++) {
            var column = {};
            column.key = fields[i];
            column.label = gt.translate._(fields[i]);
            column.sortable = true;
            column.resizeable = true;
            columnDefs.push(column);
        }
        
        //Show column limitation tooltip
        if (maxLength == gt.config.maxFieldsAmount) {
        	var firstNFieldsDisplayed = gt.translate._('firstNFieldsDisplayed').replace("%1$s", gt.config.maxFieldsAmount);
            gt.tooltip.cfg.setProperty('text', firstNFieldsDisplayed);
            gt.tooltip.showTooltip();
        }
        
        //Create table
        this.dt = new YAHOO.widget.ScrollingDataTable("gtSearchDataTable", columnDefs, ds, {
            selectionMode: "single",
            width: '99.5%',
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });     
        
        this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);
        
        this.dt.subscribe("rowMouseoverEvent", this.dt.onEventHighlightRow);
        this.dt.subscribe("rowMouseoutEvent", this.dt.onEventUnhighlightRow);
        
        //Select row
        this.dt.subscribe("cellClickEvent", function (oArgs) {
            this.dt.onEventSelectRow(oArgs);
            
            var row = this.dt.getSelectedRows();
            var geometry = OpenLayers.Geometry.fromWKT(this.dt.getRecord(row[0]).getData('the_geom'));
            
            if(this.selected != null) {
                gt.layer.vector.destroyFeatures([this.selected]);
                this.selected = null;
            }
            
            this.selected = new OpenLayers.Feature.Vector(geometry);
            
            gt.layer.vector.addFeatures([this.selected]);
            
            gt.map.zoomToExtent(geometry.getBounds());
            
            //Zoom to scale if available
            if(scale) {
                gt.map.zoomToScale(scale);
            }
            
        }, null, this);
        
        //FIXME: Hack for datatable overflow
        if(YAHOO.env.ua.opera) {
            var contentPanel = gt.accord.getPanel(this.getPanelIndex()).children[1];
            contentPanel.style.overflow = 'hidden';
        }
        
    },
    
    submitFailure: function(o) {
        this.searchButton.set('disabled', false);
        
        Giscuit.Util.callbackFailure(o);
    },

    submit: function () {
        this.searchButton.set('disabled', true);
        
        for(var i = 0; i < this.elements.length; i++) {
            var element = YAHOO.util.Dom.get(this.elements[i]);
            element.style.display = 'none';
        }
        
        YAHOO.util.Dom.get('gtSearchBack').style.display = 'inline-block';
        
        //Add Layer and Field information to the label
        if (this.textLabel == null) {
            this.textLabel = YAHOO.util.Dom.get('gtSearchText-label').firstChild.innerHTML;
        }

        var searchLabel = YAHOO.util.Dom.get('gtSearchText-label').firstChild;
        
        var searchLayerText = this.searchLayer.options[this.searchLayer.selectedIndex].text.split('/');
        searchLayerText = searchLayerText[searchLayerText.length - 1];
        
        var searchField = YAHOO.util.Dom.get('gtSearchField');
        var searchFieldText = searchField.options[searchField.selectedIndex].text;
        
        searchLabel.innerHTML = this.textLabel + ' (' + searchLayerText + ' - ' + searchFieldText + ')';

        var whereBbox = YAHOO.util.Dom.get('gtSearchWhere-bbox')
        if(whereBbox.checked) {
            whereBbox.value = gt.map.getExtent().toBBOX();
        }
        
        //Check for javascript layer
        var jsLayer = false;
        for(i=0; i < gt.control.layers.records.length; i++) {
            var layerSettings = gt.control.layers.records[i];
            
            if(this.searchLayer.options[this.searchLayer.options.selectedIndex].value == layerSettings.id &&
                (layerSettings.js_layer || layerSettings.jsLayer)) {
                
                jsLayer = true;
                break;
            }
        }
        
        if(jsLayer) {
            if(typeof(layerSettings.layerClass) != 'undefined') {
                var className = layerSettings.layerClass;
            } else {
                var className = layerSettings['class'];
            }
            var layer = gt.layer[className];
            
            var currentBounds = gt.map.getExtent();
            
            var field = YAHOO.util.Dom.get('gtSearchField').value;
            var fieldValue = YAHOO.util.Dom.get('gtSearchText').value.toString().toLowerCase();
            
            var searchEverywhere = YAHOO.util.Dom.get('gtSearchWhere-everywhere').checked;
            
            //Get default field
            if(typeof(layer.defaultField) != 'undefined') {
                var defaultField = layer.defaultField;
            } else {
                for(var defaultField in layer.features[0].attributes) {
                    break;
                }
            }
            
            //Get search scale
            var searchScale = null;
            if(layerSettings.search_scale) {
                searchScale = layerSettings.search_scale;
            } else if(typeof(layer.searchScale) != 'undefined') {
                searchScale = layer.searchScale;
            }
            
            var data = {
                'defaultField': defaultField,
                'records': [],
                'fields': [],
                'limit': this.limit,
                'scale': searchScale
            };
            
            //
            for (var i = 0; i < layer.features.length; i++) {
                if (layer.features[i].attributes[field] == null ||
                    data.records.length >= this.limit) {
                    continue;
                }
                if (layer.features[i].attributes[field].toString().toLowerCase().search(fieldValue) != -1) {
                    var featureAttributes = layer.features[i].attributes;
                    featureAttributes.the_geom = layer.features[i].geometry.toString();
                    
                    if(! searchEverywhere) {
                        if (currentBounds.intersectsBounds(layer.features[i].geometry.bounds)) {
                            data.records.push(featureAttributes);
                        }
                    } else {
                        data.records.push(featureAttributes);
                    }
                }
            }
            
            //
            for (var key in layer.features[0].attributes) {
                data.fields.push(key);
            }
            
            this.submitSuccess({'responseText': data});
        } else {
            
            
            var submitCallback = {
                success: this.submitSuccess,
                timeout: this.callbackTimeout + this.timeout,
                failure: this.submitFailure,
                scope: this
            };
            
            YAHOO.util.Connect.setForm('gtSearchForm');
            YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/search/index" + gt.map.getLayersString(), submitCallback);
        }
        
    },
    
    initInput: function () {
        this.searchButton = new YAHOO.widget.Button('gtSearchSubmit');
        this.searchButton.on("click", this.submit, null, this);
        
        this.backButton = new YAHOO.widget.Button('gtSearchBack');
        this.backButton.on("click", this.back, null, this);
        
        YAHOO.util.Dom.get('gtSearchBack').style.display = 'none';
    },

    createSuccess: function (o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        this.timeout = responseText.timeout;
        this.limit = responseText.limit;
        
        var firstNResultsDisplayed = gt.translate._('firstNResultsDisplayed').replace("%1$s", this.limit);
        
        var html = responseText.form + 
            '<div id="gtSearchDataTable"></div>' +
            '<div id="gtSearchLimitText" style="display: none; padding-top:2px;"><i>' + firstNResultsDisplayed  + '</i></div>';
        gt.accord.setBody(this.getPanelIndex(), html);
        
        this.initInput();
        
        this.searchLayer = YAHOO.util.Dom.get('gtSearchLayer');
        
        //Add javascript layers
        for(i=0; i < gt.control.layers.records.length; i++) {
            var layerSettings = gt.control.layers.records[i];
            
            if(typeof(layerSettings.layerClass) != 'undefined') {
                var className = layerSettings.layerClass;
            } else {
                var className = layerSettings['class'];
            }
                                            
            if((layerSettings.js_layer || layerSettings.jsLayer) &&
                typeof(gt.layer[className].features) != 'undefined') {
                
                var elOptNew = document.createElement('option');
                elOptNew.text = layerSettings.name;
                elOptNew.value = layerSettings.id;
                
                try {
                    this.searchLayer.add(elOptNew, null);
                } catch(ex) {
                    this.searchLayer.add(elOptNew);
                }
            }    
        }
        
        //
        this.layers = [];
        for(var i=0; i < this.searchLayer.options.length; i++) {
            var option = this.searchLayer.options[i];
            this.layers.push(option);
        }
        
        //
        this.loadImg = document.createElement('img');
        this.loadImg.src = gt.config.httpImagePath + '/spinner.gif';
        this.loadImg.style.marginLeft = '3px';
        this.loadImg.style.visibility = 'hidden';
        
        //
        this.filterByVisibility();
        
        //
        YAHOO.util.Event.addListener(this.searchLayer, "change", function (e, obj) {
            obj.selectTable();
        }, this);
        
        Giscuit.Util.insertAfter(YAHOO.util.Dom.get('gtSearchField'), this.loadImg);
        
        //
        gt.control.layers.events.unregister('changelayer', this, this.filterByVisibility);
        gt.control.layers.events.register('changelayer', this, this.filterByVisibility);
        
    },
    
    filterByVisibility: function() {
        //Filter layers
        var visibleLayers = [];
        
        //Get visible layers
        if(gt.config.guest == 't') {
            for(var obj in gt.control.layers.nodes) {
                var node = gt.control.layers.nodes[obj];
                
                if(node.data.layerClass && node.highlightState == 1) {
                    visibleLayers = visibleLayers.concat(node.data.ids.split(','));
                }
            }
        } else {
            for(i=0; i < gt.control.layers.records.length; i++) {
                var layerSettings = gt.control.layers.records[i];
                
                if(layerSettings.view) {
                    visibleLayers.push(layerSettings.id);
                }
            }
        }
        
        //Clear layer
        for (var i = this.searchLayer.options.length - 1; i >= 0; i--) { 
            this.searchLayer.removeChild(this.searchLayer.options[i]);
        }
        
        //
        for(var i=0; i < this.layers.length; i++) {
            if(Giscuit.Util.inArray(visibleLayers, this.layers[i].value)) {
                var elOptNew = document.createElement('option');
                elOptNew.text = this.layers[i].text;
                elOptNew.value = this.layers[i].value;
                try {
                    this.searchLayer.add(elOptNew, null);
                } catch(ex) {
                    this.searchLayer.add(elOptNew);
                }
            }
        }
        
        //Clear field
        var fields =  YAHOO.util.Dom.get('gtSearchField');
        
        for (var i = fields.options.length - 1; i >= 0; i--) { 
            fields.removeChild(fields.options[i]);
        }
        
        //
        if(! this.searchLayer.options.length) {
            this.searchButton.set('disabled', true);
        } else {
               this.selectTable();
            this.searchButton.set('disabled', false);
        }
    },

    create: function () {

        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('search') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.search.pkg.html');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        
        gt.accord.openPanel(this.getPanelIndex());
        
        gt.accord.on('afterPanelOpen', this.updateDtHeight, null, this);
        
        var createPanelCallback = {
            success: this.createSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/search", createPanelCallback);

    },
    
    updateDtHeight: function(args) {
        if(this.panel == args.panel && this.dt) {
            this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);
        }
    },

    getFieldsSuccess: function (o) {
        this.loadImg.style.visibility = 'hidden';

        try {
            if(typeof(o.responseText) == 'string') {
                var responseText = YAHOO.lang.JSON.parse(o.responseText);

                if (responseText.replyCode != 200) {
                    Giscuit.Util.invalidReplyCode(responseText);
                    return;
                }
            } else {
                var responseText = o.responseText;
            }
            
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        var fieldsObj = YAHOO.util.Dom.get('gtSearchField');
        var options = fieldsObj.getElementsByTagName('option');

        //Add new options
        for (var i in responseText.fields) {
            var elOptNew = document.createElement('option');
            elOptNew.text = responseText.fields[i];
            elOptNew.value = i;

            if (i == responseText.defaultField) {
                elOptNew.selected = true;
            }
            try {
                fieldsObj.add(elOptNew, null);
            } catch(ex) {
                fieldsObj.add(elOptNew);
            }
        }
    },

    selectTable: function () {
        var contentPanel = gt.accord.getPanel(this.getPanelIndex()).children[1];
        if(contentPanel.style.visibility == 'visible') {
            this.loadImg.style.visibility = 'visible';
        }
        
        var layers = YAHOO.util.Dom.get('gtSearchLayer');
        var currLayer = layers[layers.selectedIndex];

        //Clear fields
        var fields = YAHOO.util.Dom.get('gtSearchField');
        for (var i = fields.options.length - 1; i >= 0; i--) { 
            fields.removeChild(fields.options[i]);
        }
        
        //Check for javascript layer
        var jsLayer = false;
        for(i=0; i < gt.control.layers.records.length; i++) {
            var layerSettings = gt.control.layers.records[i];
            
            if(currLayer.value == layerSettings.id &&
                (layerSettings.js_layer || layerSettings.jsLayer)) {
                
                jsLayer = true;
                break;
            }
        }
        
        if(jsLayer) {
            if(typeof(layerSettings.layerClass) != 'undefined') {
                var className = layerSettings.layerClass;
            } else {
                var className = layerSettings['class'];
            }
            var layer = gt.layer[className];
            
            //Get default field
            if(typeof(layer.defaultField) != 'undefined') {
                var defaultField = layer.defaultField;
            } else {
                for(var defaultField in layer.features[0].attributes) {
                    break;
                }
            }
            
            var data = {
                'defaultField': defaultField,
                'fields': {}
            };
            
            for (var key in layer.features[0].attributes) {
                data.fields[key] = gt.translate._(key);
            }

            this.getFieldsSuccess({'responseText': data});
        } else {
            var getFieldsCallback = {
                success: this.getFieldsSuccess,
                timeout: this.callbackTimeout,
                failure: this.callbackFailure,
                scope: this
            };

            var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/search/fields/layer_id/" + currLayer.value, getFieldsCallback);
        }
    },
    
    back: function() {
        YAHOO.util.Dom.get('gtSearchLimitText').style.display = 'none';
        
        for(var i = 0; i < this.elements.length; i++) {
            var element = YAHOO.util.Dom.get(this.elements[i]);
            element.style.display = 'block';
        }
        YAHOO.util.Dom.get('gtSearchBack').style.display = 'none';
        
        //Set text label back
        YAHOO.util.Dom.get('gtSearchText-label').firstChild.innerHTML = this.textLabel;
       
        this.dt.destroy();
        this.dt = null;
    },
    
    CLASS_NAME: "Giscuit.Control.Search"
});