/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editattr.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Editattr
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Editattr = OpenLayers.Class(Giscuit.Control, {

    feature: null,

    initialize: function (feature, version) {
        if(feature != null) {
            this.feature = feature;
        }
        
        if(gt.control.offline.isOnline) {
             $.mobile.showPageLoadingMsg();
            
            var idStm = '';
            if(this.feature.attributes.gid != null) {
                idStm = "/id/" + this.feature.attributes.gid;
            }
            
            var url = gt.config.httpMobilePath + "/editgeom/get" + idStm + "/version/" + version;
            $.getJSON(url, Giscuit.Util.bind(this.getSuccess, this))
                .error(Giscuit.Util.jqmAjaxError); 
        } else {
            var editForm = Giscuit.Util.getStorageObject('gtEditForm');
            var formHtml = editForm['layer_' + gt.control.layers.getEditLayer().id];
            this.getSuccess({
                'form': formHtml
            });
        }
    },
    
    getSuccess: function (data) {
        var error = null;
        
        //Geometry errors
        if (data.errors) {
            for (var error in data.errors) {
                Giscuit.Util.jqmMessage(data.errors[error]);
            }
        }
        
        if(data.geometry) {
            //Change geometry
            
            gt.control.editgeom.editLayer.removeFeatures(this.feature, true);
            this.feature.geometry.destroy();
            this.feature.geometry = new OpenLayers.Geometry.fromWKT(data.geometry);
            gt.control.editgeom.editLayer.addFeatures(this.feature, true);
            
            this.feature.state = OpenLayers.State.UPDATE;
        }

        //Render
        var content = $("#gtEditattr > div[data-role='content']");
        content.html(data.form)

        //Toggle
        $('#gtEditattrPH fieldset').attr('data-role', 'collapsible');
        $('#gtEditattrPH fieldset').each(function(index, value) {
            if($(value).find('ul.errors').length) {
                $(value).attr('data-collapsed', 'false');
            }
        });
        $('#gtEditattrPH fieldset').parent().prev().css('margin-top', '0.5em');
        $('#gtEditattrPH fieldset').unbind().bind('expand', function () {
            var el = this;
            $('#gtEditattrPH fieldset').each(function(index, value) {
                if(el != value) {
                    $(value).trigger('collapse');
                }
            });
        });
        
        content.trigger('create');
        
        //Populate form
        if(! gt.control.offline.isOnline) {
            this.setFormValues('gtEditattrForm', this.feature.attributes)
        }
        
        $.mobile.changePage($('#gtEditattr'), {
            'changeHash': true
        });

        var submited = false;
        //Submit
        $('#gtEditattrSubmit').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
            submited = true;
            evt.data.submit();
            
            return false;
        });
        
        //Remove geometry if not submited
        $('#gtEditattr').die().live('pagehide', this, function(evt, ui) {
            if (! submited) {
                gt.control.editgeom.controls.editattrControl.unselect(evt.data.feature);
                gt.control.editgeom.clearInsertedFeatures();
            }
        });
        
        //Cancel
        $('#gtEditattrCancel').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, this, function(evt) {
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });
        
        //
        $('#gtEditattrVersionsButton').closest('.ui-btn').hide();
        
        //Change
        $('#gtEditattrVersionsSelect').unbind().change(this, function(evt) {
            evt.data.initialize(null, $(this).val());
        });
            
        if(! error) {
            $.mobile.hidePageLoadingMsg();
        }
    },
    
    submitOffline: function() {
        var editLayer = gt.control.editgeom.editLayer;

        var feature = this.feature;

        //Update
        if(feature.fid) {
            feature.attributes = this.getFormValues('gtEditattrForm', feature.attributes);
            if(typeof(feature.attributes.hstatus) == 'undefined') {
                feature.attributes.hstatus = 'U';
            }
            editLayer.protocol.update(feature);

        //Create
        } else {
            feature.attributes = this.getFormValues('gtEditattrForm', feature.attributes, true);
            feature.attributes.hstatus = 'C';
            editLayer.protocol.create(feature, {
                'callback': function(response) {
                    feature.fid = response.priv.result.insertId;
                }
            });
        }
        
        feature.state = null;
        gt.control.editgeom.controls.editattrControl.unselect(feature);
        
        $.mobile.changePage($('#gtMap'), {
            'changeHash': true
        });
    },
    
    submit: function() {
        if(! gt.control.offline.isOnline) {
            this.submitOffline();
            return;
        }
        
        var data = {};
        
        if(this.feature.attributes.gid) {
            var stm = '/edit/id/' + this.feature.attributes.gid;
            if (this.feature.state == OpenLayers.State.UPDATE) {
                var data = {'geometry': this.feature.geometry.toString()};
            }
        } else {
            var stm = '/add';
            var data = {'geometry': this.feature.geometry.toString()};
        }
        
        var url = gt.config.httpMobilePath + "/editgeom" + stm;
                
        $('#gtEditattrForm').ajaxSubmit({
            'type': 'POST',
            'url': url,
            'data': data,
            'success': this.submitSuccess,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json',
            'context': this
        }); 

    },
    
    submitSuccess: function (data) {
        if(data.success == false) {
            this.getSuccess(data);
        } else {
            //Destroy original
            if(typeof(this.feature.originalGeometry) != 'undefined') {
                this.feature.originalGeometry.destroy();
                this.feature.originalGeometry = undefined;
            }
            this.feature.state = null;
            gt.control.editgeom.controls.editattrControl.unselect(this.feature);
                            
            if(typeof(data.gid) != 'undefined') {
                //Update attributes
                this.feature.attributes = this.getFormValues('gtEditattrForm', this.feature.attributes, true);
                this.feature.attributes.gid = data.gid;
            } else {
                //Update attributes
                this.feature.attributes = this.getFormValues('gtEditattrForm', this.feature.attributes);
            }
            
            gt.control.editgeom.backActivate();
            gt.control.editgeom.editLayer.removeFeatures(this.feature, true);
            gt.control.editgeom.editLayer.addFeatures(this.feature, true);
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            $('#gtEditattrSubmit').button('enable');
        }
    } ,
    
    getFormValues: function(form, attributes, overwrite) {
        if(typeof(overwrite) == 'undefined') {
            overwrite = false;
        }
        
        var values = $('#' + form).serializeArray();
                
        //
        for(var i=0; i < values.length; i++) {
            var input = $('#' + form + ' input[name="' + values[i].name + '"]');
            
            if(input.length && 
                $(input[0]).attr('type') == 'checkbox' && 
                $(input[1]).attr('type') == 'hidden' &&
                $(input[0]).is(':checked')) {
                
                var value = $(input[0]).val();
            } else {
                var value = values[i].value;
            }
                        
            if(typeof(attributes[values[i].name]) != 'undefined' || overwrite) {
                attributes[values[i].name] = value;
            }
        }
        
        return attributes;
    },
    
    setFormValues: function(form, attributes) {
        $('#' + form + ' :input').each(Giscuit.Util.bind(function(index, value) {
            var name = $(value).attr('name');
            var type = $(value).attr('type');
            
            if(value.tagName.toLowerCase() != 'input') {
                type = value.tagName.toLowerCase();
            }
            
            if(typeof(attributes[name]) != 'undefined') {
                if(type == 'checkbox' || type == 'radio') {
                    if(attributes[name] == $(value).val()) {
                        $(value).attr('checked','checked').checkboxradio("refresh");
                    } else {
                        $(value).removeAttr('checked').checkboxradio("refresh");
                    }
                } else if (type == 'select') {
                    $(value).val(attributes[name]).selectmenu("refresh");
                } else if (type == 'hidden') {
                    
                }  else {
                    $(value).val(attributes[name]);
                }
            }
        }, this));
        
    },

    CLASS_NAME: "Giscuit.Mobile.Control.Editattr"
});

