/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editgeom.js 1600 2011-08-09 13:52:10Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
//
Giscuit.Mobile.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.editgeom = new Giscuit.Mobile.Control.Editgeom();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Editgeom
 * 
 *  Inherits from:
 *   - <Giscuit.Control.Editgeom>
 */
Giscuit.Mobile.Control.Editgeom = OpenLayers.Class(Giscuit.Control.Editgeom, {
    
    initialize: function (options) {
        
        //Touch Offset
        if(gt.config.touchOffset == 1) {
            this.touchOffsetToleranceX = 0;
            this.touchOffsetToleranceY = 0;
            
        } else if(gt.config.touchOffset == 2) {
            this.touchOffsetToleranceX = -35;
            this.touchOffsetToleranceY = -35;
            
        } else if(gt.config.touchOffset == 3) {
            this.touchOffsetToleranceX = -45;
            this.touchOffsetToleranceY = -45;
            
        } else if(gt.config.touchOffset == 4) {
            this.touchOffsetToleranceX = 35;
            this.touchOffsetToleranceY = -35;
            
        } else if(gt.config.touchOffset == 5) {
            this.touchOffsetToleranceX = 45;
            this.touchOffsetToleranceY = -45;
        }
        
        //
        $(document).ready(Giscuit.Util.bind(function() {
            if(gt.config.guest == 't') {
                return;
            }
            
            $("#gtMapControls").append('<div>'
                    + '<div class="ui-bar ui-bar-c" id="gtEditButton">'
                    + '<div class="gtMapControlsElImage" style="background-position: -384px 50%;"></div>'
                    + '<div class="gtMapControlsElText">' + gt.translate._('editing') + '</div>'
                    + '</div></div>');
                        
            $("#gtEditButton").bind(gt.config.clickEvt, Giscuit.Util.bind(function() {
                if(this.active) {
                    this.deactivate();
                } else {
                    //Hide map controls
                    $("#gtMapControls").css('display', 'none');
                    $("#gtToggleControlsButton").css('bottom', '5px').removeClass('ui-btn-active');
                    
                    this.activate();
                }
                
                return false;
            }, this));
            
            var html = 
            '<div data-role="page" id="gtEditattr" data-dom-cache="true" data-url="gtEditattr">' +
                '<div data-role="header" >' +
                    '<h1>' + gt.translate._('editAttributes') + '</h1>' +
                    '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                        gt.translate._('map') +
                '</a>' + 
                '</div>' +                 
                '<div data-role="content"></div>' +        
            '</div>';
            $.mobile.pageContainer.append(html);
            
        }, this));
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
    },
    
    enableEditing: function() {
        var editLayerData = gt.control.layers.getEditLayer();
        
        if (editLayerData 
            && ! editLayerData.offline
            && gt.control.offline.isOnline) {
            this.editLayer = gt.layer.edit;
            
            $.mobile.showPageLoadingMsg();
            
            var url = gt.config.httpMobilePath + "/privatelayers/editenable/rand/" + Math.random();
            $.getJSON(url, Giscuit.Util.bind(function(data) {
                $("#gtEditButton").addClass('ui-btn-active');
                
                this.initializeControls(data);
                
                if(gt.map.getScale() < data.edit_maxscale) {
                    $.mobile.hidePageLoadingMsg();
                }
                
                Giscuit.Util.updateApplicationCache();
            }, this))
                .error(Giscuit.Util.jqmAjaxError);
            
        } else if(editLayerData
            && editLayerData.offline
            && editLayerData.view
            && editLayerData.edit) {
            
            this.editLayer = gt.layer[editLayerData['class']];
            gt.snap.edit.layer = this.editLayer; 
            
            $("#gtEditButton").addClass('ui-btn-active');
            
            var mapGeometryType = Giscuit.Util.getStorageObject('gtMapGeometryType');
            
            this.initializeControls({
                'editgeom': true,
                'type': mapGeometryType['layer_' + editLayerData.id],
                'changesCount': 0,
                'history': false
            });
            
        } else {
            this.deactivate();
            Giscuit.Util.jqmMessage(gt.translate._('noEditLayerSelected'));
        }
    },
    
    disableEditing: function() {
        if(gt.control.offline.isOnline) {
            var url = gt.config.httpToolPath + "/privatelayers/editdisable/rand/" + Math.random();
        
            $.getJSON(url, Giscuit.Util.bind(function(data) {
                if(this.editLayer && this.editLayer.map) {
                    this.editLayer.setVisibility(false);
                    gt.map.refresh();
                }
                
                $("#gtEditButton").removeClass('ui-btn-active');
                
                Giscuit.Util.updateApplicationCache();
            }, this))
            .error(Giscuit.Util.jqmAjaxError);
        
            
        } else {
            $("#gtEditButton").removeClass('ui-btn-active');
        }
    },
    
    /**
     * Add
     */
    addAction: function (evt) {
        //Check if valid Polygon
        if(evt.feature.geometry instanceof OpenLayers.Geometry.Polygon
            && evt.feature.geometry.getVertices().length < 3) {
        
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        //Check if valid Line
        if(evt.feature.geometry instanceof OpenLayers.Geometry.LineString
            && evt.feature.geometry.getVertices().length == 2
            && evt.feature.geometry.components[0].distanceTo(evt.feature.geometry.components[1]) == 0
            ) {
        
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        if(gt.control.editattr) {
            gt.control.editattr.destroy();
            gt.control.editattr = null;
        }
        
        gt.control.editattr = new Giscuit.Mobile.Control.Editattr(evt.feature, 0);
        
        this.controls.addControl.deactivate();
    },

    /**
     * Edit
     */
    editAction: function(evt) {
        if(evt.feature == this.controls.editControl.feature) {
            return;
        }
        
        evt.feature.originalGeometry = evt.feature.geometry.clone();
    },
    
    touchOffset: function() {
        var self = this;
        var moved = false;
                
        //
        this.editLayer.events.on({
            'vertexmodified': function(evt) {
                var newPos = {
                    'x': evt.pixel.x + this.touchOffsetToleranceX,
                    'y': evt.pixel.y + this.touchOffsetToleranceY
                };
                
                if(moved == false) {
                    this.moveVertex(evt.vertex, newPos);
                    moved = true;
                }
                
                //considerSnapping
                if(typeof(gt.layer.snap) != 'undefined') {
                    gt.snap.edit.feature = evt.feature;
                    var loc = gt.map.getLonLatFromViewPortPx(newPos);
                    gt.snap.edit.considerSnapping(
                        evt.vertex, new OpenLayers.Geometry.Point(loc.lon, loc.lat)
                    );
                }
            },
            'featuremodified': function(evt) {
                moved = false;
            },
            scope: this
        });
        
        //unregister previous considerSnapping
        if(typeof(gt.layer.snap) != 'undefined') {
            this.editLayer.events.un({
                vertexmodified: gt.snap.edit.onVertexModified,
                scope: gt.snap.edit
            });
            gt.snap.edit.events.on({
                'unsnap': function(evt) {
                    moved = false;
                },
                scope: this
            });
        }
    },
    
    moveVertex: function(vertex, xy) {
        var newPos = gt.map.getLonLatFromViewPortPx({
            'x': xy.x,
            'y': xy.y
        });
        
        vertex.x = newPos.lon;
        vertex.y = newPos.lat;
        vertex.clearBounds();
    },
    
    /**
     * Delete
     */
    deleteAction: function (evt) {
        setTimeout(Giscuit.Util.bind(function() {
            var answer = confirm(gt.translate._('deleteConfirm'));
            if (answer) {
                if(gt.control.offline.isOnline) {
                    $.mobile.showPageLoadingMsg();
                
                    this.editLayer.removeFeatures(evt.feature);
                
                    var url = gt.config.httpMobilePath + "/editgeom/delete/id/" + evt.feature.attributes.gid;
                    $.getJSON(url, Giscuit.Util.bind(function(data) {
                        this.backActivate();
                        $.mobile.hidePageLoadingMsg();
                    }, this))
                        .error(Giscuit.Util.jqmAjaxError);
                } else {
                    this.editLayer.removeFeatures(evt.feature);
                    
                    if(typeof(evt.feature.attributes.hstatus) == 'undefined') {
                        evt.feature.attributes.hstatus = 'D';
                        this.editLayer.protocol.update(evt.feature);
                    } else if(evt.feature.attributes.hstatus == 'C') {
                        this.editLayer.protocol['delete'](evt.feature);
                    }
                }
            } else {
                this.controls.deleteControl.unselect(evt.feature);
            }
        }, this), 100);
    },
    
    /**
     * Back
     */
    backAction: function() {
        $.mobile.showPageLoadingMsg();
        
        var url =  gt.config.httpMobilePath + "/editgeom/back";
        
        $.getJSON(url, Giscuit.Util.bind(function(data) {
            if(data.changesCount == 0) {
                this.backDeactivate();
            } else {
                this.backActivate();
            }
            
            this.editLayer.mergeNewParams({
                'rand' : Math.random()
            });
            
            $.mobile.hidePageLoadingMsg();
        }, this))
            .error(Giscuit.Util.jqmAjaxError);
        
    } ,
    
    /**
     * Editsave
     */
    editsaveControl: function() {
        this.controls.editsaveControl = new OpenLayers.Control({
            'displayClass': 'olControlEditsave',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                var panelDiv = this.controls.editsaveControl.panel_div;
                if(panelDiv.className == 'olControlEditsaveItemInactive olButton') {
                    return;
                }
                
                var feature = this.controls.editControl.feature;
                
                if(gt.control.offline.isOnline) {
                    $.mobile.showPageLoadingMsg();
                    var url = gt.config.httpMobilePath + '/editgeom/editgeometry';
                    
                    $.ajax({
                        'type': 'POST',
                        'url': url,
                        'data': {
                            'id': feature.attributes.gid ,
                            'geometry': feature.geometry.toString()
                        },
                        'success': function(data) {
                            var error = null;
                            
                            //Destroy original
                            if(typeof(feature.originalGeometry) != 'undefined') {
                                feature.originalGeometry.destroy();
                                feature.originalGeometry = undefined;
                            }
                            feature.state = null;
                            
                            if (data.errors) {
                                for (var error in data.errors) {
                                    Giscuit.Util.jqmMessage(data.errors[error]);
                                }
                            } else {
                                this.backActivate();
                                
                                this.controls.editControl.selectControl.unselectAll();
                            }
                            
                            if(! error) {
                                $.mobile.hidePageLoadingMsg();
                            }
                        },
                        'error': Giscuit.Util.jqmAjaxError ,
                        'dataType': 'json' ,
                        'context': this
                    });
                } else {
                    feature.state = null;
                    
                    if(typeof(feature.attributes.hstatus) == 'undefined') {
                        feature.attributes.hstatus = 'U';
                    }
        
                    this.editLayer.protocol.update(feature);
                    
                    this.controls.editControl.selectControl.unselectAll();
                }
                                
                panelDiv.className = 'olControlEditsaveItemInactive olButton';
            }, this),
            title: gt.translate._('save')
        });   
        
        //
        this.controls.editControl.events.on({
            "activate": function() {
                this.controls.editsaveControl.panel_div.style.display = 'block';
            },
            "deactivate": function() {
                this.controls.editsaveControl.panel_div.style.display = 'none';
            },
            scope: this
        });
        
        //
        this.secondaryControlPanel.addControls(this.controls.editsaveControl);
        this.controls.editsaveControl.panel_div.style.display = 'none';
    },
    
    /**
     * Editattr
     */
    editattrControl: function () {
        this.controls.editattrControl = new OpenLayers.Control.SelectFeature(this.editLayer, {
            'displayClass': 'olControlEditattr',
            title: gt.translate._('editAttributes')
        });
        
        this.controls.editattrControl.events.on({
            'featurehighlighted': Giscuit.Util.bind(function(evt) {
                evt.feature.originalGeometry = evt.feature.geometry.clone();
                gt.control.editattr = new Giscuit.Mobile.Control.Editattr(evt.feature, 0);
            }, this) ,
            'featureunhighlighted': this.editFeatureUnhighlighted,
            scope: this
        });
        
        //
        this.controlPanel.addControls(this.controls.editattrControl);
    },
    
    /**
     * Gpsnode
     */
    gpsnodeControl: function() {
        var wgs = new OpenLayers.Projection("EPSG:4326");
        
        this.controls.gpsnodeControl = new OpenLayers.Control({
            'displayClass': 'olControlGpsnode',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                if(! this.controls.addControl.active) {
                    return;
                }
                var panelDiv = this.controls.gpsnodeControl.panel_div;
                
                if(gt.control.geolocation.point) {
                    this.controls.addControl.insertXY(
                        gt.control.geolocation.point.geometry.x,
                        gt.control.geolocation.point.geometry.y
                    );
                } else {
                	Giscuit.Util.jqmMessage(gt.translate._('enableGeolocationTool'));
                	
                	/*
                    gt.control.geolocation.gl.getCurrentPosition(Giscuit.Util.bind(function(position) {
                        var currPosition = new OpenLayers.LonLat(position.coords.longitude, position.coords.latitude);
                        currPosition.transform(wgs, gt.map.projection);
                        this.controls.addControl.insertXY(currPosition.lon, currPosition.lat);
                    }, this), 
                    null, 
                    {
                        'enableHighAccuracy': true ,
                        'maximumAge': 0,
                        'timeout': 3 * 60 * 1000 
                    });
                    */
                }
            }, this),
            title: gt.translate._('addVertexAtCurrentPosition')
        });
        
        //
        this.controls.addControl.events.on({
            "activate": function() {
                this.controls.gpsnodeControl.panel_div.style.display = 'block';
            },
            "deactivate": function() {
                this.controls.gpsnodeControl.panel_div.style.display = 'none';
            },
            scope: this
        });
        
        //
        this.secondaryControlPanel.addControls(this.controls.gpsnodeControl);
        this.controls.gpsnodeControl.panel_div.style.display = 'none';
    },
    
    /**
     * Finalize
     */
    finalizeControl: function() {
        this.controls.finalizeControl = new OpenLayers.Control({
            'displayClass': 'olControlFinalize',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                if(! this.controls.addControl.active) {
                    return;
                }
                
                this.controls.addControl.handler.finishGeometry();
            }, this),
            title: gt.translate._('finalizeSketch')
        });
        
        //
        this.controls.addControl.events.on({
            "activate": function() {
                if(this.controls.addControl.displayClass == 'olControlAddPoint') {
                    return;
                }
                
                this.controls.finalizeControl.panel_div.style.display = 'block';
            },
            "deactivate": function() {
                this.controls.finalizeControl.panel_div.style.display = 'none';
            },
            scope: this
        });
        
        //
        this.secondaryControlPanel.addControls(this.controls.finalizeControl);
        this.controls.finalizeControl.panel_div.style.display = 'none';
    },
    
    invalidEditScale: function(editMaxscale) {
        Giscuit.Util.jqmMessage(
            gt.translate._('layerIsEditableFromScale') + ' 1:' + editMaxscale 
        );
    },
    
    //Initialize Controls
    initializeControls: function (data) {
        this.controls = {};
        
        //Activate snap
        if(typeof(gt.layer.snap) != 'undefined') {
            gt.layer.snap.setVisibility(true);
        }
        if (typeof(gt.snap.edit) != 'undefined') {
            gt.snap.edit.activate();
        }
        
        if(gt.control.offline.isOnline) {
            //Invalid edit scale message
            if(gt.map.getScale() > data.edit_maxscale) {
                this.invalidEditScale(data.edit_maxscale);
            }
            
            //Style
            this.editLayer.styleMap.styles['default'].rules = [];
            this.editLayer.styleMap.styles['default'].addRules(gt.map.getStyleRules(data.sld, true));
        
            if(this.editLayer && this.editLayer.map) {
                this.editLayer.setVisibility(true);
                gt.map.refresh();
            }
        }

        //Initialize panels
        this.controlPanel = new OpenLayers.Control.Panel({
            'displayClass': 'gtEditgeomToolbar'
        });
        this.secondaryControlPanel = new OpenLayers.Control.Panel({
            'displayClass': 'gtEditgeomToolbar gtEditgeomSecondaryToolbar'
        });

        //Primary panel controls
        if (data.editgeom) {
            this.addControl(data.type);
            
            this.editControl();
            gt.map.addBboxHandler(this.controls.editControl.selectControl, this.editLayer, gt.map.selectTolerance);
            this.touchOffset();
            
            this.editattrControl();
            gt.map.addBboxHandler(this.controls.editattrControl, this.editLayer, gt.map.selectTolerance);
            
            this.backControl(data.history, data.changesCount);
            this.deleteControl();
            gt.map.addBboxHandler(this.controls.deleteControl, this.editLayer, gt.map.selectTolerance);

        } else {
            this.editattrControl();
        }

        this.layerNameControl();

        //
        gt.map.addControl(this.controlPanel);
        
        if (data.editgeom) {
            //Secondary panel controls
               this.editsaveControl();
            this.finalizeControl();
            this.gpsnodeControl();
            
            gt.map.addControl(this.secondaryControlPanel);
        }
        
        //
        if(data.changesCount > 0) {
            this.backActivate();
        }
    },
    
    featuremodified: function() {
        if(typeof(this.controls.editsaveControl) != 'undefined') {
            this.controls.editsaveControl.panel_div.className = 'olControlEditsaveItemActive olButton';
        }
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Editgeom"
});