/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Geolocation.js 1749 2011-09-15 11:10:45Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.geolocation = new Giscuit.Mobile.Control.Geolocation();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Geolocation
 * 
 *  Inherits from:
 *   - <Giscuit.Control.Geolocation>
 */
Giscuit.Mobile.Control.Geolocation = OpenLayers.Class(Giscuit.Control.Geolocation, {
    
    orientationWatchProcess: null,
    
    heading: 0,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        //this.initializeOrientation();
        
        try {
            if(typeof(navigator.geolocation) != 'undefined') {
                this.gl = navigator.geolocation;
            } else if (typeof(google) != 'undefined') {
                this.gl = google.gears.factory.create('beta.geolocation');
            } else {
                debug.warn('Geolocation not supported');
            }
        } catch(e) {
            debug.warn('Geolocation not supported');
        }
        
        if(this.gl != null) {
            $(document).ready(Giscuit.Util.bind(function() {
                $("#gtMapControls").append('<div>'
                    + '<div class="ui-bar ui-bar-c" id="gtGeolocationButton">'
                    + '<div class="gtMapControlsElImage" style="background-position: -64px 50%;"></div>'
                    + '<div class="gtMapControlsElText">' + gt.translate._('location') + '</div>'
                    + '</div></div>');
                            
                $("#gtGeolocationButton").bind(gt.config.clickEvt, function(evt) {
                    if($(this).hasClass('ui-btn-active')) {
                        self.stop();
                        $(this).removeClass('ui-btn-active');
                    } else {
                        self.start();
                        $(this).addClass('ui-btn-active');
                    }
                    
                    return false;
                });
            }, this));
        }
    },
    
    initializeOrientation: function() {
        if (! ('ondeviceorientation' in window)) {
            return;
        }
        
        var heading, 
            timerId,
            self = this;

        this.pointStyle = {
            'pointRadius': 8,
            'externalGraphic': gt.config.httpImagePath + '/up.png',
            'rotation': 0
        };
                
        var setHeading = function(evt) {
            var direction, heading;
            
            if (typeof(evt.webkitCompassHeading) != 'undefined') {
                direction = evt.webkitCompassHeading;
                if (typeof(window.orientation) != 'undefined') {
                    direction += window.orientation;
                }
            } else {
                direction = 360 - evt.alpha;
            }
            
            heading = direction;
            while (heading >= 360) {
                heading -= 360;
            }
            while (heading < 0) {
                heading += 360;
            }
            heading = Math.round(heading);
            
            /*
            if(Math.abs(heading - self.heading) > 3) {
                $('#gtMapDiv').html(heading);
                self.point.style.rotation = heading;
            }
            */
            
            self.heading = heading;
        };
        
        window.addEventListener('deviceorientation', function(evt) {
            clearTimeout(timerId);
            timerId = setTimeout(function() {
                if(self.point) {
                    setHeading(evt);
                }
            }, 100);
        }, true);
    },
    
    error: function(error) {
        $("#gtGeolocationButton").removeClass('ui-btn-active');
        
        var msg = '';
        switch(error.code) {
            case error.PERMISSION_DENIED: msg = 'geolocationDataNotShared';
            break;
        
            case error.POSITION_UNAVAILABLE: msg = 'geolocationPositionNotDetected';
            break;
            
            case error.TIMEOUT: msg = 'geolocationTimedOut';
            break;
            
            default: msg = 'geolocationUnknownError';
            break;
        }
                
        Giscuit.Util.jqmMessage(gt.translate._(msg));
        this.stop();
    },
    
    start: function () {
        $("#gtGeolocationButton").addClass('ui-btn-active');
        
        Giscuit.Control.Geolocation.prototype.start.apply(this, [arguments]);
    },
    
    stop: function () {
        $("#gtGeolocationButton")
            .find('.gtMapControlsElText')[0]
            .innerHTML = gt.translate._('location');
        $("#gtGeolocationButton").removeClass('ui-btn-active');
        
        Giscuit.Control.Geolocation.prototype.stop.apply(this, [arguments]);
    },
    
    updateCallback: function(position) {
        $("#gtGeolocationButton")
            .find('.gtMapControlsElText')[0]
            .innerHTML = (Math.round(position.coords.accuracy * 100) / 100) + 'm';
    },
    
    CLASS_NAME: "Giscuit.Mobile.Util.Geolocation"
});