/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 *
 *
 * @version    $Id: Link.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 *
 */

//
Giscuit.Mobile.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.link = new Giscuit.Mobile.Control.Link();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Link
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Link = OpenLayers.Class(Giscuit.Control, {
    
    drawControl: null,
    
    features: [],
    
    markerExists: false,
    
    initialize: function (options) {
        
        if(typeof(gt.control.offline) != 'undefined') {
            gt.control.offline.events.on({
                'online': this.initializeLayer,
                'offline': this.initializeLayer,
                'scope': this
            });
        }
        
        //FIMXE: initializeLayer errors if ran in cold start mode
        if(typeof(gt.control.offline) == 'undefined' ||
            gt.control.offline.isOnline) {
        
            this.initializeLayer();
        }
        
        //Populate markers from address bar
        this.populate();
        
        //Initialize page and events
        $(document).ready(Giscuit.Util.bind(function() {
            $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtLinkButton">'
                + '<div class="gtMapControlsElImage" style="background-position: -640px 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('share') + '</div>'
                + '</div></div>');
            
            var contentHtml = '<button type="button" id="gtAddMarkerButton">' + gt.translate._('addMarker') + '</button><p>' +
                '<button type="button" id="gtLinkEmailButton">' + gt.translate._('sendToEmail') + '</button></p>' +
                '<fieldset class="gtLinkInputs"><legend>' + gt.translate._('desktopLayout') + '</legend><p>' + gt.translate._('pasteUrl') + 
                ': <br><input type="text" id="gtLinkUrl" style="width: 95%;"></p>';
                
            if(gt.config.guest == 't') {    
                contentHtml += '<p style="margin-bottom: 0px;">' + gt.translate._('pasteHtml') + 
                    ': <br><input type="text" id="gtLinkHtml" style="width: 95%;"></p>';
            }
            contentHtml += '</fieldset><br>';
            
            contentHtml += '<fieldset class="gtLinkInputs"><legend>' + gt.translate._('mobileLayout') + '</legend><p>' + gt.translate._('pasteUrl') + 
            ': <br><input type="text" id="gtLinkUrlMobile" style="width: 95%;"></p>';
            
            if(gt.config.guest == 't') {    
                contentHtml += '<p style="margin-bottom: 0px;">' + gt.translate._('pasteHtml') + 
                    ': <br><input type="text" id="gtLinkHtmlMobile" style="width: 95%;"></p>';
            }
            contentHtml += '</fieldset>';
            
            var html = 
            '<div data-role="page" id="gtLink" data-dom-cache="true" data-url="gtLink">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('markAndShare') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
         
            '<div data-role="content">' + 
                contentHtml +
            '</div>' +    
            '</div>';
            
            $.mobile.pageContainer.append(html);
            $('#gtLink').page();
            
            // 
            $("#gtLinkButton").bind(gt.config.safeClickEvt, this, function(evt) {
                if(evt.data.active) {
                    evt.data.deactivate();
                } else {
                    evt.data.activate();
                }
                
                return false;
            });
            
            $("#gtAddMarkerButton").bind(gt.config.safeClickEvt, Giscuit.Util.bind(function () {
                this.add();
                
                return false;
            }, this));
            
            $("#gtLinkEmailButton").bind(gt.config.clickEvt, Giscuit.Util.bind(function () {
                var subj = encodeURIComponent(gt.translate._('mapLink'));
                
                var desktopUrl = encodeURI(decodeURIComponent($('#gtLinkUrl').val()));
                var mobileUrl = encodeURI(decodeURIComponent($('#gtLinkUrlMobile').val()));
                var body = encodeURIComponent(
                	gt.translate._('emailLinkText')
                		.replace("%1$s", desktopUrl)
                		.replace("%2$s", mobileUrl)
                );

                window.location = 'mailto:?subject=' + subj + '&body=' + body;
                
                return false;
            }, this));

        }, this));
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
    },

    initializeLayer: function() {
        var control = gt.map.globalSelectControl;
        
        for(var i=0; i < control.layers.length; i++) {
            if(! control.layers[i] || ! control.layers[i].map) {
                control.layers.splice(i, 1);
            }
        }
        
        control.layers.push(gt.layer.marker);
           control.setLayer(control.layers);
           
           gt.layer.marker.events.on({
            "featureselected": this.onFeatureSelect ,
            scope: this
        });
    },
    
    activate: function () {
        gt.map.deactivateAll();
        var activated = Giscuit.Control.prototype.activate.call(this);
                        
        if (activated == true) {
            gt.map.globalSelectControl.activate();
            
            //Hide map controls
            $("#gtMapControls").css('display', 'none');
            $("#gtToggleControlsButton").css('bottom', '5px').removeClass('ui-btn-active');
            
            //
            this.change();
            
            //
            $.mobile.changePage($('#gtLink'));
        }
        
        return activated;
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if (deactivated == true) {
            //
            gt.map.globalSelectControl.deactivate();
            
            //
            if(this.drawControl) {
                this.drawControl.deactivate();
                this.drawControl.destroy();
                this.drawControl = null;
            }
            
            //
            $("#gtLinkButton").removeClass('ui-btn-active');
        }

        return deactivated;
    },
    
    add: function() {
        this.drawControl = new OpenLayers.Control.DrawFeature(gt.layer.marker, OpenLayers.Handler.Point, {
            callbacks: {
                "done": OpenLayers.Function.bind(this.done, this)
            }
        });
        gt.map.addControl(this.drawControl);
        this.drawControl.activate();
        
        $("#gtLinkButton").addClass('ui-btn-active');
        
        $.mobile.changePage($('#gtMap'));
        //Giscuit.Util.jqmMessage(gt.translate._('drawMarkerHelp'));
    },
    
    populate: function () {
        var pathName = window.location.pathname.split('/');
        var markersHash = '';

        for (var i=0; i < pathName.length; i++) {
            if(pathName[i] == 'markers') {
                markersHash = decodeURIComponent(pathName[i + 1]);
                this.markerExists = true;
            }
        }

        if (this.markerExists == false) {
            return;
        } 

        var markers = markersHash.split('|');

        for (var i = 3; i < markers.length; i+=4) {
            this.createMarker({
                type: parseInt(markers[i-3]),
                x: parseFloat(markers[i-2]),
                y: parseFloat(markers[i-1]),
                text: markers[i] ,
                action: 'populate'
            });
        }
    },
    
    change: function() {
        var pathName = window.location.pathname.split('/');
        var lon,lat,zoom,layers,markers = '';
        var visibleLayers = [];
        var center = gt.map.getCenter();
        
        lat = center.lat.toFixed(6);
        lon = center.lon.toFixed(6);
        zoom =  gt.map.getZoom();
        
        for (var i = 0; i < this.features.length; i++) {
            if ( this.features[i].data != null) {
                markers += this.features[i].data.type + '|' +
                    this.features[i].geometry.x + '|' +
                    this.features[i].geometry.y + '|' +
                    this.features[i].markerText + '|';
            }
        }

        $('.ui-btn-active').find('.gtLayersMenuName').each(function(i, el) {
            var layerId = $(el).data('layer-id');
            visibleLayers = visibleLayers.concat(layerId);
        });
        
        var desktopParams = 'lat=' + lat + '&lon=' + lon + '&zoom=' + zoom;
        var mobileParams = '/lat/' + lat + '/lon/' + lon + '/zoom/' + zoom;
        if (markers != '') {
            desktopParams += '&markers=' + encodeURIComponent(markers).replace(/%0A/g, '%0D').replace(/%0D%0D/g, '%0D');
            mobileParams += '/markers/' + encodeURIComponent(markers).replace(/%0A/g, '%0D').replace(/%0D%0D/g, '%0D');
        }
        if (gt.config.guest == 't' && visibleLayers.length != 0) {
            desktopParams += '&layers=' + visibleLayers.join();
            mobileParams += '/layers/' + visibleLayers.join();
        }

        //Desktop
        var urlValue = window.location.protocol + '//' + window.location.host + gt.config.httpPath + '/map#' + desktopParams;
        $('#gtLinkUrl').first().val(urlValue);
            
        //Mobile
        var httpPathMobile = gt.config.httpPath.substring(0, gt.config.httpPath.length - 7) + 'mobile';
        var urlValueMobile = window.location.protocol + '//' + window.location.host + httpPathMobile + '/map/index' + mobileParams;

        $('#gtLinkUrlMobile').first().val(urlValueMobile);
            
        if(gt.config.guest == 't') {   
            //Desktop
            var htmlValue = '<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="' +
                window.location.protocol + '//' + window.location.host + gt.config.httpPath + '/map/index/type/link#' + desktopParams +
                '"></iframe>';
            $('#gtLinkHtml').first().val(htmlValue);
           
            //Mobile
            var htmlValueMobile = '<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="' +
                urlValueMobile + '/type/link' +
                '"></iframe>';
            $('#gtLinkHtmlMobile').first().val(htmlValueMobile);
        }
    },
    
    createMarker: function (params) {
        var geom = new OpenLayers.Geometry.Point(params.x, params.y);
        var feature = new OpenLayers.Feature.Vector(geom, {type: params.type});
        var text = params.text;

        feature.markerText = text;
        
        this.features.push(feature);
        gt.layer.marker.addFeatures([feature]);
        
        if (params.action == 'create') {
            this.onFeatureSelect({'feature': feature});
        }
    },
    
    done: function (geom) {
        this.createMarker({
            type: 1,
            x: geom.x,
            y: geom.y,
            text: '',
            action: 'create'
        });
        
        //
        this.drawControl.deactivate();
        this.drawControl.destroy();
        this.drawControl = null;
        
        //
        $("#gtLinkButton").removeClass('ui-btn-active');
    },
    
    onFeatureSelect: function(e) {
        var html = 
        '<div data-role="page" id="gtLinkMarkerDialog" data-dom-cache="true">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('markerText') + '</h1>' +
                '<a id="gtLinkMarkerDelete" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('delete') +
                '</a>' +
                '<a id="gtLinkMarkerSave" data-icon="gear" class="ui-btn-left">' +
                    gt.translate._('save') +
                '</a>' +
            '</div>' +
            '<div data-role="content">' + 
                '<textarea id="gtLinkMarkerText">' + e.feature.markerText + '</textarea>' +
            '</div>' +
        '</div>';
                    
        $('#gtLinkMarkerDialog').unbind().remove();
        $.mobile.pageContainer.append(html);
        gt.map.globalSelectControl.unselect(e.feature);

        $("#gtLinkMarkerDelete").bind(gt.config.clickEvt, Giscuit.Util.bind(function() {
            gt.layer.marker.removeFeatures([e.feature]);
            e.feature.destroy();

            $.mobile.changePage('#gtMap');
            
            return false;
        }, this));
        
        $("#gtLinkMarkerSave").bind(gt.config.clickEvt, Giscuit.Util.bind(function() {
            e.feature.markerText = this.filter($("#gtLinkMarkerText").val());
            
            $.mobile.changePage('#gtMap');
            
            return false;
        }, this));
        
        $.mobile.changePage('#gtLinkMarkerDialog');
        
        $('#gtLinkMarkerText').focus();
        
        //Remove tooltip
        $('.ui-loader').each(function (evt) {
            $(this).css('display', 'none');
        });
    },
    
    filter: function(str) {
        str = str.replace('|', ' ');
        
        return str;
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Link"
});
