/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Info.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Offline
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Offline = OpenLayers.Class(Giscuit.Control, {
    
    isOnline: null,
    
    downloadIds: [],
    
    overwriteIds: {},
    
    skipIds: {},
    
    currDownloadId: null,
    
    db: null,
    
    tiles: {},
    
    tilesCount: {},
    
    maxDownloadTiles: 2000,
    
    initialize: function (options) {
        //Deprecated in 2.12
        //this.EVENT_TYPES.push('online');
        //this.EVENT_TYPES.push('offline');
                
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        //
        if(typeof (window.openDatabase) !== "function" ) {
            this.isOnline = true;
            return;
        }
          
        //
        this.getStatus();
        
        //
        $(document).ready(Giscuit.Util.bind(function() {

            if(gt.config.guest == 't' || gt.config.offlineAllowed == 'f') {
                return;
            }
            
            //Toggle online, offline
            $(document).keydown(Giscuit.Util.bind(function(e) {
                var key = 'O';
                if(e.keyCode == key.charCodeAt(0) && e.ctrlKey && e.shiftKey) {
                    if(this.isOnline) {
                        this.setStatus(false);
                    } else {
                        this.setStatus(true);
                    }
                    return false;
                }
            }, this));
            
            //Toggle layer
            $('.gtOfflineMenuName').bind(gt.config.clickEvt, this, function(evt) {
                var el = this.parentNode.parentNode.parentNode;
                var deleteEl = el.children[1];
                
                if($(el).hasClass('ui-btn-active')) {
                    $(el).removeClass('ui-btn-active');
                    $(deleteEl).removeClass('ui-btn-active');
                } else if(! $(this).hasClass('ui-disabled')) {
                    $(el).addClass('ui-btn-active');
                    $(deleteEl).addClass('ui-btn-active');
                }
                
                return false;
            });
            
            //Delete
            $('.gtOfflineMenuDelete').bind(gt.config.clickEvt, this, function(evt) {
                var layerId = $($(this.parentNode).find('*[data-layer-id]')[0]).jqmData('layer-id');
                
                if(confirm(gt.translate._('confirmDeleteDbTable'))) {
                    evt.data.deleteLayer(layerId);
                }
                
                return false;
            });
            
            //Download
            $('#gtOfflineDownloadButton').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, this, function(evt) {
                evt.data.downloadInitialize();
                
                return false;
            });
            
            //Offline
            $('#gtOfflineGoOfflineButton').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, this, function(evt) {
                $(this).button('disable');
                
                evt.data.setStatus(false);
                
                return false;
            });
            
            //Online
            $('#gtOfflineGoOnlineButton').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, this, function(evt) {
                $(this).button('disable');
                
                evt.data.setStatus(true);
                
                return false;
            });
            
            //Count layer features
            $('#gtOffline').live('pagebeforeshow ', this, function(evt) {
                if(evt.data.isOnline) {
                    evt.data.setFeaturesCount();
                } else {
                    evt.data.setEditErrors();
                }
                
                //
                var resolutions = gt.map.resolutions;
                var units = gt.map.getUnits();
                var select = $('#gtOfflineMaxDownloadScale').get(0);
                
                if(! select.options.length) {
                    for(var i = 0; i < resolutions.length; i++) {
                        var scale = Math.round(OpenLayers.Util.getScaleFromResolution(resolutions[i], units));
                        var elOptNew = document.createElement('option');
                        elOptNew.text = '1:' + scale;
                        elOptNew.value = i;
                        try {
                            select.add(elOptNew, null);
                        }
                        catch(ex) {
                            select.add(elOptNew);
                        }
                    }
                    select.selectedIndex = resolutions.length - 1;
                    $('#gtOfflineMaxDownloadScale').selectmenu('refresh');
                }
                
                $('#gtOfflineMaxDownloadScaleLabel').html(
                    gt.translate._('maximumDownloadScale') + 
                    ' (' + 
                    gt.translate._('currentScale') + 
                    ' - 1:' + Math.round(OpenLayers.Util.getScaleFromResolution(resolutions[gt.map.zoom], units)) + 
                    ')'
                );
            });
            
            this.addOfflineButton();
            
            this.changeOfflineLayout(this.isOnline);

        }, this));
    },
    
    
    addOfflineButton: function() {
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtOfflineButton">'
            + '<div class="gtMapControlsElImage" style="background-position: -576px 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('offline') + '</div>'
            + '</div></div>');
        
        //FIXME:
        $("#gtOfflineButton").die(gt.config.safeClickEvt).live(gt.config.safeClickEvt, Giscuit.Util.bind(function() {
            //Hide map controls
            $("#gtMapControls").css('display', 'none');
            $("#gtToggleControlsButton").css('bottom', '5px').removeClass('ui-btn-active');
            
            $.mobile.changePage($('#gtOffline'));
            
            return false;
        }, this));
    } ,
    
    
    changeOfflineLayout: function(status) {
        if(status) {
            $('#gtOffline .online').css('display', 'block');
            $('#gtOffline .offline').css('display', 'none');
        } else {
            $('#gtOffline .online').css('display', 'none');
            $('#gtOffline .offline').css('display', 'block');
        }
    } ,
           
    getStatus: function () {
        if (navigator.onLine) {
            if(this.isOnline === null) {
                this.setStatus(true);
            }
        } else {
            this.setStatus(false);
        }
    } ,
    
    setStatus: function(status) {
        var lastIsOnline = this.isOnline;
        
        if(status !== lastIsOnline) {
            //Status change
            if(lastIsOnline !== null) {
                if(status) {
                    this.synchronize();
                } else {
                    this.isOnline = false;
                    this.wentOffline();
                }
            //Initial status
            } else {
                if(status) {
                    //Clear layer changes
                    Giscuit.Util.removeStorageObject('gtLayersChanges');
                } else {

                }
                
                this.isOnline = status;
            }
        }
    } ,
    
    synchronize: function() {
        this.synchronizeLayers();
    } ,
    
    synchronizeLayers: function() {
        var layers = gt.control.layers.records;
        var items = ['id', 'view', 'select', 'label', 'edit', 'label_size', 'label_color', 'label_fields', 'order'];
        var boolItems = ['view', 'select', 'label', 'edit'];
        var result = [];
        
        for(var i=0; i < layers.length; i++) {
            if(typeof(layers[i].changed) != 'undefined' && layers[i].changed === true) {
                var row = {}; 
                for(var obj in layers[i]) {
                    if(Giscuit.Util.inArray(boolItems, obj)) {
                        row[obj] = (layers[i][obj] === true) ? 't' : 'f' ;
                    } else if(Giscuit.Util.inArray(items, obj)) {
                        row[obj] = layers[i][obj];
                    }
                }
                result.push(row);
            }
        }
        

        var jsonFormat = new OpenLayers.Format.JSON();
        var url =  gt.config.httpMobilePath + '/privatelayers/settingsbulk';
        
        $.ajax({
            'cache': false ,
            'timeout': 5000 ,
            'type': 'POST',
            'url': url,
            'data': 'data=' + jsonFormat.write(result),
            'success': this.synchronizeLayersSuccess ,
            'error': function() {
                Giscuit.Util.message(gt.translate._('noInternetConnection'));
            } ,
            'dataType': 'json' ,
            'context': this
        });
    } ,
    
    synchronizeLayersSuccess: function(data) {
        //FIXME:
        gt.control.editgeom.deactivate();
        
        Giscuit.Util.updateApplicationCache();
        
        this.isOnline = true;
        this.wentOnline();
        
        //
        this.synchronizeEditing();
    },
    
    synchronizeEditing: function() {
        var self = this;
        
        var layerType = Giscuit.Util.getStorageObject('gtLayerType');
        
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        editErrors = {};
        Giscuit.Util.setStorageObject('gtEditErrors', editErrors);
        
        this.getDbMetadata(function (metadata) {
            for(var obj in metadata) {
                if(layerType[metadata[obj].name] != 'vector') {
                    continue;
                }
                
                var table = new Giscuit.Util.OfflineEditing({tableName: metadata[obj].name});
                table.read({
                    withHStatus: true,
                    //FIXME: layer id is bind to this
                    callback: Giscuit.Util.bind(function(result) {
                        var records = [];
                        
                        //FIXME:
                        var layerId = this;
                                                
                        var len = result.features.length, i;
                        for (i = 0; i < len; i++) {
                           var row = result.features[i].attributes;
                           row.the_geom = result.features[i].geometry.toString();
                           row.__fid = result.features[i].fid;
                           records.push(row);
                        }
                        
                        if(! records.length) {
                            return;
                        }
                        
                        var overwriteIdsValue = '/overwrite';
                        if(typeof(self.overwriteIds[layerId]) != 'undefined') {
                             overwriteIdsValue += '/' + self.overwriteIds[layerId].join(','); 
                        }
                        
                        var skipIdsValue = '/skip';
                        if(typeof(self.skipIds[layerId]) != 'undefined') {
                             skipIdsValue += '/' + self.skipIds[layerId].join(','); 
                        }
                        
                        editErrors[layerId] = false;
                                        
                        var jsonFormat = new OpenLayers.Format.JSON();
                        var url =  gt.config.httpMobilePath + 
                            '/editgeom/bulk/id/' + layerId + overwriteIdsValue + skipIdsValue;
                
                        $.ajax({
                            'cache': false ,
                            'type': 'POST',
                            'url': url,
                            'data': 'data=' + jsonFormat.write(records),
                            'success': self.synchronizeEditingSuccess ,
                            'error': Giscuit.Util.jqmAjaxError ,
                            'dataType': 'json' ,
                            'context': self
                        });
                    }, obj)
                });
                
            }
        });
    } ,
        
    synchronizeEditingSuccess: function(data) {
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        var format = new OpenLayers.Format.GeoJSON();
        
        if(data.errors) {
            editErrors[data.layerId] = data.errors;
        } else {
            delete editErrors[data.layerId];
        }
        
        var table = new Giscuit.Util.OfflineEditing({tableName: 'layer_' + data.layerId});
        table.read({
            withHStatus: true,
            callback: Giscuit.Util.bind(function(result) {
                //
                var len = result.features.length, a;
                for (a = 0; a < len; a++) {
                   var feature = result.features[a];
                   
                   if(typeof(editErrors[data.layerId]) == 'undefined' ||
                       typeof(editErrors[data.layerId][feature.fid]) == 'undefined') {
                       
                       if(feature.attributes.hstatus == 'D') {
                           table['delete'](feature);
                       } else if (feature.attributes.hstatus == 'U') {
                           feature.attributes.__hdate = data.timestamp;
                           delete feature.attributes.hstatus;
                           
                           table.update(feature);
                       } else if (feature.attributes.hstatus == 'C') {
                           feature.attributes.gid = data.createRecords[feature.fid];
                           feature.attributes.__hdate = data.timestamp;
                           delete feature.attributes.hstatus;
                           
                           table.update(feature);
                       }
                   
                   }

                   //Update features
                   for(var b=0; b < data.updateRecords.length; b++) {
                       if(data.updateRecords[b].properties.gid == feature.attributes.gid) {
                           var newFeature = format.read(data.updateRecords[b], "Feature");
                           newFeature.attributes.__hdate = data.timestamp;
                           newFeature.fid = feature.fid;
                           table.update(newFeature);
                           
                           break;
                       }
                   }
                   
                   //Delete features
                   for(var b=0; b < data.deleteRecords.length; b++) {
                       if(data.deleteRecords[b] == feature.attributes.gid) {
                           table['delete'](feature);
                           
                           break;
                       }
                   }
                }

                //
                if(typeof(this.skipIds[data.layerId]) != 'undefined') {    
                    delete this.skipIds[data.layerId];
                }
                if(typeof(this.overwriteIds[data.layerId]) != 'undefined') {
                    delete this.overwriteIds[data.layerId];
                }
            }, this)
        });
        
        //TODO: Stop sync animation
        
        //Finished
        var finished = true;
        var withErrors = false;
        for(var obj in this.syncStatus) {
            withErrors = true;
            if(editErrors[obj] === false) {
                finished = false;
                break;
            }
        }
        if(finished) {
            for (var error in editErrors) {
                var errorsExist = true;
            }
            if (typeof errorsExist == 'undefined') {
                Giscuit.Util.jqmMessage(gt.translate._('synchronizeSuccess'));
            } else {
                Giscuit.Util.jqmMessage(gt.translate._('synchronizeFailed'));
            }
            Giscuit.Util.setStorageObject('gtEditErrors', editErrors);
            gt.map.refresh();
        }
    },
    
    wentOnline: function() {
        debug.info('Online');
        
        //
        gt.map.deactivateAll();
        gt.control.loading.setVisible(true);
                
        //Layers
        gt.map.removeAll();
        gt.control.layers.add();
        
        //Buttons
        this.changeOfflineLayout(true);
        gt.control.home.addHomeButton();
        
        //Show online layers
        $('.gtLayersMenuName').each(function(index, value) {
            if($(value).parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $(value).parent();
            } else if($(value).parent().parent().parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $(value).parent().parent().parent();
            }
                        
            parent.css('display', 'block');
        });
        
        //Show containers
        $('.gtLayersMenuContainer').each(function(index, value) {
            if($(value).parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $(value).parent();
            } else if($(value).parent().parent().parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $(value).parent().parent().parent();
            }
            
               parent.css('display', 'block');
        });

        //Show online layers in order
        $('.gtLayersOrderControlGroup').each(function(index, value) {
            $(value).parent().css('display', 'block');
        });
        gt.control.layers.changePosition(null, '');
        
        //
        this.events.triggerEvent('online');
        gt.init.layout.renderMapControls();
        
        //
        $('#gtOfflineGoOnlineButton').button('enable');
        
        //
        $.mobile.changePage($('#gtMap'));
    } ,
    
    wentOffline: function() {
        debug.info('Offline');
        
        //
        gt.map.deactivateAll();
        gt.control.loading.setVisible(false);
        
        //Layers
        gt.map.removeAll();
        gt.control.layers.add();
        
        //Buttons
        gt.control.home.deleteHomeButton();
        this.changeOfflineLayout(false);
                
        //
        $('#gtOfflineGoOfflineButton').button('enable');
        
        $.mobile.changePage($('#gtMap'));
    } ,
    
    downloadInitialize: function() {
        $('#gtOfflineDownloadButton').button('disable');
        $('.gtOfflineMenuDelete').each(function(index, value) {
            $(this).addClass('ui-disabled');
        });
        
        this.downloadIds = [];
        
        //Show only layers available for offline
        $('.gtOfflineMenuName').each(Giscuit.Util.bind(function(index, value) {
            if($(value).parent().parent().parent().hasClass('ui-btn-active')) {
                this.downloadIds.push($(value).jqmData('layer-id'));
            }
        }, this));
        
        this.downloadGet();
    } ,
    
    downloadGet: function() {
        //
        if(this.downloadIds.length == 0) {

            var enableButton = true;
            for(var obj in this.tiles) {
                enableButton = false;
            }
            if(enableButton) {
                $('#gtOfflineDownloadButton').button('enable');
                $('.gtOfflineMenuDelete').each(function(index, value) {
                    $(this).removeClass('ui-disabled');
                });
            }
            
            return;
        }
        
        //
        var id = this.downloadIds.shift();
        var where = gt.map.getExtent().toBBOX();
        var url =  gt.config.httpMobilePath + 
            '/offline/download/id/' + id + '/where/' + where;

        this.currDownloadId = id;
        
        $.ajax({
            'type': 'GET' ,
            'url': url ,
            'success': this.downloadSet ,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this 
        });
    } ,
    
    
    
    downloadSet: function(data) {
        var self = this;
        var layer = gt.control.layers.getLayerRecordById(this.currDownloadId);
        
        $('.gtOfflineMenuName').parent().parent().parent().removeClass('ui-btn-active');
        $('.gtOfflineMenuName').parent().parent().next().removeClass('ui-btn-active');
        
        if(data.tilecache) {
            var db = this.getDbConnection();
            this.tiles[layer.id] = {};

            this.tilesCount[layer.id] = 0;
            var maxZoomLevel = parseInt($('#gtOfflineMaxDownloadScale').val()) + 1;
            if(maxZoomLevel <= gt.map.zoom) {
                maxZoomLevel = gt.map.zoom + 1;
            }            
            
            for(var zoom = gt.map.zoom; zoom < maxZoomLevel; zoom++) {
                var tiles = this.getTileURLs(gt.map.getExtent(), zoom);
                
                this.tilesCount[layer.id] += tiles.length;
                this.tiles[layer.id][zoom] = tiles; 
            }
            
            if(this.tilesCount[layer.id] > this.maxDownloadTiles) {
                var error = layer.name + ' ' + gt.translate._('wasNotDownloaded') + ', ' + gt.translate._('tooManyFeaturesRequested') + '.';
                alert(error);
                
                this.currDownloadId = null;
                this.downloadGet();
                return;
            }
            
            //
            $('.gtOfflineMenuName').each(function(index, value) {
                var layerId = $(value).jqmData('layer-id');
                
                if(layerId != layer.id) {
                    return;
                }
                
                var countEl = $($(value).find('.ui-li-count')[0]);
                countEl.html('0/' + self.tilesCount[layer.id]);
                countEl.css('display', 'block');
            });
            
            var sql = "CREATE TABLE IF NOT EXISTS layer_" + layer.id + 
                " (url TEXT PRIMARY KEY, image TEXT)";
            
            db.transaction(function (tx) {
                tx.executeSql(sql, [], function (tx, results) {
                    var sql = 'DELETE FROM layer_' + layer.id;
                    
                    db.transaction(function (tx) {
                        tx.executeSql(sql, [], function (tx, results) {
                            self.downloadTiles(layer, data.tilecache.tmpname);
                        }, Giscuit.Util.websqlFailure);
                    });
                }, Giscuit.Util.websqlFailure);
            });

            //Layer type
            var layerType = Giscuit.Util.getStorageObject('gtLayerType');
            layerType['layer_' + layer.id] = 'raster';
            Giscuit.Util.setStorageObject('gtLayerType', layerType);
            
            //
            this.currDownloadId = null;
            this.downloadGet();
            
        } else {
            if(data.error) {
                var error = layer.name + ' ' + gt.translate._('wasNotDownloaded') + ', ' + gt.translate._(data.error) + '.';
                alert(error);
                
                this.currDownloadId = null;
                this.downloadGet();
                return;
            }
            
            //Records
            var table = new Giscuit.Util.OfflineEditing({tableName: 'layer_' + layer.id});
            table.clear();
            table.initialCreate(data.records, {
                timestamp: data.timestamp,
                callback: function(result) {
                    if(result.code == OpenLayers.Protocol.Response.FAILURE) {
                        var error = result.priv.error.message;
                        
                        Giscuit.Util.jqmMessage(error);
                        debug.warn(error);
                    }
                }
            });
            
            //Style
            var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
            mapSLD['layer_' + layer.id] = data.sld;
            Giscuit.Util.setStorageObject('gtMapSLD', mapSLD);
            
            //Form
            if(data.form) {
                var editForm = Giscuit.Util.getStorageObject('gtEditForm');
                editForm['layer_' + layer.id] = data.form;
                Giscuit.Util.setStorageObject('gtEditForm', editForm);
            }
            
            //Geometry type
            var mapGeometryType = Giscuit.Util.getStorageObject('gtMapGeometryType');
            mapGeometryType['layer_' + layer.id] = data.geometryType;
            Giscuit.Util.setStorageObject('gtMapGeometryType', mapGeometryType);
                    
            //Legend
            var layersLegend = Giscuit.Util.getStorageObject('gtLayersLegend');
            layersLegend['layer_' + layer.id] = data.legend;
            Giscuit.Util.setStorageObject('gtLayersLegend', layersLegend);
            
            //Layer type
            var layerType = Giscuit.Util.getStorageObject('gtLayerType');
            layerType['layer_' + layer.id] = 'vector';
            Giscuit.Util.setStorageObject('gtLayerType', layerType);
            
            //
            this.currDownloadId = null;
            this.downloadGet();
        
        }
    } ,
    
    getDbMetadata: function(callback) {
        var db = this.getDbConnection();
        var sql = "SELECT name FROM sqlite_master WHERE type='table' AND name != '__WebKitDatabaseInfoTable__'";
        
        db.transaction(function (tx) {
            tx.executeSql(sql, [], function (tx, results) {
               var countSQL = [];
               var tables = {};
                
               var len = results.rows.length, i;
               for (i = 0; i < len; i++) {
                   var name = results.rows.item(i).name;
                   countSQL.push("SELECT '" + name + "' as name, count(*) as count FROM " + name);
               }

               if(countSQL.length == 0) {
                   callback(tables);
                   return;
               }

               countSQL = countSQL.join(' UNION ');
               
               tx.executeSql(countSQL, [], function (tx, results) {
                   var len = results.rows.length, i;
                   for (i = 0; i < len; i++) {
                       var id = results.rows.item(i).name;
                       id = id.split('_')[1];
                       if(! id) {
                           continue;
                       }
                       
                       tables[id] = {
                           'name': results.rows.item(i).name,
                           'count': results.rows.item(i).count
                       };
                   }

                   callback(tables);
               }, Giscuit.Util.websqlFailure);
            }, Giscuit.Util.websqlFailure);
        });
    },
    
    deleteLayer: function(id) {
        var self = this;

        //Records
        var db = this.getDbConnection();
        var sql = 'DROP TABLE IF EXISTS layer_' + id;
        db.transaction(function (tx) {
            tx.executeSql(sql, [], function (tx, results) {
                self.setFeaturesCount();
            }, Giscuit.Util.websqlFailure);
        });
        
        //Style
        var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
        if(mapSLD['layer_' + id]) {
            delete mapSLD['layer_' + id];
        }
        Giscuit.Util.setStorageObject('gtMapSLD', mapSLD);
        
        //Form
        var editForm = Giscuit.Util.getStorageObject('gtEditForm');
        if(editForm['layer_' + id]) {
            delete editForm['layer_' + id];
        }
        Giscuit.Util.setStorageObject('gtEditForm', editForm);
        
        //Geometry Type
        var mapGeometryType = Giscuit.Util.getStorageObject('gtMapGeometryType');
        if(mapGeometryType['layer_' + id]) {
            delete mapGeometryType['layer_' + id];
        }
        Giscuit.Util.setStorageObject('gtMapGeometryType', mapGeometryType);
    },
    
    getDbConnection: function() {
        if(this.db !== null) {
            return this.db;
        }

        var dbParams = Giscuit.Util.OfflineEditing.prototype;
        this.db = openDatabase(dbParams.databaseName, dbParams.databaseVersion, dbParams.databaseDescription, dbParams.initialSize);
        if(this.db === null) {
            this.db = openDatabase(dbParams.databaseName, dbParams.databaseVersion, dbParams.databaseDescription, 5 * 1024 * 1024);
        }

        return this.db;
    },
    
    setFeaturesCount: function(type) {
        var layerType = Giscuit.Util.getStorageObject('gtLayerType');
        
        this.getDbMetadata(function (metadata) {
            $('.gtOfflineMenuName').each(Giscuit.Util.bind(function(index, value) {
                var layerId = $(value).jqmData('layer-id');
                
                if(typeof(type) != 'undefined' && layerType['layer_' + layerId] != type) {
                    return;
                }
                
                var countEl = $($(value).find('.ui-li-count')[0]);
                
                if(typeof(metadata[layerId]) == 'undefined') {
                    countEl.css('display', 'none');
                } else {
                    countEl.html(metadata[layerId].count);
                    countEl.css('display', 'block');
                }
            }, this));
        });
    },
    
    setEditErrors: function() {
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        
        var html = '';
        for (var layerId in editErrors) {
            var layer = gt.control.layers.getLayerRecordById(layerId);
            html += '<li data-role="list-divider">' + layer.name + '</li>';
            for (var feature in editErrors[layerId]) {
                for (var field in editErrors[layerId][feature]) {
                    var errors = '';
                    for (var error in editErrors[layerId][feature][field]) {
                        var errorData = editErrors[layerId][feature][field][error];
                        
                        if(errorData instanceof Array && errorData.length == 2) {
                            html += '<li data-type="' + errorData[1] + '" data-layer="'+ layerId +'" data-feature="'+ feature +'">' + 
                                '<a>' + errorData[0] + '</a>' +
                                '</li>';
                        } else {
                            html += '<li data-type="attribute" data-layer="'+ layerId +'" data-feature="'+ feature +'">' + 
                                '<a>' + field + ' - ' +  errorData + '</a>' +
                                '</li>';
                        }
                        
                    }
                }
            }
        }

        $('#gtOfflineEditErrors').html(html).listview('refresh');
                
        $('#gtOfflineEditErrors li').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, this, function(evt) {
            var type = $(this).jqmData('type');
            var layerSettings = gt.control.layers.getLayerRecordById($(this).jqmData('layer'));
            
            //Set editable layer
            for (var i = 0; i < gt.control.layers.records.length; i++) {
                gt.control.layers.records[i].edit = false;
            }
            layerSettings.edit = true;
            
            //Activate editgeom control
            gt.control.editgeom.deactivate();
            gt.control.editgeom.activate();
            
            //Get current layer and feature
            var layer = gt.layer[layerSettings['class']];
            var feature = layer.getFeatureByFid($(this).jqmData('feature'));
            
            if(type == 'attribute') {
                if(feature) {
                    gt.control.editgeom.controls.editattrControl.select(feature);
                }
            } else if (type == 'geometry') {
                if(feature) {
                    gt.control.editgeom.controls.editControl.selectControl.select(feature);
                    $.mobile.changePage('#gtMap');
                }
            } else {
                var featureId = $(this).jqmData('feature');
                var layerId = $(this).jqmData('layer');
                
                if(typeof(evt.data.overwriteIds[layerId]) == 'undefined') {
                    evt.data.overwriteIds[layerId] = [];
                }
                if(typeof(evt.data.skipIds[layerId]) == 'undefined') {
                    evt.data.skipIds[layerId] = [];
                }
                
                $('#gtOfflineConflictOverwriteButton').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, evt.data, function(evt) {
                    evt.data.overwriteIds[layerId].push(featureId);
                    evt.data.skipIds[layerId] = jQuery.grep(evt.data.skipIds[layerId], function(value) {
                        return value != featureId;
                    });
                    $.mobile.changePage('#gtOffline');
                });
                
                $('#gtOfflineConflictSkipButton').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, evt.data, function(evt) {
                    evt.data.skipIds[layerId].push(featureId);
                    evt.data.overwriteIds[layerId] = jQuery.grep(evt.data.overwriteIds[layerId], function(value) {
                        return value != featureId;
                    });
                    $.mobile.changePage('#gtOffline');
                });
                
                $('#gtOfflineConflictCancelButton').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, evt.data, function(evt) {
                    evt.data.overwriteIds[layerId] = jQuery.grep(evt.data.overwriteIds[layerId], function(value) {
                        return value != featureId;
                    });
                    evt.data.skipIds[layerId] = jQuery.grep(evt.data.skipIds[layerId], function(value) {
                        return value != featureId;
                    });
                    $.mobile.changePage('#gtOffline');
                });
                
                $('#gtOfflineConflictShowButton').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, evt.data, function(evt) {
                    if(feature) {
                        gt.control.editgeom.controls.editControl.selectControl.select(feature);
                        $.mobile.changePage('#gtMap');
                    }
                });
                
                $('#gtMetadata').css('min-height', '100%');
                $.mobile.changePage('#gtOfflineConflict', {'role': 'dialog', 'transition': 'none'});
            }
            
        });
    },
    
    
    getTileURLs: function (bounds, zoom) {
        var result = [];

        function getTileOrigin() {
            var extent = gt.map.getMaxExtent();
            var edges = ({
                "tl": ["left", "top"],
                "tr": ["right", "top"],
                "bl": ["left", "bottom"],
                "br": ["right", "bottom"]
            })["bl"];
            var origin = new OpenLayers.LonLat(extent[edges[0]], extent[edges[1]]);
            
            return origin;
        }
        
        function calculateGridLayout (bounds, origin, resolution) {
            //FIXME: buffer removed
            var buffer = 0;
            
            var tilelon = resolution * gt.map.tileSize.w;
            var tilelat = resolution * gt.map.tileSize.h;
            
            var offsetlon = bounds.left - origin.lon;
            var tilecol = Math.floor(offsetlon/tilelon) - buffer;
            var tilecolremain = offsetlon/tilelon - tilecol;
            var tileoffsetx = -tilecolremain * gt.map.tileSize.w;
            var tileoffsetlon = origin.lon + tilecol * tilelon;
            
            var offsetlat = bounds.top - (origin.lat + tilelat);  
            var tilerow = Math.ceil(offsetlat/tilelat) + buffer;
            var tilerowremain = tilerow - offsetlat/tilelat;
            var tileoffsety = -tilerowremain * gt.map.tileSize.h;
            var tileoffsetlat = origin.lat + tilerow * tilelat;
            
            return { 
              tilelon: tilelon, tilelat: tilelat,
              tileoffsetlon: tileoffsetlon, tileoffsetlat: tileoffsetlat,
              tileoffsetx: tileoffsetx, tileoffsety: tileoffsety
            };
        }
        
        function getURL (bounds) {
            var tileOrigin = getTileOrigin();
            
            var res = gt.map.resolutions[zoom];
            var x = Math.round((bounds.left - tileOrigin.lon) / (res * gt.map.tileSize.w));
            var y = Math.round((bounds.bottom - tileOrigin.lat) / (res * gt.map.tileSize.h));
                
            return zoom + "/" + x + "/" + y + '.jpeg'; 
        }
        
        function getTiles (bounds) {
            var origin = getTileOrigin();
            var resolution = gt.map.resolutions[zoom];
            
            var tileLayout = calculateGridLayout(bounds, origin, resolution);

            var tileoffsetx = Math.round(tileLayout.tileoffsetx);
            var tileoffsety = Math.round(tileLayout.tileoffsety);
            
            var tileoffsetlon = tileLayout.tileoffsetlon;
            var tileoffsetlat = tileLayout.tileoffsetlat;

            var tilelon = tileLayout.tilelon;
            var tilelat = tileLayout.tilelat;

            this.origin = new OpenLayers.Pixel(tileoffsetx, tileoffsety);

            var startX = tileoffsetx;
            var startLon = tileoffsetlon;
            
            do {
                tileoffsetlon = startLon;
                tileoffsetx = startX;

                do {
                    var tileBounds = new OpenLayers.Bounds(tileoffsetlon, tileoffsetlat, tileoffsetlon + tilelon, tileoffsetlat + tilelat);
                    result.push(getURL(tileBounds));

                    tileoffsetlon += tilelon;
                    tileoffsetx += gt.map.tileSize.w;
                //FIXME: buffer removed
                } while ((tileoffsetlon <= bounds.right));

                tileoffsetlat -= tilelat;
                tileoffsety += gt.map.tileSize.h;
            //FIXME: buffer removed
            } while ((tileoffsetlat >= bounds.bottom - tilelat));

        }

        getTiles(bounds, zoom);

        return result;
    },
    
    downloadTiles: function(layer, tmpname) {
        var urls = [];
        var self = this;
        
        //
        for(var a in this.tiles[layer.id]) {
            var len = 5;
            var deleteZoomLevel = false;
            var deleteLayer = true;
            
            //
            if(this.tiles[layer.id][a].length < len) {
                len = this.tiles[layer.id][a].length;
                var deleteZoomLevel = true;
            }
            
            //
            for(var b=0; b < len; b++) {
                urls.push(this.tiles[layer.id][a][b]);
            }
            this.tiles[layer.id][a].splice(0, len);

            //
            if(deleteZoomLevel) {
                delete this.tiles[layer.id][a];
            }
            
            //
            for(var c in this.tiles[layer.id]) {
                deleteLayer = false;
            }
            if(deleteLayer) {
                delete this.tiles[layer.id];
            }
            
            break;
        }
                
        if(Giscuit.Util.isEmpty(this.tiles[layer.id]) && !deleteZoomLevel) {
            this.downloadTilesComplete(tmpname);
        } else {
            var url =  gt.config.httpMobilePath + 
                '/offline/downloadtiles/tmpname/' + tmpname;
            
            $.ajax({
                'timeout': 30000 ,
                'type': 'POST',
                'url': url,
                'data': 'tiles=' + urls.join(','),
                'success': this.downloadTilesSuccess ,
                'error': this.downloadTilesSuccess ,
                'dataType': 'json' ,
                'context': {
                    'self': this,
                    'layer': layer,
                    'tmpname': tmpname
                }
            });
        }
    },
    
    downloadTilesSuccess: function(data) {
        var layer = this.layer;
        var tmpname = this.tmpname;
        var self = this.self;
        var db = self.getDbConnection();
        
        if(typeof(data.records) == 'undefined') {
            self.downloadTiles(layer, tmpname);
            return;
        }
        
        db.transaction(function (tx) {
            for(var url in data.records) {
                var sql = 'INSERT INTO layer_' + layer.id + ' (url, image) VALUES (?, ?)';
                tx.executeSql(sql, [url, data.records[url]], function (tx, results) {
                    
                }, Giscuit.Util.websqlFailure);
            }
        },
        Giscuit.Util.websqlFailure,
        function() {
            //
            var tilesCount = self.tilesCount[layer.id];
            var tilesCurr = 0;
            for (var obj in self.tiles[layer.id]) {
                tilesCurr += self.tiles[layer.id][obj].length;
            }
            tilesCurr = tilesCount - tilesCurr + 1;
            
            //
            $('.gtOfflineMenuName').each(function(index, value) {
                var layerId = $(value).jqmData('layer-id');
                
                if(layerId != layer.id) {
                    return;
                }
                
                var countEl = $($(value).find('.ui-li-count')[0]);
                countEl.html(tilesCurr + '/' + tilesCount);
                countEl.css('display', 'block');
            });
            
            self.downloadTiles(layer, tmpname);        
        });
    },
    
    downloadTilesComplete: function(tmpname) {
        var url =  gt.config.httpMobilePath + 
            '/offline/downloadtilessuccess/tmpname/' + tmpname;
    
        $.ajax({
            'type': 'GET',
            'url': url,
            'success': function(data) {
                
            } ,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this
        });
        
        this.setFeaturesCount('raster');
        
        $('#gtOfflineDownloadButton').button('enable');
        $('.gtOfflineMenuDelete').each(function(index, value) {
            $(this).removeClass('ui-disabled');
        });
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Offline"
});

gt.control.offline = new Giscuit.Mobile.Control.Offline();

//
OpenLayers.Protocol.SQL.WebSQL=OpenLayers.Class(OpenLayers.Protocol.SQL,{maxId:0,databaseVersion:"",databaseName:"ol",databaseDescription:"OpenLayers.Protocol.WebSQL",tableName:null,db:null,initialSize:5*1024*1024,supported:typeof window.openDatabase==="function",initialize:function(a){if(this.supported){OpenLayers.Protocol.prototype.initialize.apply(this,[a]);if(!this.format){this.format=new OpenLayers.Format.GeoJSON}if(!this.tableName){this.tableName=OpenLayers.Util.createUniqueID("websql_")}this.initializeDatabase()}},destroy:function(){if(!this.options||!this.options.format){this.format.destroy()}this.format=null;this.db=null;OpenLayers.Protocol.SQL.prototype.destroy.apply(this,arguments)},initializeDatabase:function(){this.db=openDatabase(this.databaseName,this.databaseVersion,this.databaseDescription,this.initialSize);this.transaction(function(a){this.executeSql(a,"CREATE TABLE IF NOT EXISTS "+this.tableName+" (id INTEGER PRIMARY KEY, feature TEXT)",[],this.executeSql(a,"SELECT MAX(id) as count FROM "+this.tableName,[],function(a,b){this.maxId=b.rows.item(0).count||0}))})},transaction:function(a,b,c){if(this.supported){this.db.transaction(OpenLayers.Function.bind(a,this),OpenLayers.Function.bind(b||OpenLayers.Function.Void,this),OpenLayers.Function.bind(c||OpenLayers.Function.Void,this))}else{if(b){b.apply(this,["Protocol not supported"])}}},executeSql:function(a,b,c,d,e){a.executeSql(b,c,OpenLayers.Function.bind(d||OpenLayers.Function.Void,this),OpenLayers.Function.bind(e||OpenLayers.Function.Void,this))},read:function(a){a=OpenLayers.Util.applyDefaults(a,this.options||{});var b=new OpenLayers.Protocol.Response({requestType:"read"});this.transaction(function(c){this.executeSql(c,"SELECT * from "+this.tableName,[],function(c,d){b.priv={transaction:c,result:d};this.handleResponse(b,a)},function(c,d){b.priv={transaction:c,error:d};this.handleResponse(b,a)})});return b},create:function(a,b){b=OpenLayers.Util.applyDefaults(b,this.options||{});var c=new OpenLayers.Protocol.Response({reqFeatures:[a],requestType:"create"});this.transaction(function(d){this.executeSql(d,"INSERT INTO "+this.tableName+" (id, feature) VALUES (?, ?)",[this.maxId+1,this.format.write(a)],function(a,d){c.priv={transaction:a,result:d};this.handleResponse(c,b)},function(a,d){c.priv={transaction:a,error:d};this.handleResponse(c,b)})});return c},update:function(a,b){b=OpenLayers.Util.applyDefaults(b,this.options||{});var c=new OpenLayers.Protocol.Response({reqFeatures:[a],requestType:"update"});this.transaction(function(d){this.executeSql(d,"REPLACE INTO "+this.tableName+" (id, feature) VALUES (?, ?)",[a.fid,this.format.write(a)],function(a,d){c.priv={transaction:a,result:d};this.handleResponse(c,b)},function(a,d){c.priv={transaction:a,error:d};this.handleResponse(c,b)})});return c},"delete":function(a,b){b=OpenLayers.Util.applyDefaults(b,this.options||{});var c=new OpenLayers.Protocol.Response({reqFeatures:[a],requestType:"delete"});this.transaction(function(d){this.executeSql(d,"DELETE FROM "+this.tableName+" where id=?",[a.fid],function(a,d){c.priv={transaction:a,result:d};this.handleResponse(c,b)},function(a,d){c.priv={transaction:a,error:d};this.handleResponse(c,b)})});return c},clear:function(a){this.transaction(function(a){this.executeSql(a,"DELETE FROM "+this.tableName)})},handleResponse:function(a,b){var c=a.priv.result;if(c){switch(a.requestType){case"create":this.maxId=c.insertId;break;case"read":var d,e;var f=c.rows.length;a.features=new Array(f);for(var g=0;g<f;++g){d=c.rows.item(g);e=this.format.read(d["feature"],"Feature");e.fid=d["id"];a.features[g]=e}break}a.code=OpenLayers.Protocol.Response.SUCCESS}else{a.code=OpenLayers.Protocol.Response.FAILURE}if(b.callback){b.callback.call(b.scope,a)}},commit:function(a,b){function n(a){this.callUserCallback(a,b);k=k&&a.success();d++;if(d>=j){if(b.callback){l.code=k?OpenLayers.Protocol.Response.SUCCESS:OpenLayers.Protocol.Response.FAILURE;b.callback.apply(b.scope,[l])}}}function m(a){return function(b){l.insertIds[a]=b.priv.result&&b.priv.result.insertId;n.apply(this,[b])}}b=OpenLayers.Util.applyDefaults(b,this.options||{});var c=[],d=0;var e={};e[OpenLayers.State.INSERT]=[];e[OpenLayers.State.UPDATE]=[];e[OpenLayers.State.DELETE]=[];var f,g;for(var h=0,i=a.length;h<i;++h){f=a[h];g=e[f.state];if(g){g.push(f)}}var j=(e[OpenLayers.State.INSERT].length>0?1:0)+e[OpenLayers.State.UPDATE].length+e[OpenLayers.State.DELETE].length;var k=true;var l=new OpenLayers.Protocol.Response({reqFeatures:a});l.insertIds=new Array(e[OpenLayers.State.INSERT].length);var o=e[OpenLayers.State.INSERT];for(var h=o.length-1;h>=0;--h){c.push(this.create(o[h],OpenLayers.Util.applyDefaults({callback:m(h),scope:this},b.create)))}o=e[OpenLayers.State.UPDATE];for(var h=o.length-1;h>=0;--h){c.push(this.update(o[h],OpenLayers.Util.applyDefaults({callback:n,scope:this},b.update)))}o=e[OpenLayers.State.DELETE];for(var h=o.length-1;h>=0;--h){c.push(this["delete"](o[h],OpenLayers.Util.applyDefaults({callback:n,scope:this},b["delete"])))}return c},callUserCallback:function(a,b){var c=b[a.requestType];if(c&&c.callback){c.callback.call(c.scope,a)}},CLASS_NAME:"OpenLayers.Protocol.SQL.WebSQL"})

Giscuit.Util.OfflineEditing = OpenLayers.Class(OpenLayers.Protocol.SQL.WebSQL, {
    databaseName: 'gt',
    
    databaseDescription: 'Giscuit',
    
    initialSize: 50 * 1024 * 1024,
    
    databaseVersion: '1.0',
    
    /**
     * Method: initialCreate
     * Construct a request for writing newly created features.
     */
    initialCreate: function(data, options) {
        options = OpenLayers.Util.applyDefaults(options, this.options || {});
        
        var jsonFormat = new OpenLayers.Format.JSON();
        
        var response = new OpenLayers.Protocol.Response({
            reqFeatures: [data], requestType: "create"
        });
        
        this.transaction(function(tx) {
            for(var i=0; i < data.length; i++) {
                var id = data[i].properties.gid;
                data[i].properties.__hdate = options.timestamp;
                
                var feature = jsonFormat.write(data[i]);
                
                if(id > this.maxId) {
                    this.maxId = id;
                }
                
                this.executeSql(
                    tx,
                    "INSERT INTO " + this.tableName + " (id, feature) VALUES (?, ?)",
                    [id, feature],
                    function(tx, result) { 
                        response.priv = {transaction: tx, result: result};
                        this.handleResponse(response, options);
                    },
                    function(tx, error) {
                        response.priv = {transaction: tx, error: error};
                        this.handleResponse(response, options);
                    }
                );
            
            }
        }, 
        null,
        function () {
            gt.control.offline.setFeaturesCount('vector');
        });
        
        return response;
    },
    
    /**
     * Method: initializeDatabase
     */
    initializeDatabase: function() {
        this.db = openDatabase(this.databaseName, this.databaseVersion, this.databaseDescription, this.initialSize);
        this.transaction(function(tx) {
            this.executeSql(
                tx,
                "CREATE TABLE IF NOT EXISTS " + this.tableName +
                    " (id INTEGER PRIMARY KEY, feature TEXT)",
                [],
                function (tx, results) {
                },
                Giscuit.Util.websqlFailure
            );
        }, null, this.transaction(function(tx) {
            this.executeSql(
                tx,
                "SELECT MAX(id) as count FROM " + this.tableName,
                [],
                function(tx, result) {
                    this.maxId = result.rows.item(0).count || 0;
                },
                function (tx, evt) {
                    debug.warn(evt.message);
                }
            );
        }));
    },
    
    /**
     * Method: handleResponse
     * Deals with responses and calls any specified callback.
     *
     * Parameters:
     * response - {OpenLayers.Protocol.Response} The response.
     * options - {Object}
     */
    handleResponse: function(response, options) {
        var result = response.priv.result;
        if (result) {
            switch (response.requestType) {
                case "create":
                    this.maxId = result.insertId;
                    break;
                case "read":
                    var row, feature;
                    var len = result.rows.length;
                    response.features = [];
                    for(var i=0; i < len; ++i) {
                        var add = true;
                        
                        row = result.rows.item(i);
                        feature = this.format.read(row["feature"], "Feature");
                        feature.fid = row["id"];
                        
                        //withHStatus
                        if(typeof(options.withHStatus) != 'undefined' && options.withHStatus === true) {
                            if(! feature.attributes.hstatus) {
                                add = false;
                            }
                        } else {
                            if(feature.attributes.hstatus == 'D') {
                                add = false;
                            }
                        }
                        
                        if(add) {
                            response.features.push(feature);
                        }
                    }
                    break;
            }
            response.code = OpenLayers.Protocol.Response.SUCCESS;
        } else {
            response.code = OpenLayers.Protocol.Response.FAILURE;
        }
        if (options.callback) {
            options.callback.call(options.scope, response);
        }
    },
    
    CLASS_NAME: "Giscuit.Util.OfflineEditing"
});