/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Privatelayers.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Privatelayers
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Privatelayers = OpenLayers.Class(Giscuit.Control, {
    
    records: [],
    
    currSettingsLayerId: null,

    initialize: function (records, options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        debug.info('Initializing layers');
        
        //Deprecated in 2.12
        //this.EVENT_TYPES.push('changelayer');
        
        //Add layers
        if(gt.control.offline.isOnline) {
            this.records = records;
               Giscuit.Util.setStorageObject('gtLayers', this.records);
        } else {
            this.records = Giscuit.Util.getStorageObject('gtLayers');
        }
        
        this.add();
        
        $(document).ready(Giscuit.Util.bind(function() {
            $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtLayersButton">'
                + '<div class="gtMapControlsElImage" style="background-position: 0 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('layers') + '</div>'
                + '</div></div>');
            
            $("#gtLayersButton").bind(gt.config.clickEvt, function(evt) {
                //Hide map controls
                $("#gtMapControls").css('display', 'none');
                $("#gtToggleControlsButton").css('bottom', '5px').removeClass('ui-btn-active');
                
                $.mobile.changePage($('#gtLayers'));
                
                return false;
            });
            
            var html =
            '<div data-role="page" id="gtLayersSettings" data-dom-cache="true" data-url="gtLayersSettings">' +
                '<div data-role="header">' +
                    '<h1>' + gt.translate._('layers') + '</h1>' +
                    '<a href="#gtLayers" data-icon="arrow-l" data-iconpos="notext" data-direction="reverse" class="ui-btn-right jqm-home">' +
                        gt.translate._('back') +
                    '</a>' + 
                '</div>' +
                '<div data-role="content"></div>' +        
            '</div>';
            
            $.mobile.pageContainer.append(html);
            $('#gtLayersSettings').page();

            $('#gtLayers').live('pagehide', function(evt, ui){
                gt.map.refresh();
                gt.map.fixGoogleLayers();
                
                if(gt.control.editgeom && gt.control.editgeom.active) {
                    gt.control.editgeom.controlPanel.redraw();
                }
            });
            
            //Enable or disable layer on click
            var menuNameTaphold = false;
            $('.gtLayersMenuName').bind(gt.config.clickEvt, this, function(evt) {
                if(menuNameTaphold == false) {
                    evt.data.nameClick($(this).data('layer-id'), this.parentNode.parentNode.parentNode);
                }
                
                //
                menuNameTaphold = false;
                
                return false;
            });
            
            //Show layer metadata on hold
            $('.gtLayersMenuName').bind('taphold', this, function(evt) {
                var layer = evt.data.getLayerRecordById($(this).data('layer-id'));

                gt.map.showMetadata(layer);
                
                $('#gtMetadata').bind('pagehide', function() {
                    menuNameTaphold = false;
                });
                //
                menuNameTaphold = true;
                
                return false;
            });
            
            //Access layer settings on icon click
            $('.gtLayersMenuSettings').bind(gt.config.clickEvt, this, function(evt) {
                if (! $(this).hasClass('ui-disabled')) {
                    var layerId = $($(this.parentNode).find('*[data-layer-id]')[0]).jqmData('layer-id');
                    evt.data.settingsClick(layerId, this);
                }
                
                return false;
            });
            
            this.initializeLayersOrderPage();
            
        }, this));
        
    } ,
    
    initializeLayersOrderPage: function () {
        //Change layer order on click
        $('.gtLayersOrderUpButton').bind(gt.config.clickEvt, this, function(evt) {
            if (! $(this).hasClass('ui-disabled')) {
                evt.data.changePosition($(this).closest('div').closest('li'), 'up');
            }
            
            return false;
        });
        
        $('.gtLayersOrderDownButton').bind(gt.config.clickEvt, this, function(evt) {
            if (! $(this).hasClass('ui-disabled')) {
                evt.data.changePosition($(this).closest('div').closest('li'), 'down');
            }
            
            return false;
        });
        
        //Save the current layer order
        $('#gtLayersOrderSave').bind(gt.config.safeClickEvt, this, function(evt) {
            $.mobile.showPageLoadingMsg();
            
            var layerIds = [];
            $('.gtLayersOrderControlGroup').each(function(index, value) {
                layerIds.push($(this).jqmData('layer-id'));
            });

            if(gt.control.offline.isOnline) {
                var url = gt.config.httpMobilePath + '/privatelayers/order/ids/' + layerIds.join();
                
                $.getJSON(url, function(data) {
                        $.mobile.changePage($('#gtLayers'));
                        $.mobile.hidePageLoadingMsg();
                        
                        Giscuit.Util.updateApplicationCache();
                    })
                       .error(Giscuit.Util.jqmAjaxError);
            } else {
                var offlineLayers = [];
                for (var i = 0; i < layerIds.length; i++) {
                    var layerInfo = gt.control.layers.getLayerRecordById(layerIds[i]);
                    layerInfo.changed = true;
                    layerInfo.order = i + 1;
                    if (layerInfo.offline) {
                        offlineLayers.push(layerIds[i]);
                    }
                }
                offlineLayers.reverse();
                for (var i = 0; i < offlineLayers.length; i++) {
                    var layer = gt.layer[gt.control.layers.getLayerRecordById(offlineLayers[i])['class']];
                    gt.map.setLayerIndex(layer, i + 2);
                }
                
                $.mobile.changePage($('#gtLayers'));
                $.mobile.hidePageLoadingMsg();
            }
            
            return false;
        });
        
        //Calculate width for middle layer order button
        var fixedDivWidth = $(window).width() - 86;
        $("<style type='text/css' id='gtLayersOrderStyle'>" + 
            ".gtLayersOrderControlGroup div {opacity: 1 !important; width:" + fixedDivWidth + "px;}" +
            "#gtLayersOrderList li:not(:last-child) {margin-bottom: -1px}" +
            "</style>").appendTo("head");
        
        //FIXME: #96, In mobile version reorder section there is spacing between lines in IE9
        if($.browser.msie || $.browser.mozilla) {
            $('#gtLayersOrderList li:not(:last-child)').each(function(index, value) {
                $(this).css('margin-bottom', '-5px');
            });
        }

        $(window).bind("orientationchange throttledresize", this, function(evt) {
            var isPageVisible = $("#gtLayersOrder > div[data-role='content']").is(':visible');
            
            if (isPageVisible) {
                $('#gtLayersOrderStyle').remove();
                
                var fixedDivWidth = $(window).width() - 86;
                $("<style type='text/css' id='gtLayersOrderStyle'> .gtLayersOrderControlGroup div {" + 
                    "opacity: 1 !important; width:" + fixedDivWidth + "px;} </style>").appendTo("head");
            } else {
                return false;
            }
        });
        
        //Disable upper and lowest button
        $('.gtLayersOrderUpButton').first().addClass('ui-disabled');
        $('.gtLayersOrderDownButton').last().addClass('ui-disabled');
    } ,
    
    add: function () {
        var snapLayers = [];
        
        //Order the array by order column
        function byOrder(a, b) {
            var x = a.order;
            var y = b.order;
            
            return ((x < y) ? -1 : ((x > y) ? 1 : 0));
        }
        this.records = this.records.sort(byOrder).reverse();
        
        if(gt.control.offline.isOnline) {
            //Javascript
            var overlays = [];
            for(var i=0; i < this.records.length; i++) {
                this.records[i].offline = false;
                
                if(this.records[i]['js_layer']) {
                    var layerName = this.records[i]['class'];
                    gt.layer[layerName] = new Giscuit.Mobile.Layer[layerName]();
                    gt.layer[layerName].order = this.records[i]['order'];
                    
                    if(this.records[i]['js_isbase'] == 't') {
                        gt.map.addLayer(gt.layer[layerName]);
                        
                    } else {
                        overlays.push(gt.layer[layerName]);
                    }
                    
                    gt.layer[layerName].setVisibility(this.records[i]['view']);
                    
                    //
                    if(this.records[i]['select'] === true && 
                        typeof(gt.layer[layerName].features) != 'undefined') {
                        
                        snapLayers.push(gt.layer[layerName]);
                    }
                }
            }
            
            //Main
            gt.layer.main = new OpenLayers.Layer.MapServer("main", gt.config.httpPath + "/draw", {
                layers: "basic"
            }, {
                transitionEffect: "resize",
                ratio: 1,
                singleTile: true
            });
            gt.map.addLayer(gt.layer.main);
    
            //Javascript
            gt.map.addLayers(overlays);

            //Edit
            gt.map.addEditLayer();
            
            //Vector
            gt.map.addVectorLayer();
            
            //Marker
            gt.map.addMarkerLayer();
            
            //Snap
            if (gt.config.snap == 't') {
                snapLayers.push(gt.layer.vector);
                snapLayers.push(gt.layer.edit);
                gt.map.addSnapLayer(snapLayers, true);
            }
        } else {
            gt.control.offline.getDbMetadata(Giscuit.Util.bind(this.addOffline, this));
        }
    },
    
    addOffline: function(metadata) {
        var layers = [];
        var snapLayers = [];

        //SLD
        var layerType = Giscuit.Util.getStorageObject('gtLayerType');
        var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
                
        //Default styles
        var defaultStyle = OpenLayers.Util.applyDefaults({
            pointRadius: gt.map.pointRadius ,
            strokeWidth: 3
        }, OpenLayers.Feature.Vector.style["default"]);
        var selectStyle = OpenLayers.Util.applyDefaults({
            pointRadius: gt.map.pointRadius ,
            strokeWidth: 3
        }, OpenLayers.Feature.Vector.style["select"]);
        
        for(var i=0; i < this.records.length; i++) {
            var record = this.records[i];
            
            if(! metadata[record.id]) {
                this.records[i].offline = false;
                continue;
            }
            this.records[i].offline = true;
            
            if(layerType['layer_' + record.id] == 'vector') {
                //
                var styleMap = new OpenLayers.StyleMap( {
                    'default': defaultStyle,
                    'select': selectStyle
                });
                
                styleMap.styles['default'].context = {
                    '__getLabel': Giscuit.Util.bind(function (feature) {
                        //
                        if(! this.label) {
                            return '';
                        };
                                                                                                
                        var labelDefinition = '';
                        for(var j=0; j < this.label_fields.length; j++) {
                            var labelField = feature.attributes[this.label_fields[j]];
                            if(labelField !== null && typeof(labelField) != 'undefined') {
                                labelDefinition += labelField + "\n";
                            }
                        }
                        
                        return labelDefinition;
                    }, record)
                };

                var layer = new OpenLayers.Layer.Vector(record['class'], {
                    strategies: [new OpenLayers.Strategy.Fixed()],
                    protocol: new Giscuit.Util.OfflineEditing({tableName: 'layer_' + record.id}),
                    'visibility': record.view ,
                    'order': record.order,
                    'layername': record['class'],
                    'maxscaledenom': parseFloat(record.maxscaledenom),
                    'maxScale': parseFloat(record.minscaledenom),
                    'minscaledenom': parseFloat(record.minscaledenom),
                    'minScale': parseFloat(record.maxscaledenom),
                    'styleMap': styleMap
                });

                //Style
                if(typeof(mapSLD['layer_' + record.id]) != undefined) {
                    var styleRules = gt.map.getStyleRules(mapSLD['layer_' + record.id]);
                    
                    //Add labels
                    this.setLabel(styleRules, record);
                    
                    layer.styleMap.styles['default'].rules = [];
                    layer.styleMap.styles['default'].addRules(styleRules);
                } else {
                    debug.warn('SLD for layer "' + className + '" was not found');
                }
                                
                snapLayers.push(layer);
                
            } else if (layerType['layer_' + record.id] == 'raster') {
                var db = gt.control.offline.getDbConnection();
                
                var layer = new OpenLayers.Layer.TMS(record['class'], gt.config.staticserverUrls, {
                    'serviceVersion': '' ,
                    'type': 'jpeg' ,
                    'visibility': record.view ,
                    'order': record.order,
                    'layername': record.id ,
                    'maxscaledenom': parseFloat(record.maxscaledenom),
                    'maxScale': parseFloat(record.minscaledenom),
                    'minscaledenom': parseFloat(record.minscaledenom),
                    'minScale': parseFloat(record.maxscaledenom),
                    'buffer': 1,
                    'isBaseLayer': false,
                    'async': true
                });
                
                layer.getURLasync = function(bounds, callback, scope) {
                    var sortURL = scope.layer.getURL(bounds).split('/');
                    sortURL.splice(0, 3);
                    sortURL = sortURL.join('/');
                    
                    var sql = "SELECT image FROM layer_" + scope.layer.layername + 
                        " WHERE url = ?";
                    
                    db.readTransaction(function (tx) {
                        tx.executeSql(sql, [sortURL], function (tx, results) {
                            if(results.rows.length) {
                                var url = results.rows.item(0).image;
                            } else {
                                var url = gt.config.httpImagePath + '/blank.gif';
                            }
                            callback.call(scope, url);
                            
                        }, Giscuit.Util.websqlFailure);
                    }, Giscuit.Util.websqlFailure);
                };
            }
            
            layers.push(layer);
            gt.layer[record['class']] = layer;
            
        }
        
        gt.map.addLayers(layers);
        
        //Vector
        gt.map.addVectorLayer();
        
        //Marker
        gt.map.addMarkerLayer();
        
        //Snap
        snapLayers.push(gt.layer.vector);
        gt.map.addSnapLayer(snapLayers, false);
        
        //Show only layers available for offline 
        $('.gtLayersMenuName').each(Giscuit.Util.bind(function(index, value) {
            //
            if($(value).parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent());
            } else if($(value).parent().parent().parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent().parent().parent());
            }

            var layer = this.getLayerRecordById($(value).data('layer-id'));
            
            if(layer.offline) {
                if(parent.hasClass('ui-btn-active') != layer.view) {
                    if(layer.view) {
                        parent.addClass('ui-btn-active');
                        parent.find('.gtLayersMenuSettings').addClass('ui-btn-active');
                    } else {
                        parent.removeClass('ui-btn-active');
                        parent.find('.gtLayersMenuSettings').removeClass('ui-btn-active');
                    }
                }
            } else {
                parent.css('display', 'none');
            }
            
        }, this));
        
        //Hide empty containers
        var maxIterations = 0;
        var allContainersHidden = false;
        while(! allContainersHidden) {
            //
            if(maxIterations >= 10) {
                debug.warn('Max iterations reached (10) while trying to hile all empty containers');
                break;
            }

            //
            allContainersHidden = true;
            
            $('.gtLayersMenuContainer').each(function(index, value) {
                //Unenhanced
                if($(value).parent().get(0).tagName.toLowerCase() == 'li') {
                    var parent = $($(value).parent());
                    var lis = parent.children()[1].children;
                
                //Enhanced
                } else if($(value).parent().parent().parent().get(0).tagName.toLowerCase() == 'li') {
                    var parent = $($(value).parent().parent().parent());
                    
                    var href = $(value).attr('href');
                    href = href.substr(1);
                    
                    var lis = $("div:jqmData(url='" + href + "')").find('li');
                }
                
                //
                if(parent.css('display') == 'none') {
                    return;
                }
                
                //
                var hiddenAmount = 0;
                for(var i=0; i < lis.length; i++) {
                    var li = $(lis[i]);
                    
                    if(li.css('display') == 'none') {
                        hiddenAmount++;
                    }
                }
                
                //
                if(hiddenAmount == lis.length) {
                    parent.css('display', 'none');
                    allContainersHidden = false;
                }
            });
            
            maxIterations++;
        }
                
        //Reorder only layers available for offline
        $('.gtLayersOrderControlGroup').each(function(index, value) {
            var layerInfo = gt.control.layers.getLayerRecordById($(value).jqmData('layer-id'));
            if (! layerInfo.offline) {
                $(value).parent().css('display', 'none');
            }
        });
        this.changePosition(null, '');
        
        //
        gt.control.offline.events.triggerEvent('offline');
        gt.init.layout.renderMapControls();
    },
    
    setLabel: function(styleRules, record) {
        //Parse rules
        for(var i=0; i < styleRules.length; i++) {
            for (var type in styleRules[i].symbolizer) break;

            if(type == 'Line') {
                continue;
            }
            
            var symbolizer = styleRules[i].symbolizer[type];
            
            symbolizer.label = '${__getLabel}';
            symbolizer.fontFamily = 'sans-serif'
            symbolizer.fontSize = record.label_size + 'pt';
            symbolizer.fontColor = record.label_color;
            
            if(type == 'Point') {
                symbolizer.labelAlign = 'cb';
            } else {
                symbolizer.labelAlign = 'cm';
            }
        }
        
    },
    
    nameClick: function(id, el) {
        //Offline
        if(! gt.control.offline.isOnline) {
            this.toggleVisibility(id, el);
            return;
        }
    
        //Online
        if($('.ui-loading').length > 0) {
            return;
        }
        $.mobile.showPageLoadingMsg();
        
        var record = this.getLayerRecordById(id);
        var url = gt.config.httpMobilePath + '/privatelayers/edit/id/' + id + '/column/view/value/' + (! record.view);
        
        var self = this;
        $.getJSON(url, function(data) {
            self.toggleVisibility(id, el);
            $.mobile.hidePageLoadingMsg();
            
            self.events.triggerEvent('changelayer');
            
            Giscuit.Util.updateApplicationCache();
        })
            .error(Giscuit.Util.jqmAjaxError);
        
    } ,
    
    toggleVisibility: function(id, el) {
        var record = this.getLayerRecordById(id);
        
        optionsEl = el.children[1];
    
        $(el).removeClass('ui-btn-active');
        $(optionsEl).removeClass('ui-btn-active');
    
        //Save
        record.view = ! record.view;
        
        if(record.view) {
            $(el).addClass('ui-btn-active');
            $(optionsEl).addClass('ui-btn-active');
        }
        
        //
        if(! gt.control.offline.isOnline) {
            record.changed = true;
        }
        Giscuit.Util.setStorageObject('gtLayers', this.records);
        
        //Javascript layer
        if(record.js_layer || ! gt.control.offline.isOnline) {
            var layer = gt.layer[record['class']];
            if(typeof(layer) != "undefined") {
                layer.setVisibility(record.view);
            }
        }
    } ,
    
    settingsClick: function(id, el) {
    	var maxFields = gt.config.maxFieldsAmount;
    	
        this.currSettingsLayerId = id;
        
        //Get layer
        var layer = this.getLayerRecordById(id);
        var submitDisabled = '';

        var html = '<form id="gtPrivatelayersSettingsForm" enctype="application/x-www-form-urlencoded" method="post"><dl class="zend_form">' +
        '<dt id="select-label"><label for="select" class="required">' + gt.translate._('information') + '</label></dt>' +
        '<dd id="select-element"><input type="checkbox" name="select" id="select" value="t"' + ((layer.select) ? ' checked="checked"' : '');
        
        //Check if info is available
        if (layer.select == 'none') {
            html += ' disabled></dd>';
            submitDisabled = ' disabled';
        } else {
            html += '></dd>';
        }
        
        if ($(el).hasClass('info-only')) {
            html += '<button type="button" name="submit_button" id="gtPrivatelayersLabelSubmit" ' + submitDisabled + '>' + gt.translate._('save') + '</button>' +
            
            '</dl></form>';
        } else {
            html += '<dt id="label-label"><label for="label" class="required">' + gt.translate._('label') + '</label></dt>' +
            '<dd id="label-element"><input type="checkbox" name="label" id="label" value="t"' + ((layer.label) ? ' checked="checked"' : '') + '></dd>';
    
            if(layer.edit != 'none') {
                html += '<dt id="edit-label"><label for="edit" class="required">' + gt.translate._('edit') + '</label></dt>' +
                '<dd id="edit-element"><input type="checkbox" name="edit" id="edit" value="t"' + ((layer.edit) ? ' checked="checked"' : '') + '></dd>';
            }
    
            if(layer.classlabel_active == 'f' || ! gt.control.offline.isOnline) {
            
                html += '<dt id="label_size-label"><label for="label_size" class="required">' + gt.translate._('size') + '</label></dt>' +
                '<dd id="label_size-element"><input type="text" name="label_size" id="label_size" value="' + layer.label_size + '"></dd>' +
        
                '<dt id="label_color-label"><label for="label_color" class="required">' + gt.translate._('color') + '</label></dt>' +
                '<dd id="label_color-element"><input data-role="none" type="text" name="label_color" id="label_color" value="' + layer.label_color + '"></dd>' +
        
                '<dt id="label_fields-label"><label for="label_fields" class="optional">' + gt.translate._('fields') + '</label></dt>' +
                '<fieldsset data-role="controlgroup">';
                        
                for(var i=0; i < layer.fields.length; i++) {
                	if(i >= maxFields) {
                		debug.warn('Maximum number of fields (' + maxFields + ') reached');
                    	var firstNFieldsDisplayed = gt.translate._('firstNFieldsDisplayed').replace("%1$s", gt.config.maxFieldsAmount);
                    	Giscuit.Util.jqmMessage(firstNFieldsDisplayed);
                		break;
                	} 
                	
                    var checked = '';
                    if(Giscuit.Util.inArray(layer.label_fields, layer.fields[i])) {
                        var checked = ' checked="checked"';
                    }
                    
                    html += '<label for="label_fields-' + layer.fields[i] + '">' +
                    '<input type="checkbox" name="label_fields[]" id="label_fields-' + layer.fields[i] + '" value="' + layer.fields[i] + '"' + checked + '>' + gt.translate._(layer.fields[i]) + '</label>';
                }
        
                html += '</fieldsset>';
            
            }
       
            html += '<button type="button" name="submit_button" id="gtPrivatelayersLabelSubmit">' + gt.translate._('save') + '</button>' +
            
            '</dl></form>';
        }
        
        //Render
        var content = $("#gtLayersSettings > div[data-role='content']");
            content.html(html).trigger('create');
            
        $('#label_color').colorPicker();
                    
        //Set header
        var header = $("#gtLayersSettings > div[data-role='header'] > h1");
        header.html(layer.name);
        
        //Submit
        $('#gtPrivatelayersLabelSubmit').unbind(gt.config.clickEvt).bind(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
            
            //Validate size and color
            var form = $('#gtPrivatelayersSettingsForm');
            var size = form.find('input[name="label_size"]').attr('value');
            var color = form.find('input[name="label_color"]').attr('value');
            
            if (!isNaN(parseInt(size)) && isFinite(size)) {
                if (parseInt(size) < 4) {
                    size = 4;
                } else if (parseInt(size) > 32) {
                    size = 32;
                }
            } else {
                size = 10;
            }

            if(! /^#[0-9A-F]{6}$/i.test(color)) {
                color = '#000000';
            }

            form.find('input[name="label_size"]').attr('value', size);
            form.find('input[name="label_color"]').attr('value', color);

            if(gt.control.offline.isOnline) {
                var url =  gt.config.httpMobilePath + 
                    '/privatelayers/settings/id/' + evt.data.currSettingsLayerId;
                                        
                $.ajax({
                    'type': 'POST',
                    'url': url,
                    'data': form.serialize(),
                    'success': evt.data.settingsSubmit ,
                    'error': Giscuit.Util.jqmAjaxError ,
                    'dataType': 'json' ,
                    'context': evt.data
                });
            } else {
                evt.data.settingsSubmit();
            }
            
            return false;
        });
        
        $.mobile.changePage($('#gtLayersSettings'));
    } ,
    
    settingsSubmit: function (data) {

        var layerSettings = this.getLayerRecordById(this.currSettingsLayerId);
        var layer = gt.layer[layerSettings['class']];
        
        var form = $('#gtPrivatelayersSettingsForm');
            
        //Select
        layerSettings.select = form.find('input[name="select"]').is(':checked');
        
        //Activate deactivate javascript layer snap
        if(layerSettings.js_layer) {
            var newTargets = [];
            for(var i=0; i < gt.snap.vector.targets.length; i++) {
                var target = gt.snap.vector.targets[i];
                
                if(target.layer != layer) {
                    newTargets.push(target);
                }
            }
                        
            gt.snap.vector.targets = newTargets;
            gt.snap.edit.targets = newTargets;
            if (layerSettings.select) {
                var defaults = gt.snap.vector.defaults;
                var newTarget = {
                    'edge': defaults.edge,
                    'edgeTolerance': defaults.tolerance,
                    'node': defaults.node,
                    'nodeTolerance': defaults.tolerance,
                    'tolerance': defaults.tolerance,
                    'vertex': defaults.vertex,
                    'vertexTolerance': defaults.tolerance,
                    'layer': layer
                };
                
                if(typeof(layer.features) != 'undefined') {
                    gt.snap.vector.targets.push(newTarget);
                    gt.snap.edit.targets.push(newTarget);
                }
            }
        }
            
        //Label
        layerSettings.label = form.find('input[name="label"]').is(':checked');
                                
        //Edit
        if(form.find('input[name="edit"]').is(':checked')) {
            for(var i = 0; i < this.records.length; i++) {
                this.records[i]['edit'] = false;
            }
            layerSettings.edit = true;
               
            if(gt.control.editgeom.active) {
                gt.control.editgeom.deactivate();
                gt.control.editgeom.activate();
            }
        } else {
            layerSettings.edit = false;
            gt.control.editgeom.deactivate();
        }
            
        //Label size
        layerSettings.label_size = form.find('input[name="label_size"]').val();
            
        //Label color
        layerSettings.label_color = form.find('input[name="label_color"]').val();
            
        //Label fields
        layerSettings.label_fields = [];
        form.find("input[name='label_fields[]']:checked").each(function() {
            layerSettings.label_fields.push($(this).val());
        });
            
        //
        if(! gt.control.offline.isOnline) {
            layerSettings.changed = true;
            
            //
            var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
            var styleRules = gt.map.getStyleRules(mapSLD['layer_' + layerSettings.id]);
            this.setLabel(styleRules, layerSettings);
            layer.styleMap.styles['default'].rules = [];
            layer.styleMap.styles['default'].addRules(styleRules);
            layer.redraw();
        }
        Giscuit.Util.setStorageObject('gtLayers', this.records);
            
        //
        $('#gtPrivatelayersLabelSubmit').button('enable');
               
        //
        $.mobile.changePage($('#gtLayers'));
        
        Giscuit.Util.updateApplicationCache();
    } ,
       
    getLayerRecordById: function(val) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i].id == val) {
                return this.records[i];
            }
        }
        
        return false;
    } ,
    
    getEditLayer: function() {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i]['edit'] === true && this.records[i]['view'] === true) {
                return this.records[i];
            }
        }
        
        return false;
    },
    
    changePosition: function (element, direction) {
        //Avoid hidden layers while in offline
        if (direction == 'up') {
            var prevElement = element.prev();
            while (prevElement.css('display') == 'none') {
                prevElement = prevElement.prev();
            }
            prevElement.before(element);
        } else if (direction == 'down') {
            var nextElement = element.next();
            while (nextElement.css('display') == 'none') {
                nextElement = nextElement.next();
            }
            nextElement.after(element);
        }
        
        //Refresh disabled buttons
        $('.gtLayersOrderUpButton').removeClass('ui-disabled');
        $('.gtLayersOrderDownButton').removeClass('ui-disabled');

        $('.gtLayersOrderUpButton').filter(function (index) {
            return $(this).parent().parent().css('display') != 'none';
        }).first().addClass('ui-disabled');
        
        $('.gtLayersOrderDownButton').filter(function (index) {
            return $(this).parent().parent().css('display') != 'none';
        }).last().addClass('ui-disabled');
    },
    
    setSettings: function(settingsName, layerId, action) {
        //
        if(settingsName != 'view') {
            return false;
        }

        //
        var el = null;
        $('.gtLayersMenuName').each(function(index, value) {
            //
            if($(value).parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent());
            } else if($(value).parent().parent().parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent().parent().parent());
            }

            if($(value).data('layer-id') == layerId) {
                el = parent;
            }
        });
        
        if(! el) {
            return false;
        }
                
        //
        if(this.getLayerRecordById(layerId).view != action) {
            this.nameClick(layerId, el);
        }
                
        return true;
    },
    
    getSettings: function(settingsName, layerId, action) {
        return this.getLayerRecordById(layerId)[settingsName]; 
    },
    
    getLayerIdByClassName: function(val) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i]['class'] == val) {
                return this.records[i].id;
            }
        }
        
        return false;
    },
          
    CLASS_NAME: "Giscuit.Mobile.Control.Privatelayers"
});