/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Privatelayers.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Privatelayers
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Publiclayers = OpenLayers.Class(Giscuit.Control, {
    records : [],
    
    currSettingsLayerId: null,

    initialize: function (records, options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        //Deprecated in 2.12
        //this.EVENT_TYPES.push('changelayer');
        
        debug.info('Initializing layers');
        
        this.records = records;
        
        //Add layers
        this.add();
        
        $(document).ready(Giscuit.Util.bind(function() {
            $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtLayersButton">'
                + '<div class="gtMapControlsElImage" style="background-position: 0 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('layers') + '</div>'
                + '</div></div>');
            
            $("#gtLayersButton").bind(gt.config.clickEvt, function(evt) {
                //Hide map controls
                $("#gtMapControls").css('display', 'none');
                $("#gtToggleControlsButton").css('bottom', '5px').removeClass('ui-btn-active');
                
                $.mobile.changePage($('#gtLayers'));
                
                return false;
            });
            
            $('#gtLayers').live('pagehide', function(evt, ui){
                gt.map.fixGoogleLayers();
            });
            
            //
            var removed = null;
            while(removed != false) {
                removed = false;
                $('.gtLayersMenuContainer').each(function(i, el) {
                    var li = $(el.parentNode);
                    if(li.find('li').length == 0) {
                        li.remove();
                        removed = true;
                    }
                });
            }
            
            var menuNameTaphold = false;
            $('.gtLayersMenuName').bind(gt.config.clickEvt, this, function(evt) {
                if(menuNameTaphold == false) {
                    evt.data.nameClick($(this).data('layer-id'), this.parentNode.parentNode.parentNode);
                    
                    //Select only one base layer at a time
                    if($(this).hasClass('gtLayersMenuBase') == true) {
                        var curr = this;
                        
                        $('.gtLayersMenuBase').each(function(index) {
                            if(this == curr) {
                                return;
                            }
                            
                            var el = this.parentNode.parentNode.parentNode;
                            if($(el).hasClass('ui-btn-active')) {
                                var record = evt.data.getLayerRecordById($(this).data('layer-id'));
                                var layer = gt.layer[record.layerClass];
                                
                                $(el).removeClass('ui-btn-active');
                                layer.setVisibility(false);
                            }
                            
                        });
                    }
                    
                    evt.data.saveVisibleLayers();
                }
                
                //
                menuNameTaphold = false;
                
                return false;
            });
            
            $('.gtLayersMenuName').bind('taphold', this, function(evt) {
                var layer = evt.data.getLayerRecordById($(this).data('layer-id'));

                gt.map.showMetadata(layer);
                
                $('#gtMetadata').live('pagehide', function() {
                    menuNameTaphold = false;
                });
                //
                menuNameTaphold = true;
                
                return false;
            });
        
        }, this));
    } ,
    
    add: function () {
        //Order
        function byOrder(a, b) {
            var x = a.order;
            var y = b.order;

            return ((x < y) ? -1 : ((x > y) ? 1 : 0));
        }
        this.records.sort(byOrder).reverse();
        
        for(var i in this.records) {
            if(! this.records[i].layerClass) {
                continue;
            }
            
            var layerId = this.records[i].id;
            
            //Visibility
            var visibility = false;
            $('.ui-btn-active .gtLayersMenuName').each(function() {
                if($(this).jqmData('layer-id') == layerId) {
                    visibility = true;
                } 
            });
            
            //Javascript
            if(this.records[i].js_layer == true) {
                gt.layer[this.records[i].layerClass] = new Giscuit.Mobile.Layer[this.records[i].layerClass]();
                gt.map.addLayer(gt.layer[this.records[i].layerClass]);
                gt.layer[this.records[i].layerClass].setVisibility(visibility);
                
            //Tms
            } else {
                var extension = this.records[i].format;
                if(extension == 'png24') {
                    extension = 'png';
                }
                
                var layer = new OpenLayers.Layer.TMS(this.records[i].layerClass, gt.config.staticserverUrls, {
                    'order': this.records[i].order ,
                    'layername': this.records[i].layerClass ,
                    'transitionEffect': this.records[i].transitionEffect ,
                    'serviceVersion': '' ,
                    'type': extension ,
                    'visibility': visibility ,
                    'maxscaledenom': parseFloat(this.records[i].maxscaledenom),
                    'maxScale': parseFloat(this.records[i].minscaledenom),
                    'minscaledenom': parseFloat(this.records[i].minscaledenom),
                    'minScale': parseFloat(this.records[i].maxscaledenom),
                    'buffer': 1,
                    'isBaseLayer': false
                });
                
                gt.map.addLayer(layer);
                gt.layer[this.records[i].layerClass] = layer;
            }
        }
        
        //Vector
        gt.map.addVectorLayer();
        
        //Marker
        gt.map.addMarkerLayer();
    },
    
    nameClick: function(id, el) {
        var record = this.getLayerRecordById(id);
        var layer = gt.layer[record.layerClass];
        
        if(layer.visibility) {
            $(el).removeClass('ui-btn-active');
            layer.setVisibility(false);
        } else {
            $(el).addClass('ui-btn-active');
            layer.setVisibility(true);
        }
        
        this.events.triggerEvent('changelayer');
    } ,

    getLayerRecordById: function(id) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i].layerClass && this.records[i].id == id) {
                return this.records[i];
            }
        }
        
        return false;
    } ,
    
    
    saveVisibleLayers: function() {
        var visibleLayers = [];
        for(var i = 0; i < this.records.length; i++) {
                        
            if(this.records[i].layerClass 
                && gt.layer[this.records[i].layerClass]
                && gt.layer[this.records[i].layerClass].visibility) 
                {
                visibleLayers.push(this.records[i].layerClass);
            }
        }
        visibleLayers.join(',');
        
        Giscuit.Util.setCookie('gtLayers', visibleLayers, 1);
    },
    
    setSettings: function(settingsName, layerId, action) {
        //
        if(settingsName != 'view') {
            return false;
        }

        //
        var el = null;
        $('.gtLayersMenuName').each(function(index, value) {
            //
            if($(value).parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent());
            } else if($(value).parent().parent().parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent().parent().parent());
            }

            if($(value).data('layer-id') == layerId) {
                el = parent;
            }
        });
        
        if(! el) {
            return false;
        }
                
        //
        if(el.hasClass('ui-btn-active') != action) {
            this.nameClick(layerId, el);
        }
                
        return true;
    },
    
    getSettings: function(settingsName, layerId) {
        //
        if(settingsName != 'view') {
            return undefined;
        }

        //
        var el = null;
        $('.gtLayersMenuName').each(function(index, value) {
            //
            if($(value).parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent());
            } else if($(value).parent().parent().parent().get(0).tagName.toLowerCase() == 'li') {
                var parent = $($(value).parent().parent().parent());
            }

            if($(value).data('layer-id') == layerId) {
                el = parent;
            }
        });
        
        if(! el) {
            return undefined;
        }
                
        return el.hasClass('ui-btn-active');
    },
    
    getLayerIdByClassName: function(val) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i].layerClass == val) {
                return this.records[i].id;
            }
        }
        
        return false;
    } ,
    
    CLASS_NAME: "Giscuit.Mobile.Control.Privatelayers"

});