/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Map.js 1403 2011-06-12 19:01:12Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Map
 * 
 *  Inherits from:
 *   - <Giscuit.Map>
 */
Giscuit.Mobile.Map = OpenLayers.Class(Giscuit.Map, {

    pointRadius: 15,
    
    snapTolerance: 20,
    
    selectTolerance: 15,
    
    initialize: function(div) {
        debug.info('Initializing map');
        
         //Deprecated in 2.12
        //this.EVENT_TYPES.push('draw');
        
        OpenLayers.IMAGE_RELOAD_ATTEMPTS = 2;
        //OpenLayers.Util.onImageLoadError = function() {
        //};
        
        OpenLayers.ImgPath = gt.config.httpImagePath + "/";
        
        OpenLayers.Projection.addTransform("EPSG:4326", "EPSG:3857", OpenLayers.Layer.SphericalMercator.projectForward);
        OpenLayers.Projection.addTransform("EPSG:3857", "EPSG:4326", OpenLayers.Layer.SphericalMercator.projectInverse);
                
        OpenLayers.Map.prototype.initialize.apply(this, [div, this.getOptions()]);
                
        //
        for(var i=0; i < this.gtEventListeners.length; i++) {
            this.events.on(this.gtEventListeners[i]);
        }
        
        //Permanent link
        this.events.register("moveend", this, function() {
               localStorage.gtMapExtent = this.getExtent().toString();
        });
        
        //
        this.globalSelectControl = new OpenLayers.Control.SelectFeature([]);
        
        //FIXME:
        var globalSelectDummyLayer = new OpenLayers.Layer.Vector("globalSelectDummy");
        this.addLayer(globalSelectDummyLayer);
        this.globalSelectControl.setLayer(
            Giscuit.Util.unique(this.globalSelectControl.layers.concat([globalSelectDummyLayer]))
        );
        
        //
        this.editgeomPath  = gt.config.httpMobilePath + "/editgeom/";
    },
        
    setBounds: function() {
        //Url
        var pathName = window.location.pathname.split('/');
        var lon, lat, zoom, ac = null;
        for(var i=0; i < pathName.length; i++) {
            if(pathName[i] == 'lon') {
                var lon = pathName[i + 1]; 
            } else if(pathName[i] == 'lat') {
                var lat = pathName[i + 1]; 
            } else if(pathName[i] == 'zoom') {
                var zoom = pathName[i + 1]; 
            } else if(pathName[i] == 'ac') {
                var ac = pathName[i + 1]; 
            }
            
        }
        
        if(lon && lat && zoom) {
            var pos = new OpenLayers.LonLat(parseFloat(lon), parseFloat(lat));
            this.setCenter(pos, parseInt(zoom));
            //FIXME:
            this.setCenter(pos, parseInt(zoom));
        } else {
            //Local storage
            var savedExtent = localStorage.gtMapExtent;
        
            if(savedExtent) {
                var extent = OpenLayers.Bounds.fromString(savedExtent);
            } else {
                var extent = this.maxExtent;
            } 
        
            this.zoomToExtent(extent, true);
            //FIXME:
            this.zoomToExtent(extent, true);
        }
        
        if(ac) {
            gt.map.events.register("draw", this, function() {
                $(document).ready(function() {
                    if(gt.control[ac]) {
                        gt.control[ac].activate();
                    }
                });
            });
        }
    },

    addDefaultControls: function() {
        //
        this.addControl(this.globalSelectControl);
        this.globalSelectControl.activate();
        
        //Toggle controls panel
        $("#gtToggleControlsButton").bind(gt.config.safeClickEvt, function(evt) {
            if($(this).hasClass('ui-btn-active')) {
                $(this).removeClass('ui-btn-active');
                $("#gtMapControls").css('display', 'none');
                
                $(this).css('bottom', '5px');
            } else {
                $(this).addClass('ui-btn-active');
                $("#gtMapControls").css('display', 'block');
                
                $(this).css('bottom', $("#gtMapControls").height() + 'px');
            }
            
            return false;
        });
        
        //Zoom
        $("#gtMapPlus").bind(gt.config.clickEvt, this, function(evt) {
            evt.data.zoomIn();
            
            return false;
        });
        $("#gtMapMinus").bind(gt.config.clickEvt, this, function(evt) {
            evt.data.zoomOut();
            
            return false;
        });
        
        //Prevent default
        $("#gtMapControls, #gtMapNavigation, gtMapToggleControls").live('touchmove', function(evt) {
            return false;
        });
       
        //Loading panel
        if(gt.config.guest == 'f') {
            gt.control.loading = new Giscuit.Control.LoadingPanel({'position': 1}); 
            this.addControl(gt.control.loading);
            
            if(! gt.control.offline.isOnline) {
                gt.control.loading.setVisible(false);
            }
        }
    },

    getOptions : function() {
        var navigation = new OpenLayers.Control.TouchNavigation({
            dragPanOptions: {
                enableKinetic: true
            }
        });
        
        return {
            theme: null,
            maxExtent: new OpenLayers.Bounds(gt.config.minx, gt.config.miny, gt.config.maxx, gt.config.maxy),
            numZoomLevels: gt.config.resolutions.length ,
            allOverlays: true,
            units: gt.config.units,
            controls: [navigation],
            resolutions: gt.config.resolutions,
            projection: new OpenLayers.Projection("EPSG:" + gt.config.srid)
        };

    },
    
    showMetadata: function(layer) {
        //Create a html list of data
        var text = '';
        var title = '';
        if (layer.metadata instanceof Array) {
            text += '<ul data-role="listview">';
            for (var i = 0; i < layer.metadata.length; i++) {
                text += '<li><h3><i>' + gt.translate._(layer.metadata[i].ows_title) + ':</i></h3></li>';
                
                for (var key in layer.metadata[i]) {
                    if (layer.metadata[i].hasOwnProperty(key) && key != 'ows_title') {
                        text += '<li>' + gt.translate._(key) + ': ' + layer.metadata[i][key] + '</li>';
                    }
                }
            }
            text += '</ul>';
            title = '<h1> </h1>';
        } else {
            text += '<ul data-role="listview">';
            if (layer.maxscaledenom) {
                text += '<li>' + gt.translate._('maxscaledenom') + ': 1:' + layer.maxscaledenom + '</li>';
            }
            if (layer.minscaledenom) {
                text += '<li>' + gt.translate._('minscaledenom') + ': 1:' + layer.minscaledenom + '</li>';
            }
            
            for (var key in layer.metadata) {
                if (layer.metadata.hasOwnProperty(key) && key != 'ows_title') {
                    text += '<li>' + gt.translate._(key) + ': ' + layer.metadata[key] + '</li>';
                }
            }
            text += '</ul>';
            title = '<h1>' + gt.translate._(layer.metadata.ows_title) + '</h1>';
        }

        var html = 
        '<div data-role="page" id="gtMetadata" data-dom-cache="true" data-url="gtMetadata">' +
            '<div data-role="header">' +
                title +
            '</div>' +
            '<div data-role="content">' + 
                text +
            '</div>' +
        '</div>';
                
        $('#gtMetadata').remove();
        $.mobile.pageContainer.append(html);
        $('#gtMetadata').css('min-height', '100%');
        $.mobile.changePage('#gtMetadata', {'role': 'dialog', 'transition': 'none'});
    },
    
    removeReservedAttributes: function(data) {
        var newData = $.extend({}, data);
        
        if(typeof(newData.hstatus) != 'undefined') {
            delete newData.hstatus;
        }
        
        if(typeof(newData.__hdate) != 'undefined') {
            delete newData.__hdate;
        }
        
        //FIXME: Remove table primary key not gid
        if(typeof(newData.gid) != 'undefined') {
            delete newData.gid;
        }
        
        return newData;
    },
    
    addBboxHandler: function(control, layer, tolerance) {
        if (control.multipleSelect()) {
            debug.warn('Multiple selection is not supported');
            return;
        }
        
        control.handlers.click = new OpenLayers.Handler.Click(
            control,
            {click: function(evt) {
                //Bounds
                var topLeft = gt.map.getLonLatFromViewPortPx({
                    'x': evt.xy.x - tolerance,
                    'y': evt.xy.y - tolerance
                });
                var bottomRight = gt.map.getLonLatFromViewPortPx({
                    'x': evt.xy.x + tolerance,
                    'y': evt.xy.y + tolerance
                });
                
                var bounds = new OpenLayers.Bounds();
                bounds.extend(topLeft);
                bounds.extend(bottomRight);
                
                var boundsGeometry = bounds.toGeometry();
                
                //
                var result = [];
                
                for(var i=0, len = layer.features.length; i<len; ++i) {
                    var feature = layer.features[i];
                    if (! feature.getVisibility()) {
                        continue;
                    }
                                        
                    if (bounds.intersectsBounds(feature.geometry.bounds) &&
                        boundsGeometry.intersects(feature.geometry)) {
                        
                        result.push(feature);
                    }
                }
                
                //
                if(result.length) {
                    control.unselectAll();
                    
                    if(result.length > 1) {
                        var center = boundsGeometry.getCentroid();
                        var shortestDistance = 0, 
                            closestFeature = null, 
                            feature = null;
                        
                        for(var i=0, len = result.length; i<len; ++i) {
                            var feature = result[i];
                            
                            if((! shortestDistance || feature.geometry.distanceTo(center) < shortestDistance) &&
                                ! feature._sketch) {
                                closestFeature = feature;
                            }
                        }
                        control.select(closestFeature);
                    } else {
                        control.select(result[0]);
                    }
                }

            }}, 
            {}
        );
        
        control.events.on({
            "activate": function() {
                control.handlers.feature.deactivate();
                
                control.handlers.click.setMap(control.map);
                control.handlers.click.activate();
            },
            "deactivate": function() {
                control.handlers.click.deactivate();
            }
        });
    },
    
    fixGoogleLayers: function() {
        if(typeof(OpenLayers.Layer.Google) == 'undefined') {
            return;
        } 
        
        for (var obj in gt.layer) {
            if(gt.layer[obj] instanceof OpenLayers.Layer.Google) {
                google.maps.event.trigger(gt.layer[obj].mapObject, "resize");
                gt.layer[obj].redraw();
            }
        }
    },
    
    CLASS_NAME : "Giscuit.Mobile.Map"
});