/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Util.js 2958 2012-08-24 10:16:47Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Namespace: Giscuit.Util
 * 
 */

Giscuit.Util.insertAfter = function (referenceNode, newNode) {
    referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
};

Giscuit.Util.inArray = function(arr, p_val) {
    for(var i = 0, l = arr.length; i < l; i++) {
        if(arr[i] == p_val) {
            return true;
        }
    }
    return false;
};

Giscuit.Util.unique = function (arr) {
    var r = new Array();
    o:for(var i = 0, n = arr.length; i < n; i++)
    {
        for(var x = 0, y = r.length; x < y; x++)
        {
            if(r[x] == arr[i])
            {
                continue o;
            }
        }
        r[r.length] = arr[i];
    }
    return r;
};

Giscuit.Util.isEmpty = function (obj) {
    for(var prop in obj) {
        if(obj.hasOwnProperty(prop))
            return false;
    }
    return true;
};

Giscuit.Util.getHash = function()
{
    var hash = window.location.hash;
    hash = hash.substring(1, hash.length);
    hash = hash.split("&");

    var result = new Object;
    for(var i = 0; i < hash.length; i++) {
        var item = hash[i].split("=");
        
        if(item[0] && item[1]) {
            result[item[0]] = item[1];
        }
    }
    return result;
};

Giscuit.Util.setHash = function(hash)
{
    var queryStr = '';
    for(var obj in hash) {
        queryStr += obj + '=' + hash[obj] + '&';
    }
    
    if(queryStr.charAt(queryStr.length - 1) == '&') {
        queryStr = queryStr.substring(0, queryStr.length - 1);
    }
    
    window.location.hash = queryStr;
    
    return queryStr;
};

Giscuit.Util.bind = function (func, object) {
    var args = Array.prototype.slice.apply(arguments, [2]);
    return function () {
        var newArgs = args.concat(
        Array.prototype.slice.apply(arguments, [0]));
        return func.apply(object, newArgs);
    };
};

Giscuit.Util.parseDate = function(string) {
    if(gt.config.dateFormat == 'iso8601') {
        var regexp = "([0-9]{4})(-([0-9]{2})(-([0-9]{2})" +
        "(T([0-9]{2}):([0-9]{2})(:([0-9]{2})(\.([0-9]+))?)?" +
        "(Z|(([-+])([0-9]{2}):([0-9]{2})))?)?)?)?";
        
        var d = string.match(new RegExp(regexp));
    
        var offset = 0;
        var date = new Date(d[1], 0, 1);

        if (d[3]) { date.setMonth(d[3] - 1); }
        if (d[5]) { date.setDate(d[5]); }
        if (d[7]) { date.setHours(d[7]); }
        if (d[8]) { date.setMinutes(d[8]); }
        if (d[10]) { date.setSeconds(d[10]); }
        if (d[12]) { date.setMilliseconds(Number("0." + d[12]) * 1000); }
        if (d[14]) {
            offset = (Number(d[16]) * 60) + Number(d[17]);
            offset *= ((d[15] == '-') ? 1 : -1);
        }

        offset -= date.getTimezoneOffset();
        
        var time = (Number(date) + (offset * 60 * 1000));
        
        var returnDate = new Date();
        returnDate.setTime(Number(time));
        
        return returnDate;

    } else {
        debug.error('Invalid date format (' + gt.config.dateFormat + ')');
    }
};

Giscuit.Util.callbackFailure = function(o) {
    debug.group("Response failure");
    debug.warn("Transaction id: " + o.tId); 
    debug.warn("HTTP status: " + o.status);
    debug.warn("Status code message: " + o.statusText);
    debug.groupEnd();
    
    if(o.status == 401) {
        Giscuit.Util.message('Access forbidden');
    }
};

Giscuit.Util.parseFailure = function(o, e) {
    debug.group(e.message);
    debug.warn(o.responseText);
    debug.groupEnd();
    
    Giscuit.Util.message("An application error has occurred. Please contact your Administrator.");
};

Giscuit.Util.invalidReplyCode = function (responseText) {
    debug.group("Status code: " + responseText.replyCode);
    debug.warn(responseText);
    debug.groupEnd();
    
    Giscuit.Util.message("An application error has occurred. Please contact your Administrator.");
};

Giscuit.Util.message = function (text) {
    alert(text);
};

Giscuit.Util.jqmMessage = function (text, timeout) {
    if(typeof(timeout) == 'undefined') {
        timeout = 5000;
    }
    
    $.mobile.showPageLoadingMsg("b", text, true);
    
    setTimeout(function() {
        $.mobile.hidePageLoadingMsg();
    }, timeout);
};

Giscuit.Util.websqlFailure = function (tx, evt) {
    Giscuit.Util.jqmMessage(evt.message);
    debug.warn(evt.message);
};

Giscuit.Util.jqmAjaxError = function (e) {
    debug.group(e.statusText);
    debug.warn(e.responseText);
    debug.groupEnd();
    
    $("<div class='ui-loader ui-overlay-shadow ui-body-e ui-corner-all'>"+ e.statusText +"</div>")
        .css({"display": "block", "opacity": 0.96, "top": $(window).scrollTop() + 100})
        .appendTo($.mobile.pageContainer)
        .delay(5000)
        .fadeOut(500, function() {
            $(this).remove();
        });

    $.mobile.hidePageLoadingMsg();
};

Giscuit.Util.getStorageObject = function(key) {
    var jsonFormat = new OpenLayers.Format.JSON();

    if(localStorage.getItem(key)) {
        return jsonFormat.read(localStorage.getItem(key));
    } else {
        return {};
    }
};

Giscuit.Util.setStorageObject = function(key, value) {
    var jsonFormat = new OpenLayers.Format.JSON();
    localStorage.setItem(key, jsonFormat.write(value));
};

Giscuit.Util.removeStorageObject = function(key) {
    localStorage.removeItem(key);
};

Giscuit.Util.setCookie = function (name, value, days) {
    if(gt.config.link == 't' || gt.config.print == 't') {
        return null;
    }
     
    if (days) {
        var date = new Date();
        date.setTime(date.getTime()+(days*24*60*60*1000));
        var expires = "; expires="+date.toGMTString();
    }
    else var expires = "";
    document.cookie = name+"="+value+expires+"; path=/";
};

Giscuit.Util.getCookie = function (name) {
    if(gt.config.link == 't' || gt.config.print == 't') {
        return null;
    }
    
    var nameEQ = name + "=";
    var ca = document.cookie.split(';');
    for(var i=0;i < ca.length;i++) {
        var c = ca[i];
        while (c.charAt(0)==' ') c = c.substring(1,c.length);
        if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
    }
    return null;
};

Giscuit.Util.deleteCookie = function (name) {
    setCookie(name,"",-1);
};

Giscuit.Util.applicationCacheUpdated = false;
Giscuit.Util.updateApplicationCache = function () {
    if(gt.config.offlineAllowed == 'f') {
        return;
    }
    
    Giscuit.Util.applicationCacheUpdated = true;
    
    debug.info('Updating cache');
    try {
        window.applicationCache.update();
    } catch(e) {
        
    }
};

Giscuit.Util.fireEvent = function(element,event) {
    if (document.createEvent) {
       var evt = document.createEvent("HTMLEvents");
       evt.initEvent(event, true, true );
       return !element.dispatchEvent(evt);
       
    } else {
       var evt = document.createEventObject();
       return element.fireEvent('on' + event, evt)
   }
};

Giscuit.Util.debounce = function (func, threshold, execAsap) {
    var timeout;
    return function debounced () {
        var obj = this, args = arguments;
        function delayed () {
            if (!execAsap)
                func.apply(obj, args);
            timeout = null; 
        };
 
        if (timeout)
            clearTimeout(timeout);
        else if (execAsap)
            func.apply(obj, args);
 
        timeout = setTimeout(delayed, threshold || 100); 
    };
};

Giscuit.Util.populateYuiMenu = function (button, values, selectedValue) {
    var menu = button.getMenu();

    menu.clearContent();
    menu.addItems(values);
    menu.render();
    
    //Selected
    var selectedItem = menu.getItem(0);
    
    var items = menu.getItems();
    for(var i=0; i < items.length; i++) {
        if(selectedValue == items[i].value) {
            selectedItem = items[i];
            break;
        } 
    }
            
    menu.activeItem = selectedItem;
    
    if(typeof(menu.activeItem) != 'undefined') {
        button.set("label", menu.activeItem.cfg.getProperty('text'));
    } else {
        button.set("label", '');
    }
};

Giscuit.Util.getScrollBarWidth = function () {
    var inner = document.createElement('p');
    inner.style.width = "100%";
    inner.style.height = "200px";
    
    var outer = document.createElement('div');
    outer.style.position = "absolute";
    outer.style.top = "0px";
    outer.style.left = "0px";
    outer.style.visibility = "hidden";
    outer.style.width = "200px";
    outer.style.height = "150px";
    outer.style.overflow = "hidden";
    outer.appendChild (inner);
    
    document.body.appendChild (outer);
    var w1 = inner.offsetWidth;
    outer.style.overflow = 'scroll';
    var w2 = inner.offsetWidth;
    if (w1 == w2) w2 = outer.clientWidth;
    
    document.body.removeChild (outer);
    
    return (w1 - w2);
};

Giscuit.Util.getMenuItemById = function (button, id) {
    var items = button.getMenu().getItems();

    for(var i=0; i < items.length; i++) {
        if(items[i].id == id) {
            return items[i];
        }
    }

    return false;
};

/*
Giscuit.Util.buttonDown = function (btn, theme) {
	if(! theme) {
		theme = 'c'
	}
	
	var jBtn = $(btn);

	if(jBtn.hasClass('ui-btn-down-' + theme)) {
		return;
	}
	
	jBtn.addClass('ui-btn-down-' + theme);

	setTimeout(function () {
		jBtn.removeClass('ui-btn-down-' + theme);
	}, 300);
};
*/

//Giscuit.Util.mobileDevice, Giscuit.Util.isMobile
(function() {
	var a = navigator.userAgent;
	Giscuit.Util.mobileDevice = null;
	Giscuit.Util.isMobile = /android.+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|meego.+mobile|midp|mmp|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i.test(a) || /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(di|rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0, 4));
	
    if(! Giscuit.Util.isMobile) {
    	return;
    }
	
	if(a.indexOf('Android') != -1) {
		Giscuit.Util.mobileDevice = 'android';
	} else if(a.indexOf('iPad') != -1) {
		Giscuit.Util.mobileDevice = 'ipad';
	} else if(a.indexOf('iPhone') != -1 || a.indexOf('iPod') != -1) {
		Giscuit.Util.mobileDevice = 'iphone';
	}
})();


