<?php

class MenuControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{
    protected $_addParams = array (
        'title_en' => 'MenuTest',
        'tags_en' => 'title',
        'text_en' => 'Content',
        'title_ro' => '',
        'tags_ro' => '',
        'text_ro' => '',
        'title_ru' => '',
        'tags_ru' => '',
        'text_ru' => '',
        'active' => 'f',
        'type' => '2',
        'url' => '',
        'target' => 'f' ,
        'resource' => 'news_menu',
        'privilege' => ''
    );
    
    protected function _retrieveDbEntry ($whereTerm, $field = '')
    {
        $menuTable = new Giscuit_Db_Table_Menu();
        $config = Zend_Registry::get('config');
        
        $select = $menuTable->select();
        $select->setIntegrityCheck(false);
        $select->from($config->database->schema . '.menu', $field);
        $menuTable->joinMultilangField($select, 'title');
        $select->where('text_title.en = ?', $whereTerm);
        return $menuTable->fetchAll($select)->current();
    }
    
    public function testIndexAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/admin/menu');
        $this->assertModule('admin');
        $this->assertController('menu');
        $this->assertAction('index');
        $this->assertResponseCode(200);
    }

    public function testAddAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/admin/menu/add/type/node/parent_id/0');
        $this->assertQuery('form#menuForm');
        
        $languages = Zend_Registry::get('config')->language->toArray();
        foreach ($languages as $key => $language) {
            $this->assertQuery('input#title_' . $key);
        }
    }
    
    public function testAddSubmitAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->request->setMethod('POST')->setPost($this->_addParams);
        $this->dispatch('/' . $this->_language . '/admin/menu/add/parent_id/0/browse/submit');
        $this->assertRedirect();
        
        $result = $this->_retrieveDbEntry($this->_addParams['title_en']);
        $this->assertEquals($result->title, $this->_addParams['title_en']);
    }
    
    public function testEditSubmitAction()
    {
        $this->_testLoginActionSuccess();
        
        $result = $this->_retrieveDbEntry($this->_addParams['title_en'], 'id');
        $editParams = $this->_addParams;
        $editParams['active'] = 't';
        $this->request->setMethod('POST')->setPost($editParams);
        $this->dispatch('/' . $this->_language . '/admin/menu/edit/id/' . $result->id . '/browse/submit');
        $this->assertRedirect();
        
        $result = $this->_retrieveDbEntry($this->_addParams['title_en'], 'active');
        $this->assertEquals($result->active, 't');
    }
    
    public function testBrowseAction()
    {
        $result = $this->_retrieveDbEntry($this->_addParams['title_en'], 'id');
        $this->dispatch('/' . $this->_language . '/default/menu/browse/id/' . $result->id);
        
        $this->assertQueryContentContains('h2', $result->title);
    }
    
    public function testDeleteAction()
    {
        $this->_testLoginActionSuccess();
        
        $result = $this->_retrieveDbEntry($this->_addParams['title_en'], 'id');
        $this->dispatch('/' . $this->_language . '/admin/menu/delete/id/' . $result->id);
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        
        $result = $this->_retrieveDbEntry($this->_addParams['title_en']);
        $this->assertNull($result);
    }

}